C$Procedure MKSPIN ( Make spatial index of plates )
 
      SUBROUTINE MKSPIN ( NP    , VERT,
     .                    VOXSCL, NVOX,   
     .                    VOXSIZ, VOXORI,
     .                    NVXTOT, VXPTR,  
     .                    NVXLST, VXLIST,
     .                    EXTENT )
      IMPLICIT NONE

C$ Abstract
C
C     Index each plate with respect to a voxel grid.
C
C$ Copyright
C
C     Copyright (1999), California Institute of Technology.
C     U.S. Government sponsorship acknowledged.
C
C$ Required_Reading
C
C     None.
C
C$ Keywords
C
C     plate voxel index
C
C$ Declarations
  
      INCLUDE                'pltmax.inc'
      
      INTEGER               NP
      DOUBLE PRECISION      VERT     ( 3, 3, * )
      DOUBLE PRECISION      VOXSCL
      INTEGER               NVOX     ( 3 )
      DOUBLE PRECISION      VOXSIZ
      DOUBLE PRECISION      VOXORI   ( 3 )
      INTEGER               NVXTOT
      INTEGER               VXPTR    ( * )
      INTEGER               NVXLST
      INTEGER               VXLIST   ( * )
      DOUBLE PRECISION      EXTENT   ( 6 )
 
C$ Brief_I/O
C
C     VARIABLE  I/O  DESCRIPTION
C     --------  ---  --------------------------------------------------
C     NP         I   Number of plates.
C     VERT       I   Array of all vertex coordinates.
C     VOXSCL     I   Voxel scaling factor.
C     NVOX       O   Dimensions of voxel grid.
C     VOXSIZ     O   Size of a voxel in model units.
C     VOXORI     O   Origin of voxel grid in model units.
C     NVXTOT     O   Total number of voxels in grid.
C     VXPTR      O   Pointer index to the voxel to plates list.
C     NVXLST     O   Length of voxel to plates list.
C     VXLIST     O   Voxel to plates list.
C
C$ Detailed_Input
C
C     NP         Total number of plates in model.
C
C     VERT       Array containing XYZ coordinates for all vertices
C                in the model reference frame.
C
C     VOXSCL     The voxel size is determined by the
C                average extent of each plate in XYZ.  VOXSCL is a
C                scaling factor that may be used to adjust
C                final voxel size.
C
C$ Detailed_Output
C
C     NVOX       Dimension of the voxel grid in voxel units.
C
C     VOXSIZ     Size of each voxel in model units.
C
C     VOXORI     Origin of voxel grid in model units.
C
C     NVXTOT     Total number of voxels in grid.
C
C     VXPTR      Pointer index to the voxel to plates list.
C                The Nth element of this array contains a
C                pointer to the Mth element of VXLIST, or
C                -1.  If VXPTR(N) is -1, the Nth voxel is empty.
C
C     NVXLST     The total number of voxels in the grid,
C                VVXLST = NVOX(1) * NVOX(2) * NVOX(3).
C
C     VXLIST     The voxel to plates list.  For the Nth voxel,
C                VXPTR(N) points to VXLIST(M).  VXLIST(M) contains
C                the number plates that might be found in voxel N,
C                and the plates IDs are stored in the following
C                elements.
C
C$ Parameters
C
C     None.
C
C$ Files
C
C     None.
C
C$ Exceptions
C
C     NOT Error free.
C
C$ Particulars
C
C     Later.
C
C$ Examples
C
C     Later.
C
C$ Restrictions
C
C     None.
C
C$ Author_and_Institution
C
C     J.A. Bytof      (JPL)
C
C$ Literature_References
C
C     None.
C
C$ Version
C
C-    SPICELIB Version 1.1.0, 12-SEP-2004 (EDW)
C
C       Improve/expand comments and descriptions.
C
C-    SPICELIB Version 1.0.0, 03-FEB-1999 (JAB)
C
C-&
 
C$ Index_Entries
C
C     spatial index plates voxels
C
C-&

      INTEGER               VOX2ID
      INTEGER               VOXID    ( 3 )
 
      INTEGER               PNTRS    ( MAXNPV )
      INTEGER               CELLS    ( 2, MAXNPV )
 
C
C     Local Variables.
C
 
      INTEGER               I
      INTEGER               J
      INTEGER               IX
      INTEGER               IY
      INTEGER               IZ
      INTEGER               NX
      INTEGER               NY
      INTEGER               NZ
      INTEGER               NCELL
      INTEGER               GXMIN
      INTEGER               GXMAX
      INTEGER               GYMIN
      INTEGER               GYMAX
      INTEGER               GZMIN
      INTEGER               GZMAX
      INTEGER               VIXYZ    ( 3 )
 
 
      LOGICAL               INBOX
      LOGICAL               RETURN
 
      DOUBLE PRECISION      XMIN
      DOUBLE PRECISION      XMAX
      DOUBLE PRECISION      YMIN
      DOUBLE PRECISION      YMAX
      DOUBLE PRECISION      ZMIN
      DOUBLE PRECISION      ZMAX
 
 
      DOUBLE PRECISION      AVEXT
 
      DOUBLE PRECISION      BXMIN
      DOUBLE PRECISION      BXMAX
      DOUBLE PRECISION      BYMIN
      DOUBLE PRECISION      BYMAX
      DOUBLE PRECISION      BZMIN
      DOUBLE PRECISION      BZMAX
 
      DOUBLE PRECISION      XVMIN
      DOUBLE PRECISION      XVMAX
      DOUBLE PRECISION      YVMIN
      DOUBLE PRECISION      YVMAX
      DOUBLE PRECISION      ZVMIN
      DOUBLE PRECISION      ZVMAX
 
      DOUBLE PRECISION      VOXPAD
 
      DOUBLE PRECISION      XP       ( 3 )
      DOUBLE PRECISION      YP       ( 3 )
      DOUBLE PRECISION      ZP       ( 3 )

C
C     Required for f2c use on Linux, all local variables
C     to static.
C 
      SAVE

C
C     Standard SPICE error handling.
C
      IF ( RETURN () ) THEN
         RETURN
      ELSE
         CALL CHKIN ( 'MKSPIN' )
      END IF

C
C     Get the average extents of all plates and the
C     overall model extent.
C
 
      AVEXT = 0.D0
 
      XMIN =  9.0D9
      XMAX = -9.0D9
      YMIN =  9.0D9
      YMAX = -9.0D9
      ZMIN =  9.0D9
      ZMAX = -9.0D9
 
      DO I = 1, NP
 
         BXMIN =  9.0D9
         BXMAX = -9.0D9
         BYMIN =  9.0D9
         BYMAX = -9.0D9
         BZMIN =  9.0D9
         BZMAX = -9.0D9
 
         XP(1) = VERT(1,1,I)
         XP(2) = VERT(1,2,I)
         XP(3) = VERT(1,3,I)
 
         YP(1) = VERT(2,1,I)
         YP(2) = VERT(2,2,I)
         YP(3) = VERT(2,3,I)
 
         ZP(1) = VERT(3,1,I)
         ZP(2) = VERT(3,2,I)
         ZP(3) = VERT(3,3,I)
 
         DO J = 1, 3

C
C           Determine plate extents.
C
            BXMIN = DMIN1( BXMIN, XP(J) )
            BXMAX = DMAX1( BXMAX, XP(J) )
 
            BYMIN = DMIN1( BYMIN, YP(J) )
            BYMAX = DMAX1( BYMAX, YP(J) )
 
            BZMIN = DMIN1( BZMIN, ZP(J) )
            BZMAX = DMAX1( BZMAX, ZP(J) )
 
C
C           Determine model extent.
C
            XMIN = DMIN1 ( XMIN, BXMIN )
            XMAX = DMAX1 ( XMAX, BXMAX )
 
            YMIN = DMIN1 ( YMIN, BYMIN )
            YMAX = DMAX1 ( YMAX, BYMAX )
 
            ZMIN = DMIN1 ( ZMIN, BZMIN )
            ZMAX = DMAX1 ( ZMAX, BZMAX )
 
         END DO
 
         EXTENT(1) = XMIN
         EXTENT(2) = XMAX
         EXTENT(3) = YMIN
         EXTENT(4) = YMAX
         EXTENT(5) = ZMIN
         EXTENT(6) = ZMAX
 
C
C        Calculate the cumulative extent of all plates for
C        each degree of freedom.
C
         AVEXT = AVEXT + DABS(BXMAX-BXMIN) +
     .                   DABS(BYMAX-BYMIN) +
     .                   DABS(BZMAX-BZMIN)
 
      END DO

C
C     Calculate the average extent of all plates for
C     and the voxel size, i.e the length of one side
C     of a voxel cube.
C
      AVEXT  = AVEXT / DBLE(3*NP)
      VOXSIZ = VOXSCL * AVEXT

C
C     Determine the minima and maxima of the body centered
C     vertex coordinates expressed in voxel units. Scale the 
C     vertices coord values by VOXSIZ, this scales the 
C     axis in the voxel model space producing cubic voxels
C     with length 1 along each edge in voxel space,
C     VOXSIZ along an edge in model space.
C
      XVMIN = XMIN / VOXSIZ
      YVMIN = YMIN / VOXSIZ 
      ZVMIN = ZMIN / VOXSIZ
      XVMAX = XMAX / VOXSIZ
      YVMAX = YMAX / VOXSIZ
      ZVMAX = ZMAX / VOXSIZ
 
C
C     Extend the voxel grid by 1 voxel length along each 
C     degree of freedom.
C
      XVMIN = VOXPAD ( -1, XVMIN )
      YVMIN = VOXPAD ( -1, YVMIN )
      ZVMIN = VOXPAD ( -1, ZVMIN )
      XVMAX = VOXPAD (  1, XVMAX )
      YVMAX = VOXPAD (  1, YVMAX )
      ZVMAX = VOXPAD (  1, ZVMAX )
 
 
C
C     Calculate voxel grid origin in model units.
C
      VOXORI(1) = XVMIN * VOXSIZ
      VOXORI(2) = YVMIN * VOXSIZ
      VOXORI(3) = ZVMIN * VOXSIZ
 
C
C     Calculate the dimension of the voxel grid in
C     units of voxels.
C
      NX = NINT( XVMAX - XVMIN )
      NY = NINT( YVMAX - YVMIN )
      NZ = NINT( ZVMAX - ZVMIN )
 
      NVOX(1) = NX
      NVOX(2) = NY
      NVOX(3) = NZ
 
      NVXTOT = NX*NY*NZ

C
C     Enumerate all voxels that each plate might intersect.
C
      CALL INILNK ( MAXNPV, MAXNPV, NCELL, PNTRS, CELLS )

      DO I = 1, NP

         GXMIN =  MAXPLT + 1
         GXMAX =  -1
         GYMIN =  MAXPLT + 1
         GYMAX =  -1
         GZMIN =  MAXPLT + 1
         GZMAX =  -1

C
C        Loop over the plate's three vertices.
C
 
         DO J = 1, 3

C
C           Calculate the voxel coordinates for the voxels 
C           containing the body fixed vertex coordinates.
C
            CALL GETVOX ( VOXSIZ     , VOXORI, NVOX, 
     .                    VERT(1,J,I), INBOX,  VOXID )
 
            IF ( INBOX ) THEN
 
               GXMIN = MIN0( GXMIN, VOXID(1) )
               GXMAX = MAX0( GXMAX, VOXID(1) )
               GYMIN = MIN0( GYMIN, VOXID(2) )
               GYMAX = MAX0( GYMAX, VOXID(2) )
               GZMIN = MIN0( GZMIN, VOXID(3) )
               GZMAX = MAX0( GZMAX, VOXID(3) )
 
            ELSE

C
C              A plate vertex lies outside the voxel grid.
C              This should never signal. To do so indicates a
C              serious bug.
C
               CALL SETMSG ( 'BUG: Plate outside of voxel '//
     .                       'grid. plate ID = #.'          )
               CALL ERRINT ( '#', I                         )
               CALL SIGERR ( 'SPICE(PLATEOUTSIDEGRID)'      )
               CALL CHKOUT ( 'MKSPIN'                       )
               RETURN

            END IF
 
         END DO

C
C        Determine the other voxels, if any, that the plate
C        might intersect, based on the voxels that the vertices
C        of the plate lie in.
C
         DO IZ = GZMIN, GZMAX

             DO IY = GYMIN, GYMAX

                 DO IX = GXMIN, GXMAX

                    VIXYZ(1) = IX
                    VIXYZ(2) = IY
                    VIXYZ(3) = IZ
                    CALL ADDLNK ( VOX2ID(VIXYZ,NVOX), I, MAXNPV, 
     .                            PNTRS, NCELL, CELLS )
                 END DO

              END DO

          END DO

      END DO

C
C     Generate two linked lists mapping voxel ID to the plates enclosed
C     within that voxel (if any).
C
C     VXPTR : An array, indexed by voxel ID. For an array element, 
C             VXPTR(VOX_ID), greater than zero, the value identifies an 
C             index in VXLIST, the value of that VXLIST array elemement 
C             equaling the number of plates contained in the voxel 
C             specified by the ID. The condition VXPTR(VOX_ID) = -1 
C             indicates the voxel contains no plates.
C
C     VXLIST: An array, indexed by the positive entries in VXPTR. The 
C             element, N, identified by a VXPTR value describes the 
C             number of plates contained in the corresponding voxel. 
C
C                 N = VXLIST( VXPTR(VOX_ID) )
C
C             The N elements following VXLIST( VXPTR(VOX_ID) ),
C             contain the IDs of those plates within the voxel.
C
      CALL UNTNGL ( NVXTOT, PNTRS , MAXNPV, CELLS, 
     .              VXPTR , NVXLST, VXLIST )

C
C     Checkout and return.
C
      CALL CHKOUT ( 'MKSPIN' )

      RETURN
 
      END
