C$Procedure   RDFFPL ( read triangular plate model flatfile )
 
      SUBROUTINE RDFFPL ( INFILE, PLTTYP, NV, VID, X,  Y,  Z,
     .                                    NP, PID, V1, V2, V3 )
      IMPLICIT NONE
 
C$ Abstract
C
C     Read a triangular plate model's flatfile's data file.
C
C$ Copyright
C
C     Copyright (1997), California Institute of Technology.
C     U.S. Government sponsorship acknowledged.
C
C$ Required_Reading
C
C     MKPLAT User's Guide
C     SURFACE
C
C$ Keywords
C
C     PLATE
C     FLATFILE
C
C$ Declarations
  
      INCLUDE               'pltmax.inc'

      CHARACTER*(*)         INFILE
      INTEGER               PLTTYP 
      INTEGER               NV
      INTEGER               VID   ( * )
      DOUBLE PRECISION      X     ( * )
      DOUBLE PRECISION      Y     ( * )
      DOUBLE PRECISION      Z     ( * )
      INTEGER               NP
      INTEGER               PID   ( * )
      INTEGER               V1    ( * )
      INTEGER               V2    ( * )
      INTEGER               V3    ( * )

C$ Brief_I/O
C
C     VARIABLE  I/O  DESCRIPTION
C     --------  ---  --------------------------------------------------
C     INFILE     I   Name of plate data file.
C     PLTTYP     I   Integer code identifying type of data file.
C     NV         O   Number of vertices in model.
C     VID        O   NV-vector of vertex IDs.
C     X          O   NV-vector of vertices' X-coordinates.
C     Y          O   NV-vector of vertices' Y-coordinates.
C     Z          O   NV-vector of vertices' Z-coordinates.
C     NP         O   Number of plates in model.
C     PID        O   NP-vector of plate IDs.
C     V1         O   NP-vector of plates' first vertex index.
C     V2         O   NP-vector of plates' second vertex index.
C     V3         O   NP-vector of plates' third vertex index.
C
C
C$ Detailed_Input
C
C     INFILE     is the full pathname of the plate model flatfile.
C
C     PLTTYP     the integer code identifying the type of data
C                to read from INFILE. Two values are allowed:
C
C                   '1'   A standard plate-vertex data file
C                         containing vertex coordinates and
C                         the plate-vertex mappings.
C
C                   '2'   A Gaskel shape data file containing
C                         plate ordered vertex data.
C
C                   '3'   A vertex-facet table data file
C                         containing vertex coordinates and
C                         the facet listing.
C
C$ Detailed_Output
C
C     NV         is the number of vertices in the plate model.
C
C     VID        is an NV-vector of vertex ID's.
C
C     X          is the array of dp X-coordinates of the NV vertices
C                given in a body-fixed frame of reference.
C
C     Y          is the array of dp Y-coordinates of the NV vertices
C                given in a body-fixed frame of reference.
C
C     Z          is the array of dp Z-coordinates of the NV vertices
C                given in a body-fixed frame of reference.
C
C     NP         is the number of triangular plates in the model.
C
C     PID        is the array of NP plate ID's.
C
C     V1         is the NP-array of first vertex integer ID's.
C
C     V2         is the NP-array of second vertex integer ID's.
C
C     V3         is the NP-array of third vertex integer ID's.
C
C$ Parameters
C
C     None.
C
C$ Files
C
C     None.
C
C$ Exceptions
C
C     1) If the number of vertices (NV) exceeds the value of MAXVRT,
C        SPICE(TOOMANYVERTICES) signals.
C
C     2) If the number of plates (NP) exceeds the value of MAXPLT,
C        SPICE(TOOMANYPLATES) signals.
C
C     3) If the NPARSD routine fails while parsing a double string, 
C        SPICE(BADDOUBLEPRECISION) signals.
C
C     4) If the NPARSI routine fails while parsing an integer string, 
C        SPICE(BADINTEGER) signals.
C
C     5) If a data line lacks the expected format (data type 3),
C        SPICE(BADDATALINE) signals.
C
C     6) If the PLTTYP variable has an uncoded value,
C        SPICE(BADDATATYPE) signals
C
C$ Particulars
C
C     Input File Format
C     -----------------
C
C     INFILE is assumed to be structured as follows (values
C            are space delimited within each input record).
C
C     -Plate/vertex data file (PLATE_TYPE 1):
C
C        Line  1: NV
C        NV = the number of vertices (INTEGER)
C
C        Lines 2 to (NV+1): VID X Y Z
C        VID         = the vertex ID (INTEGER)
C        X, Y, and Z = the vertex coordinates (DOUBLE PRECISION)
C
C        Line  (NV+2): NP
C        NP = number of triangular plates (INTEGER)
C
C        Lines (NV+3) to (NV+3+NP+1): PID V1 V2 V3
C        PID            = the plate ID (INTEGER)
C        V1, V2, and V3 = the plate vertex ID's (INTEGER)
C
C     -Shape data file (PLATE_TYPE 2):
C
C        Line 1: Q
C        Q = number of vertex coordinates per side of the 
C            cube face - not counting the origin. A
C            face has (Q+1)^2 vertex coordinates.
C
C        Lines 2 to (Q+1): X Y Z
C        X, Y, and Z = the vertex coordinates (DOUBLE PRECISION)
C
C     -Vertex/Facet tabels data file (PLATE_TYPE 3):
C
C        NV = the number of vertices (INTEGER)
C
C        Lines 1 to NV: 'V' X Y Z
C        'V'         = character flag indicating vertex data
C        X, Y, and Z = the vertex coordinates (DOUBLE PRECISION)
C
C        NP = number of triangular plates (INTEGER)
C
C        Lines (NV+1) to (NV+1+NP): 'F' V1 V2 V3
C        'F'            = character flag indicating facet 
C                         (plate-vertex) data
C        V1, V2, and V3 = the plate vertex ID's (INTEGER)
C
C$ Examples
C
C  C     The following include file defines MAXVRT and
C  C     MAXPLT, the maximum number of vertices and plates
C  C     used in the plate model software.
C
C        INCLUDE               'pltmax.inc'
C
C        INTEGER               NV
C        INTEGER               NP
C        INTEGER               PID     ( MAXPLT )
C        INTEGER               VID     ( MAXVRT )
C        INTEGER               V1      ( MAXVRT )
C        INTEGER               V2      ( MAXVRT )
C        INTEGER               V3      ( MAXVRT )
C        INTEGER               PLTTYP
C
C        DOUBLE PRECISION      X       ( MAXVRT )
C        DOUBLE PRECISION      Y       ( MAXVRT )
C        DOUBLE PRECISION      Z       ( MAXVRT )
C
C        CHARACTER*(80)        INFILE
C
C        CALL PROMPT ( 'Enter name of plate data file : ', INFILE )
C        CALL PROMPT ( 'Enter type of plate data      : ', PLTTYP )
C
C        CALL RDFFPL ( INFILE,  PLTTYP, NV, VID, X,  Y,  Z,
C       .                               NP, PID, V1, V2, V3 )
C
C$ Restrictions
C
C     It is the user's responsibility to properly dimension
C     arrays in the calling routine large enough to accept data
C     from the given file.
C
C$ Author_and_Institution
C
C     J.A. Bytof      (JPL)
C
C$ Literature_References
C
C     None.
C
C$ Version
C
C-    SPICELIB Version 3.0.0, 25-OCT-2004 (EDW)
C
C        Added capability to process Bob Gaskell shape files.
C        Added MAXQ parameter to 'pltmax.inc'.
C
C-    SPICELIB Version 2.0.0, 22-OCT-1998 (JAB)
C
C-    SPICELIB Version 1.0.0, 09-APR-1997 (JAB)
C
C-&
 
C$ Index_Entries
C
C     read triangular plate model flatfile
C
C-&
 
C
C     SPICELIB functions
C
 
      LOGICAL               RETURN
 
C
C     Local variables
C
 
      LOGICAL               EOF

      INTEGER               PTR 
      INTEGER               I
      INTEGER               J
      INTEGER               ND
      INTEGER               NI
      INTEGER               NC
      INTEGER               NREC
      INTEGER               FACE
      INTEGER               Q
      INTEGER               N0

      INTEGER               ARI     ( 4 )
      INTEGER               N       ( 0:MAXQ, 0:MAXQ, 6)

      CHARACTER*(160)       ERROR
      CHARACTER*(10)        FORMAT
      CHARACTER*(20)        ARC     ( 6 )
 
      DOUBLE PRECISION      ARD     ( 3 )
      DOUBLE PRECISION      W1      ( 3 )
      DOUBLE PRECISION      W2      ( 3 )
      DOUBLE PRECISION      VEC     ( 3, 0:MAXQ, 0:MAXQ, 6) 

C
C     SPICE functions
C
      DOUBLE PRECISION     VNORM
      LOGICAL              EQSTR

C
C     Standard SPICE error handling.
C
 
      IF ( RETURN () ) THEN
         RETURN
      ELSE
         CALL CHKIN ( 'RDFFPL' )
      END IF

C
C     Initialize record counter.
C
      NREC = 1

C
C     Read the data file corresponding to the PLTTYP ID.
C
      IF( PLTTYP .EQ. 1 ) THEN

C
C        Plate data type 1.
C
C        First line, a single integer, number of vertices.
C
         FORMAT = 'I'
  
         CALL RDFFDI ( INFILE, NREC, FORMAT, ND, NI, NC,
     .                 ARD, ARI, ARC, EOF  )

         NV = ARI(1)

C
C        Check the routine can handle the vertices.
C
         IF ( NV .GT. MAXVRT ) THEN
            CALL SETMSG ( 'Number of vertices exceeds limit.' )
            CALL SIGERR ( 'SPICE(TOOMANYVERTICES)'            )
         END IF

C
C        Read data: integer double double double.
C
         FORMAT = 'I D D D'

         DO I = 1, NV
      
            NREC   = NREC + 1
         
            CALL RDFFDI ( INFILE, NREC, FORMAT, ND, NI, NC,
     .                    ARD, ARI, ARC, EOF  )

            IF ( EOF ) THEN
               CALL EXTMSI ( 'End of file after line #.', '#', NREC )
            END IF

            X(I)   = ARD(1)
            Y(I)   = ARD(2)
            Z(I)   = ARD(3)
            VID(I) = ARI(1)
   
         END DO

C
C        Plate data type 1 reads the plat-vertex information from
C        the plate file. 
C
C        Read number of plates.
C
         NREC   = NREC + 1
         FORMAT = 'I'
 
         CALL RDFFDI ( INFILE, NREC, FORMAT, ND, NI, NC,
     .                 ARD, ARI, ARC, EOF  )
         IF ( EOF ) THEN
            CALL EXTMSI ( 'End of file after line #.', '#', NREC )
         END IF
 
C
C        Check we can process 'NP' plates.
C
         NP = ARI(1)
         IF ( NP .GT. MAXPLT ) THEN
            CALL SETMSG ( 'Number of plates exceeds limit.'          )
            CALL SIGERR ( 'SPICE(TOOMANYPLATES)'                     )
         END IF
 
C
C        Read each plate's ID and corresponding vertex set defining
C        the plate.
C
         FORMAT = 'I I I I'
 
         DO I = 1, NP
            
            NREC   = NREC + 1
            
            CALL RDFFDI ( INFILE, NREC, FORMAT, ND, NI, NC,
     .                    ARD, ARI, ARC, EOF  )
            IF ( EOF ) THEN
               CALL EXTMSI ( 'End of file after line #.', '#', NREC )
            END IF

            PID(I) = ARI(1)
            V1(I)  = ARI(2)
            V2(I)  = ARI(3)
            V3(I)  = ARI(4)

         END DO

      ELSE IF( PLTTYP .EQ. 2 ) THEN

C
C        Plate data type 2.
C
C        Read a Gaskell shape file. Shape files contain only vertex
C        data, the plate-vertex mappings implicitly known from the 
C        ordering of the data. The Gaskell model uses quadralaterals
C        plates, a plate defined by the vector set:
C
C        [ VEC( *, I, J  , FACE), VEC( *, I+1, J  , FACE), 
C          VEC( *, I, J+1, FACE), VEC( *, I+1, J+1, FACE) ]
C
C        The file lists the vertex data in terms cube faces. Six
C        faces, each with Q+1 x Q+1 vertices.
C
         FORMAT = 'I'
  
         CALL RDFFDI ( INFILE, NREC, FORMAT, ND, NI, NC,
     .                 ARD, ARI, ARC, EOF  )

         Q  = ARI(1)
         NV = 6*( (Q+1)**2)

C
C        Does the read Q exceed the maximum value.
C
         IF ( Q .GT. MAXQ ) THEN
            CALL SETMSG ( 'Shape parameter Q = #, '
     .               //   'exceeds maximum value #.'  )
            CALL ERRINT ( '#', Q                      )
            CALL ERRINT ( '#', MAXQ                   )
            CALL SIGERR ( 'SPICE(TOOMANYPLATES)'      )
         END IF


C
C        Check the routine can handle the vertices.
C
         IF ( NV .GT. MAXVRT ) THEN
            CALL SETMSG ( 'Number of vertices exceeds limit.' )
            CALL SIGERR ( 'SPICE(TOOMANYVERTICES)'            )
         END IF

C
C        Read data: double double double
C
         FORMAT = 'D D D'

         N0 = 0

         DO FACE = 1, 6

            DO J = 0, Q

               DO I = 0, Q

                  NREC   = NREC + 1

C
C                 Read the vertex data. The 3-Vector
C                    _                 _
C                   |  VEC(1,I,J,FACE)  |
C                   |  VEC(2,I,J,FACE)  |
C                   |_ VEC(3,I,J,FACE) _|
C
C                 contains the vertex coordinates.
C
                  CALL RDFFDI ( INFILE         , NREC, FORMAT, 
     .                          ND             , NI  , NC    , 
     .                          VEC(1,I,J,FACE), ARI , ARC   ,
     .                          EOF )

                  IF ( EOF ) THEN
                     CALL EXTMSI ( 'End of file after line #.', 
     .                             '#', NREC)
                  END IF

C
C                Store the vertex ID to the 'N' array using the same
C                indexing as 'VEC'.
C
                 N0          = N0+1
                 N(I,J,FACE) = N0

C
C                Copy the vertex coordinates to the corresponding
C                plate model arrays. This operation is somewhat
C                repetative, but clarity is most important.
C
                 X(N0)    = VEC(1,I,J,FACE)
                 Y(N0)    = VEC(2,I,J,FACE)
                 Z(N0)    = VEC(3,I,J,FACE)
                 VID(N0)  = N0

               END DO

            END DO

         END DO

C
C        Plate data type 2 calculates the plate-vertex mappings 
C        from the ordering of the vertex data.

C
C        Number of plates based on the 'Q' value.
C
         NP = 12*(Q**2)

C
C        Check we can process 'NP' plates.
C
         IF ( NP .GT. MAXPLT ) THEN
            CALL SETMSG ( 'Number of plates exceeds limit.' )
            CALL SIGERR ( 'SPICE(TOOMANYPLATES)'            )
         END IF

C
C        Expand a Gaskell shape model vertex ordering to
C        form a plate-vertex map.
C

C
C        Cube edges. Associate later IDs with
C        the first occurance.
C
         DO I=1,Q-1

            N(I,Q,6) = N(Q-I, Q  , 4)
            N(I,0,6) = N(I  , Q  , 2)
            N(I,0,5) = N(Q  , Q-I, 1)
            N(I,0,4) = N(Q-I, 0  , 1)
            N(I,0,3) = N(0  , I  , 1)
            N(I,0,2) = N(I  , Q  , 1)

         END DO

         DO J=1,Q-1

            N(Q,J,6) = N(J  , Q, 5)
            N(Q,J,5) = N(0  , J, 4)
            N(Q,J,4) = N(0  , J, 3)
            N(Q,J,3) = N(0  , J, 2)
            N(0,J,6) = N(Q-J, Q, 3)
            N(0,J,5) = N(Q  , J, 2)

         END DO

C
C        Cube corners. Associate later IDs with
C        the first occurance.
C
         N(0,0,3) = N(0,0,1)
         N(Q,0,4) = N(0,0,1)
         N(0,0,2) = N(0,Q,1)
         N(Q,0,3) = N(0,Q,1)
         N(0,0,4) = N(Q,0,1)
         N(Q,0,5) = N(Q,0,1)
         N(0,0,5) = N(Q,Q,1)
         N(Q,0,2) = N(Q,Q,1)
         N(0,0,6) = N(0,Q,2)
         N(Q,Q,3) = N(0,Q,2)
         N(0,Q,5) = N(Q,Q,2)
         N(Q,0,6) = N(Q,Q,2)
         N(Q,Q,4) = N(0,Q,3)
         N(0,Q,6) = N(0,Q,3)
         N(Q,Q,5) = N(0,Q,4)
         N(Q,Q,6) = N(0,Q,4)

         N0 = 0

         DO FACE = 1, 6 

            DO I = 0, Q-1

               DO J = 0, Q-1

C
C                 As mentioned, the Gaskell shape model uses 
C                 quadralteral plates as opposed to triangular used by 
C                 the NAIF plate system. We can reduce a quadralateral 
C                 into two triangles by adding a vector connecting 
C                 opposite facing vertices creating two triangles 
C                 (1 and 2):
C
C                               V1 
C                               /\
C                              / 1\
C                          V4 /____\ V2
C                             \  2 /
C                              \  /
C                               \/
C                               V3
C
C                 or
C                                V1
C                               /|\
C                              / | \
C                         V4  /  |2 \ V2
C                             \ 1|  /
C                              \ | /
C                               \|/
C                                V3
C
C                 We chose the connecting vector to minimize the cross
C                 product magnitude of the connected vertex vectors.
C
C                    connection = MIN( V1 x V3, V2 X V4)
C

C
C                 Connection 1.
C
                  CALL VCRSS( VEC(1,I  ,J  ,FACE), 
     .                        VEC(1,I+1,J+1,FACE), W1)

C
C                 Connection 2.
C
                  CALL VCRSS( VEC(1,I+1,J  ,FACE), 
     .                        VEC(1,I  ,J+1,FACE), W2)

C
C                 Calculate the magnitudes of the cross products,
C                 branch based on the minimum value.
C
C                 Fill two entries of the plate-vertex mapping 
C                 from the divided quadralateral.
C
                  IF( VNORM(W1) .LE. VNORM(W2) ) THEN

C
C                    Apply connection 1.
C
                     N0      = N0+1
                     PID(N0) = N0
                     V1(N0)  = N(I  , J  , FACE)
                     V2(N0)  = N(I+1, J+1, FACE)
                     V3(N0)  = N(I+1, J  , FACE)

                     N0      = N0+1
                     PID(N0) = N0
                     V1(N0)  = N(I  , J  , FACE)
                     V2(N0)  = N(I  , J+1, FACE)
                     V3(N0)  = N(I+1, J+1, FACE)

                  ELSE

C
C                    Apply connection 2.
C
                     N0      = N0+1
                     PID(N0) = N0
                     V1(N0)  = N(I  , J  , FACE)
                     V2(N0)  = N(I  , J+1, FACE)
                     V3(N0)  = N(I+1, J  , FACE)

                     N0      = N0+1
                     PID(N0) = N0
                     V1(N0)  = N(I+1, J  , FACE)
                     V2(N0)  = N(I  , J+1, FACE)
                     V3(N0)  = N(I+1, J+1, FACE)
 
                  END IF

               END DO

            END DO

         END DO

      ELSE IF( PLTTYP .EQ. 3 ) THEN

C
C        Plate data type 3.
C
C        Read data. The data format depends on the
C        prefix marker either 'v' (vertex) or 'f' (facet).
C        Since we don't know the prefix a priori,
C        extract the four data elements, then parse.
C
         NP     = 0
         NV     = 0
         FORMAT = 'C C C C'
         EOF    = .FALSE.
   
         DO WHILE ( .NOT. EOF )

            CALL RDFFDI ( INFILE, NREC, FORMAT, ND, NI, NC,
     .                    ARD, ARI, ARC, EOF  )
     
            IF ( .NOT. EOF ) THEN

C
C              Branch based on the type of data, 'v' or 'f'.
C
               IF(       EQSTR( ARC(1), 'v') ) THEN

C
C                 Vertex data. Process a line of three doubles.
C                 The algorithm includes an implicit assumption that
C                 the data order defines the vertex IDs.  If not,
C                 the output plate files will be useless.
C
                  NV = NV + 1

                  VID(NV) = NV

                  DO J =2, 4

                     CALL NPARSD ( ARC(J), ARD(J-1), ERROR, PTR )
                  
                     IF ( PTR .NE. 0 ) THEN
                        CALL SETMSG ( 'D.P. error (#) in line #.' )
                        CALL ERRCH  ( '#', ERROR                  )
                        CALL ERRINT ( '#', NREC                   )
                        CALL SIGERR ( 'SPICE(BADDOUBLEPRECISION)' )
                     END IF

                  END DO

                 X(NV) = ARD(1)
                 Y(NV) = ARD(2)
                 Z(NV) = ARD(3)
   
               ELSE IF ( EQSTR( ARC(1), 'f') ) THEN

C
C                 Plate-vertex data. Process a line of three integers.
C                 The algorithm includes an implicit assumption that
C                 the data order defines the plate IDs. If not,
C                 the output plate files will be useless.
C
                  NP      = NP + 1
                  PID(NP) = NP

                  DO J =2, 4

                     CALL NPARSI ( ARC(J), ARI(J-1), ERROR, PTR )
                  
                     IF ( PTR .NE. 0 ) THEN
                        CALL SETMSG ( 'Integer error (#) in line #.' )
                        CALL ERRCH  ( '#', ERROR                     )
                        CALL ERRINT ( '#', NREC                      )
                        CALL SIGERR ( 'SPICE(BADINTEGER)'            )
                     END IF

                  END DO

                  V1(NP) = ARI(1)
                  V2(NP) = ARI(2)
                  V3(NP) = ARI(3)

               ELSE

C
C                 This block executes if the first characters of 
C                 a type 3 data lacks a 'v' or 'f'.
C
                  CALL SETMSG ( 'Bad data line at record #.' )
                  CALL ERRINT ( '#', NREC                    )
                  CALL SIGERR ( 'SPICE(BADDATALINE)'         )

               END IF

               NREC = NREC + 1

            END IF

         END DO

C
C        All type 3 data read. We should have a non zero NP and NV,
C        if not, something failed.
C
         IF ( (NP .EQ. 0) .OR. (NV .EQ. 0) ) THEN
            CALL SETMSG ( 'Read error, type 3 data file.'
     .                 // ' Num plates = #1, num verts = #2.'
     .                 // 'Both should be non-zero'         )
            CALL ERRINT ( '#1', NP                          )
            CALL ERRINT ( '#2', NV                          )
            CALL SIGERR ( 'SPICE(DATAREADFAILED)'           )
         END IF


      ELSE

         CALL SETMSG ( 'Unkown plate data type: #.' )
         CALL ERRINT( '#', PLTTYP                   )
         CALL SIGERR ( 'SPICE(BADDATATYPE)'         )

      END IF

      CALL TOSTDO ( '...Done reading plate model flatfile.' )
      CALL TOSTDO ( ' ' )
 
      CALL CHKOUT ( 'RDFFPL' )
 
      RETURN
      END

C12345678901234567890123456789012345678901234567890123456789012345678912


