C$Procedure TRAZEL ( maximum possible apparent elevation of a plate )
 
      SUBROUTINE TRAZEL ( AVERT, ANORM, BVERT, BVID, ABOVE, MAXELE )
 
C$ Abstract
C
C     Given two plates, determine the maximum elevation
C     from the plane defined by the first plate, plate A,
C     to the second plate (plate B).
C
C$ Copyright
C
C     Copyright (1999), California Institute of Technology.
C     U.S. Government sponsorship acknowledged.
C
C$ Required_Reading
C
C     None.
C
C$ Keywords
C
C     PLATE
C
C$ Declarations
 
      IMPLICIT              NONE

      DOUBLE PRECISION      AVERT     ( 3, 3 )
      DOUBLE PRECISION      ANORM     ( 3 )
      DOUBLE PRECISION      BVERT     ( 3, 3 )
      INTEGER               BVID      ( 3 ) 
      LOGICAL               ABOVE     ( 3 )
      DOUBLE PRECISION      MAXELE
 
C$ Brief_I/O
C
C     VARIABLE  I/O  DESCRIPTION
C     --------  ---  --------------------------------------------------
C     AVERT      I   Coordinates of plate-A's vertices.
C     ANORM      I   Components of plate-A's normal vector.
C     BVERT      I   Coordinates of plate-B's vertices.
C     ABOVE      I   .TRUE. if a plate B vertex is visible from
C                    plate A, .FALSE. if not.
C     MAXELE     O   Maximum elevation of plate-B as observed
C                    from plate-A.
C
C$ Detailed_Input
C
C     AVERT      the spatial coordinates of the plate A vertices.
C
C     ANORM      the A plate unit normal vector.
C
C     BVERT      the spatial coordinates of the plate B vertices.
C
C     ABOVE      the array of booleans corresponding to the plate
C                B vertices. An array element has value .TRUE. if 
C                the corresponding plate B vertex is visible from
C                plate A, .FALSE. otherwise.
C
C$ Detailed_Output
C
C     MAXELE     the approximate maximum elevation of plate B as
C                observed from plate A.
C
C$ Parameters
C
C     None.
C
C$ Files
C
C     None.
C
C$ Exceptions
C
C     Error free.
C
C$ Particulars
C
C     Thie routine calculates the approximate maximum elevation of 
C     the vertices of one plate vs. another. In this context "maximum 
C     elevation" means the maximum of the set of elevation values
C     calculated from each vertex of plate B to plate A.
C
C$ Examples
C
C     Later
C
C$ Restrictions
C
C     None.
C
C$ Author_and_Institution
C
C     J.A. Bytof      (JPL)
C
C$ Literature_References
C
C     None.
C
C$ Version
C
C-    SPICELIB Version 1.1.0, 27-JUL-2004 (EDW)
C
C        Extension abnd polish to comments. Code modified
C        with CHKTST call to improve run-time performance
C        by preventing repeating a pervious calculation on
C        a vertex.
C
C-    SPICELIB Version 1.0.0, 22-MAR-1999 (JAB)
C
C
C-&
 
C$ Index_Entries
C
C     plate
C
C-&
 
      INTEGER               I
      INTEGER               J
      INTEGER               K
 
      DOUBLE PRECISION      ELEV
      DOUBLE PRECISION      CLEN
      DOUBLE PRECISION      C       ( 3 ) 

      LOGICAL               GO
 
      MAXELE = -99999.D0
 
C
C     Loop over plate-B vertices to find the
C     line of sight that results in the maximum observed elevation
C     as seen from plate-A.
C
 
      DO I = 1, 3
 
C
C        Before executing this loop cycle, determine if the
C        program previously calculated the elevation of this
C        vertex. No need to repeat.
C
         CALL CHKTST ( BVID(I), GO )

         IF ( ABOVE(I) .AND. GO ) THEN
 
C
C          BVERT(1,I) is a visible vertex of plate-B.
C          Loop over the three sides of plate-A.
C
            DO J = 1, 3
 
C
C              Generate a baseline segment of plate-A.
C
               K = 1 + MOD( J, 3 )
 
C
C             What point on the line segment defined by
C             the points AVERT(1,J) and AVERT(1,K) is 
C             closest to BVERT(1,I), a visible vertex from 
C             the line segment?
C
               CALL PINSEG ( AVERT(1,J),
     .                       AVERT(1,K),
     .                       BVERT(1,I), C, CLEN )
 
 
C
C              Where does the closest point lie with respect to
C              the line segment
C
C                       0                    1
C               --------o--------------------o---------
C                     AVERT(1,J)           AVERT(1,K)
C 
               IF ( CLEN .LE. 0.D0 ) THEN

C
C                 Closest point is "left" and outside the 
C                 line segment.
C
C                 Logic Error. This branch does not consider
C                 the possibility of paralell, overlapping plates.
C
                  MAXELE = DMAX1( MAXELE, ELEV( ANORM,
     .                                          AVERT(1,J),
     .                                          BVERT(1,I) ) )
 
               ELSE IF ( CLEN .GE. 1.D0 ) THEN
 
C
C                 Closest point is "right" and outside 
C                 the line segment.
C
C                 Logic Error. This branch does not consider
C                 the possibility of paralell, overlapping plates.
C
                  MAXELE = DMAX1( MAXELE, ELEV( ANORM,
     .                                          AVERT(1,K),
     .                                          BVERT(1,I) ) )
 
               ELSE
 
C
C                 Closest point is inside the interval 
C                 [AVERT(1,J), AVERT(1,K)].
C
                  MAXELE = DMAX1( MAXELE, ELEV( ANORM, C, 
     .                                          BVERT(1,I) ) )
 
               END IF
 
            END DO
 
         END IF
 
      END DO
 
      RETURN
      END
      
C12345678901234567890123456789012345678901234567890123456789012345678912
