C$Procedure EXTENT ( Find the extent of a set of points )
 
      SUBROUTINE EXTENT ( DIM, N, PTS, LOWER, UPPER )
 
C$ Abstract
C
C    Given a list of points in a space of dimension DIM, return
C    the points on the diagonal of the smallest ``box'' in dimension
C    DIM that contains all of the points.
C
C$ Copyright
C
C     Copyright (1997), California Institute of Technology.
C     U.S. Government sponsorship acknowledged.
C
C$ Required_Reading
C
C     None.
C
C$ Keywords
C
C     GEOMETRY
C
C$ Declarations
 
      IMPLICIT NONE
      INTEGER               DIM
      INTEGER               N
      DOUBLE PRECISION      PTS   ( DIM, * )
      DOUBLE PRECISION      LOWER ( DIM )
      DOUBLE PRECISION      UPPER ( DIM )
 
 
C$ Brief_I/O
C
C     VARIABLE  I/O  DESCRIPTION
C     --------  ---  --------------------------------------------------
C     DIM        I   Dimension of the space
C     N          I   Number of points.
C     PTS        I   A list of points in the space
C     LOWER      I   Lower point on the diagonal of the box
C     UPPER      I   Upper point on the diagonal of the box
C
C$ Detailed_Input
C
C     DIM        is the dimension of the space in which the points
C                live.  DIM should be at least 1.
C
C     N          is the number of points.
C
C     PTS        is a list of points in a space of dimension DIM.
C
C$ Detailed_Output
C
C     LOWER      is the point on the extreme diagonal of the box that
C                contains the input list of POINTS
C
C                   LOWER(I)  <= PTS(I,J) for J = 1, 2, ... N
C                                             I = 1, 2, ... DIM.
C
C                Moreover LOWER(I) is the maximum value that satisfies
C                the relationship specified above.
C
C     UPPER      is the point on the extreme diagonal of the box that
C                contains the input list of POINTS
C
C                   UPPER(I)  => PTS(I,J) for J = 1, 2, ... N
C                                             I = 1, 2, ... DIM.
C
C                Moreover UPPER(I) is the minimum value that satisfies
C                the relationship specified above.
C
C
C$ Parameters
C
C     None.
C
C$ Files
C
C     None.
C
C$ Exceptions
C
C     1) If DIM is zero or less the error 'SPICE(BADDIMENSION)' is
C        signalled.
C
C$ Particulars
C
C     Given a set of points in a space of arbitrary dimension, there
C     is a smallest box with faces orthogonal to the coordinate
C     axes that contains the set of points.
C
C     This routine finds the two corners of this box that are on
C     the extreme diagonal.  The points LOWER and UPPER are those
C     points on this diagonal.  Each coordinate of any point
C     contained in the box satisfies the following relationship
C
C        LOWER(I) <= POINT(I) <= UPPER(I)
C
C$ Examples
C
C     Suppose you have a set of points in the plane as shown below
C     and that you want to determine the corners of the diagonal
C     of the smallest rectangle that contains all of the points.
C
C             ( * --- is a point of interest )
C
C          ^
C          |                                 UPPER
C          |           +------------*------+
C          |           |     *             |
C          |           *                   |
C          |           |                   |
C          |           |                   *
C          |           |                   |
C          |           +---*---------------+
C          |      LOWER
C          |
C      ----+------------------------------------------->
C          |
C
C     You can retrieve the LOWER and UPPER corners by calling
C     EXTENT as shown below.
C
C
C        PTS(1,1)   = x_1
C        PTS(2,1)   = y_1
C
C        PTS(1,2)   = x_2
C        PTS(2,2)   = y_2
C
C        PTS(1,3)   = x_3
C        PTS(2,3)   = y_3
C
C        PTS(1,4)   = x_4
C        PTS(2,4)   = y_4
C
C        PTS(1,5)   = x_5
C        PTS(2,5)   = y_5
C
C        We have 5 points in 2-dimensional space
C
C        N   = 5
C        DIM = 2
C
C        CALL EXTENT ( DIM, N, PTS, UPPER, LOWER )
C
C
C$ Restrictions
C
C     None.
C
C$ Author_and_Institution
C
C     W.L. Taber      (JPL)
C
C$ Literature_References
C
C     None.
C
C$ Version
C
C-    SPICELIB Version 1.0.0, 3-MAR-1997 (WLT)
C
C
C-&
 
C$ Index_Entries
C
C     Determine the extent of a collection of points
C     Find the smallest box containing a set of points
C
C-&
 
      INTEGER               I
      INTEGER               J
 
C
C     Perform the lunacy checks first.  We use discovery checkin.
C
      IF ( DIM .LT. 1 ) THEN
         CALL CHKIN  ( 'EXTENT' )
         CALL SETMSG ( 'The dimension of the space must be '
     .   //            'positive, the value supplied to EXTENT '
     .   //            'was #. ' )
 
         CALL ERRINT ( '#', DIM )
         CALL SIGERR ( 'SPICE(BADDIMENSION)'  )
         CALL CHKOUT ( 'EXTENT' )
         RETURN
      END IF
 
 
 
      IF ( N .LE. 0 ) THEN
 
         DO I = 1, DIM
            LOWER(I) = 0.0D0
            UPPER(I) = 0.0D0
         END DO
 
      END IF
 
 
      DO I = 1, DIM
 
         LOWER(I) = PTS(I,1)
         UPPER(I) = PTS(I,1)
 
         DO J = 2, N
            LOWER(I) = MIN( PTS(I,J), LOWER(I) )
            UPPER(I) = MAX( PTS(I,J), UPPER(I) )
         END DO
 
      END DO
 
      RETURN
      END
