C$Procedure GETVOX ( Return the indices of voxel containing point )
 
      SUBROUTINE GETVOX ( VOXSIZ, VOXORI, NVOX, XYZ, INBOX, VOXID )
 
C$ Abstract
C
C     This subroutine returns the voxel grid indices that contains
C     a given point.
C
C$ Copyright
C
C     Copyright (1999), California Institute of Technology.
C     U.S. Government sponsorship acknowledged.
C
C$ Required_Reading
C
C     None.
C
C$ Keywords
C
C     PLATE
C
C$ Declarations
 
      IMPLICIT NONE
 
      DOUBLE PRECISION      VOXSIZ
      DOUBLE PRECISION      VOXORI    ( 3 )
      INTEGER               NVOX      ( 3 )
      DOUBLE PRECISION      XYZ       ( 3 )
      LOGICAL               INBOX
      INTEGER               VOXID     ( 3 )
 
 
C$ Brief_I/O
C
C     VARIABLE  I/O  DESCRIPTION
C     --------  ---  --------------------------------------------------
C     VOXSIZ     I   Voxel size in model units.
C     VOXORI     I   3-Vector location of the voxel grid origin
C                    in model units.
C     NVOX       I   Number of voxel in each dimension.
C     XYZ        I   Model grid coordinates of point.
C     INBOX      O   Logical value is true if point is inside voxel
C                    grid, false if not.
C     VOXID      O   3-Index of voxel location with the voxel grid.
C
C$ Detailed_Input
C
C     VOXSIZ     is a double precision scalar value giving the size of
C                each cubical voxel in model units.
C
C     VOXORI     is a double precision 3-vector indicating the location
C                of the voxel grid origin in model units.
C
C     NVOX       is an integer 3-vector giving the length of each
C                side of the voxel grid in voxel units.
C
C     XYZ        is a double precision 3-vector containing the location
C                of the point in question in model units.
C
C$ Detailed_Output
C
C     INBOX      is a logical scalar indicating whether or not the
C                point XYZ lies within the extent of the voxel grid.
C
C     VOXID      is an integer 3-vector containing the x, y, and z
C                voxel indices of the voxel containing the point.
C
C$ Parameters
C
C     None.
C
C$ Files
C
C     None.
C
C$ Exceptions
C
C     Error free.
C
C$ Particulars
C
C     None.
C
C$ Examples
C
C
C   C
C   C     Some position in model coordinates, i.e. the body fixed
C   C     units and frame coordinates.
C   C
C         DOUBLE PRECISION       POS(3)
C
C               ...
C
C   C
C   C     Retrive the voxel grid geometry description.
C   C
C         CALL VOXDIM ( NVOX, VOXSIZ, VOXORI)
C
C   C
C   C     Convert the coordinate POS to voxel grid indices.
C   C
C         CALL GETVOX ( VOXSIZ, VOXORI, NVOX, POS, INBOX, VOXID )
C
C$ Restrictions
C
C     None.
C
C$ Author_and_Institution
C
C     J.A. Bytof      (JPL)
C
C$ Literature_References
C
C     None.
C
C$ Version
C
C-    SPICELIB Version 1.1.0, 14-SEP-2004 (EDW)
C
C        Corrected VOXID(I) functions to allow for
C        negative voxel coordinates. Original functions
C        returned values GE 1. 
C
C        Added Examples. section
C
C-    SPICELIB Version 1.0.0, 28-JAN-1999 (JAB)
C
C-&
 
C$ Index_Entries
C
C     returns the voxel grid indices containing a given point
C
C-&

C
C     SPICELIB functions
C
      LOGICAL               RETURN

C
C     Local variables.
C 
      DOUBLE PRECISION       TERM
      INTEGER                I
 
C
C     Standard SPICE error handling.
C
 
      IF ( RETURN() ) THEN
         RETURN
      END IF
 
      CALL CHKIN  ( 'GETVOX' )
     
C
C     Initialize 'point in box' flag.
C
      INBOX = .TRUE.
 
C
C     Scale the point's coordinates to voxel grid space
C     and determine the indices of the voxel that contains it.
C
      DO I = 1, 3
      
C
C        A Galilean transform. Calculate the voxel coordinate
C        corresponding to the body centered coordinate. This
C        operation performs the same task as TOGRID, but
C        including the operation here improves GETVOX's
C        runtime performance.
C
         TERM = (XYZ(I)-VOXORI(I))/VOXSIZ

C
C        Calculate the voxel index for each degree of freedom
C        corresponding to the voxel coordinate.
C
C        Will the voxel index be a positive or negative value?
C        No index will have value 0. Ever.
C
         IF ( TERM. GE. 0.D0 ) THEN
            VOXID(I) = 1 + INT(TERM)
         ELSE
            VOXID(I) =-1 + INT(TERM)
         END IF

      END DO

C
C     Determine if point lies inside or outside the voxel grid.
C
      IF(      (VOXID(1) .LT. 1) .OR. (VOXID(1) .GT. NVOX(1)) ) THEN

         INBOX = .FALSE.

      ELSE IF( (VOXID(2) .LT. 1) .OR. (VOXID(2) .GT. NVOX(2)) ) THEN
 
         INBOX = .FALSE.
 
      ELSE IF( (VOXID(3) .LT. 1) .OR. (VOXID(3) .GT. NVOX(3)) ) THEN
 
         INBOX = .FALSE.
 
      END IF

C
C     Checkout of the error system, return.
C 
      CALL CHKOUT ( 'GETVOX' )
   
      RETURN
      END
