C$Procedure      INITFV ( Check FOV and generate normals )
 
      SUBROUTINE INITFV ( INSTID, FVROOM,
     .                    FRAME,  BSIGHT, FOVSIZ, FOV, FOVNRM )
 
C$ Abstract
C
C     Obtain and return field of view parameters and side normals.
C
C$ Copyright
C
C     Copyright (1999), California Institute of Technology.
C     U.S. Government sponsorship acknowledged.
C
C$ Required_Reading
C
C     None.
C
C$ Keywords
C
C     PLATE
C
C$ Declarations
 
      IMPLICIT              NONE
 
      INTEGER               INSTID
      INTEGER               FOVSIZ
      INTEGER               FVROOM
 
      DOUBLE PRECISION      BSIGHT    ( 3 )
      DOUBLE PRECISION      FOV       ( 3, * )
      DOUBLE PRECISION      FOVNRM    ( 3, * )
 
 
C$ Brief_I/O
C
C     VARIABLE  I/O  DESCRIPTION
C     --------  ---  --------------------------------------------------
C     INSTID     I   Instrument ID code.
C     FVROOM     I   Maximum number of FOV corners.
C     FRAME      O   Name of the frame FOV vectors defined in.
C     BSIGHT     O   Boresight vector in instrument frame.
C     FOVSIZ     O   Number of corners in FOV.
C     FOV        O   Vectors to corners of FOV.
C     FOVNRM     O   Vectors normal to the FOV sides.
C
C$ Detailed_Input
C
C     INSTID     NAIF ID of instrument.
C
C     FVROOM     Maximum allowed number of corners in polygonal FOV.
C
C$ Detailed_Output
C
C     FRAME      is the name of the frame in which the field of
C                view boundary vectors are defined.
C
C     BSIGHT     Boresight vector of instrument.
C
C     FOVSIZ     Number of corners in instrument's FOV.
C
C     FOV        Vectors to FOV corners.
C
C     FOVNRM     Normal vectors  to the sides of the field of view.
C
C$ Parameters
C
C     None.
C
C$ Files
C
C     None.
C
C$ Exceptions
C
C     TBD
C
C$ Particulars
C
C     TBD
C
C$ Examples
C
C     TBD
C
C$ Restrictions
C
C     None.
C
C$ Author_and_Institution
C
C     J.A. Bytof      (JPL)
C
C$ Literature_References
C
C     None.
C
C$ Version
C
C-    SPICELIB Version 2.0.0, 27-OCT-1999 (JAB)
C
C        Replaced a 'STOP' with a 'RETURN' after
C        a call to CHKOUT.
C
C-    SPICELIB Version 1.0.0, 30-APR-1999 (JAB)
C
C-&
 
C$ Index_Entries
C
C     return field-of-view parameters and side normals
C
C-&
C
C     SPICELIB Functions
C
 
      CHARACTER*(2)         ANA
 
      DOUBLE PRECISION      TWOPI
      DOUBLE PRECISION      VDOT
 
      LOGICAL               RETURN
 
C
C     Parameters
C
 
      INTEGER               WDSIZE
      PARAMETER           ( WDSIZE = 32 )
 
      DOUBLE PRECISION      NSIDES
      PARAMETER           ( NSIDES = 16.0D0 )
 
      DOUBLE PRECISION      RECIP
      PARAMETER           ( RECIP = 1.D0 / 16.D0 )
 
C
C     Local variables
C
 
      INTEGER               I
 
      DOUBLE PRECISION      TEST
      DOUBLE PRECISION      ANGLE
      DOUBLE PRECISION      FOVAVE    ( 3 )
 
      CHARACTER*(2)         ARTCLE
      CHARACTER*(WDSIZE)    SHAPE
      CHARACTER*(WDSIZE)    FRAME
 
 
C
C     Standard SPICE error handling.
C
      IF ( RETURN () ) THEN
         RETURN
      ELSE
         CALL CHKIN ( 'INITFV' )
      END IF
 
 
C
C     Determine where the boresight and edge of the field of
C     view are pointing.
C
 
      CALL GETFOV ( INSTID, FVROOM, SHAPE, FRAME, BSIGHT,
     .              FOVSIZ, FOV )
 
      CALL UCASE  ( SHAPE,  SHAPE )
 
 
      IF ( SHAPE .EQ. 'CIRCLE' ) THEN
 
C
C     We generate a field of view for the circular case.
C
 
         ANGLE = TWOPI() / NSIDES
 
         DO I = 1, 15
            CALL VROTV ( FOV(1,I), BSIGHT, ANGLE, FOV(1,I+1) )
         END DO
 
         FOVSIZ = 16
 
 
      ELSE IF ( SHAPE .NE. 'POLYGON' ) THEN
 
         ARTCLE =  ANA ( SHAPE, 'L' )
 
         CALL SETMSG ( 'Fields of view in the shape of # ''#'' '
     .   //            'are not supported by TPNTOCC at the '
     .   //            'present time. ' )
 
         CALL ERRCH  ( '#', ARTCLE )
         CALL ERRCH  ( '#', SHAPE  )
         CALL SIGERR ( 'SPICE(UNSUPPORTEDFOV)'  )
         CALL CHKOUT ( 'INITFV' )
         RETURN
 
      END IF
 
 
C
C     Obtain the normal vector for each side of the FOV volume.
C
 
         DO I = 1, FOVSIZ-1
            CALL VCRSS  ( FOV(1,I+1), FOV(1,I),  FOVNRM(1,I) )
         END DO
 
         CALL VCRSS ( FOV(1,1), FOV(1,FOVSIZ), FOVNRM(1,FOVSIZ) )
 
C
C     Get the "average" vector of the FOV boundry.
C
 
      CALL VPACK ( 0.D0, 0.D0, 0.D0, FOVAVE )
 
      DO I = 1, FOVSIZ
         CALL NRMIZE ( FOVNRM(1,I), 3,   FOVNRM(1,I) )
         CALL VADD   ( FOV(1,I), FOVAVE, FOVAVE      )
      END DO
 
      CALL VSCL ( RECIP, FOVAVE, FOVAVE )
      CALL NRMIZE ( FOVAVE, 3, FOVAVE )
 
C
C     Verify that the normals point inward, flip direction if not.
C
 
      DO I = 1, FOVSIZ
 
C
C     Test if normal points inward.
C
 
         TEST = VDOT( FOVNRM(1,I), FOVAVE )
 
         IF ( TEST .LT. 0.D0 ) THEN
 
            CALL VMINUS ( FOVNRM(1,I), FOVNRM(1,I) )
 
         END IF
 
      END DO
 
      CALL CHKOUT ( 'INITFV' )
 
      RETURN
      END
