C$Procedure  INPRZM  ( does a plate lie in a right prism )
 
      LOGICAL FUNCTION INPRZM ( AVERT, BVERT, ANORM, ACENT  )
 
C$ Abstract
C
C     Determine if a plate lies within a right triangular prism
C     that has another plate for a base and extends in the direction
C     of that plate's normal vector.
C
C     Loop over the four planes that define the semi-infinite
C     prism bounded at one end by a plate.  If any path remains
C     the function returns .TRUE. otherwise .FALSE.
C
C
C$ Copyright
C
C     Copyright (1999), California Institute of Technology.
C     U.S. Government sponsorship acknowledged.
C
C$ Required_Reading
C
C     None.
C
C$ Keywords
C
C     PLATE
C
C$ Declarations
 
      IMPLICIT              NONE
 
      DOUBLE PRECISION      ANORM    ( 3 )
      DOUBLE PRECISION      ACENT    ( 3 )
      DOUBLE PRECISION      AVERT    ( 3, 3 )
      DOUBLE PRECISION      BVERT    ( 3, 3 )
 
 
C$ Brief_I/O
C
C     VARIABLE  I/O  DESCRIPTION
C     --------  ---  --------------------------------------------------
C     AVERT      I   Coordinates of plate-A's vertices.
C     BVERT      I   Coordinates of plate-B's vertices.
C     ANORM      I   Normal of plate-A.
C     ACENT      I   Center of plate-A.
C
C     The function returns .TRUE. if any part of plate-B lies
C     above plate-A.
C
C$ Detailed_Input
C
C     AVERT      Coordinates of plate-A's vertices.
C     BVERT      Coordinates of plate-B's vertices.
C     ANORM      Upward normal of plate-A.
C     ACENT      Center of plate-A.
C
C$ Detailed_Output
C
C     The function returns .TRUE. if any part of plate-B lies
C     above plate-A.
C
C$ Parameters
C
C     None.
C
C$ Files
C
C     None.
C
C$ Exceptions
C
C     Error free.
C
C$ Particulars
C
C     Later.
C
C$ Examples
C
C     TBD
C
C$ Restrictions
C
C     None.
C
C$ Author_and_Institution
C
C     J.B. Bytof      (JPL)
C
C$ Literature_References
C
C     None.
C
C$ Version
C
C-    SPICELIB Version 1.0.0, 16-MAR-1999 (JAB)
C
C-&
 
C$ Index_Entries
C
C     plate
C
C-&
 
      INTEGER               NPTS
      PARAMETER           ( NPTS = 3 )
 
      INTEGER               LBPOOL
      PARAMETER           ( LBPOOL = -5 )
 
      INTEGER               DIM
      PARAMETER           ( DIM = 3 )
 
      INTEGER               LNGEST
      PARAMETER           ( LNGEST = 100 )
 
      INTEGER               PTHSIZ
      PARAMETER           ( PTHSIZ = LNGEST + 2 )
 
      INTEGER               I
      INTEGER               J
      INTEGER               K
 
 
      INTEGER               SIZE
      INTEGER               BEG
      INTEGER               TAIL
      INTEGER               PTHIN
      INTEGER               PTHOUT
 
      INTEGER               PATHS  ( 2, LBPOOL:PTHSIZ )
 
      DOUBLE PRECISION      SNORM    ( 3 )
      DOUBLE PRECISION      AMID     ( 3 )
 
      DOUBLE PRECISION      VERTXS   ( DIM*PTHSIZ )
 
 
      INPRZM = .FALSE.
 
C
C     Initialize plate-B's path.
C
 
      CALL PTHINI ( DIM, LNGEST, PTHSIZ, PATHS, VERTXS )
 
 
      CALL PTHMAK ( NPTS, DIM, BVERT, PATHS, VERTXS, BEG )
 
 
C
C     Is any region of plate-B above plate-A's horizon plane?
C
 
      CALL POLYCT ( BEG,   PATHS,  VERTXS, ACENT,
     .              ANORM, PTHIN,  PTHOUT          )
 
 
      CALL PTHINF ( PTHIN, PATHS, BEG, TAIL, SIZE  )
 
 
      IF ( SIZE .EQ. 0 ) THEN
         RETURN
      END IF
 
C
C     Is any region of plate-B inside the right triangular
C     prism having plate-A as its cross-section?
C
 
      DO I = 1, 3
 
         J = I
         K = I + 1
         IF ( K .GT. 3 ) K = 1
 
C
C     Generate the bounding plane inward pointing normal.
C
 
         CALL PTSHLF_3D ( AVERT(1,J), AVERT(1,K), ACENT, SNORM, AMID  )
 
 
C
C     Is any region of plate-B on the in-side of the plane?
C
 
         CALL POLYCT ( BEG,   PATHS,  VERTXS,  AMID,
     .                 SNORM, PTHIN,  PTHOUT           )
 
         CALL PTHINF ( PTHIN, PATHS, BEG, TAIL, SIZE )
 
         IF ( SIZE .EQ. 0 ) THEN
            RETURN
         END IF
 
      END DO
 
      INPRZM = .TRUE.
 
 
      RETURN
      END
