C$Procedure      NRMIZE ( Normalize a vector )
 
      SUBROUTINE NRMIZE ( V, DIM, VOUT )
 
C$ Abstract
C
C     This routine produces a positive scalar multiple of the
C     input vector such that the largest component of the scalar
C     multiple is 1.
C
C$ Copyright
C
C     Copyright (1996), California Institute of Technology.
C     U.S. Government sponsorship acknowledged.
C
C$ Required_Reading
C
C      None.
C
C$ Keywords
C
C      UTILITY
C
C$ Declarations
 
      IMPLICIT NONE
      DOUBLE PRECISION      V    ( * )
      INTEGER               DIM
      DOUBLE PRECISION      VOUT ( * )
 
C$ Brief_I/O
C
C     VARIABLE  I/O  DESCRIPTION
C     --------  ---  --------------------------------------------------
C     V          I   Vector to be "normalized"
C     DIM        I   Dimension of the input vector
C     VOUT       O   Normalized version of V.
C
C$ Detailed_Input
C
C     V          is a vector that should be "normalized".
C
C     DIM        is the dimension of the input vector.
C
C$ Detailed_Output
C
C     VOUT       is a vector parallel to the input vector V but such
C                that the maximum value of the absolute values of
C                the components is 1.  VOUT may overwrite V.
C
C                If V is the zero vector, VOUT will be the zero vector.
C
C$ Parameters
C
C     None.
C
C$ Files
C
C     None.
C
C$ Exceptions
C
C     Error free.
C
C     1)  If V is the zero vector, the output vector, VOUT, will be
C         the zero vector.
C
C     2)  If DIM is less than zero, no action is taken.
C
C$ Particulars
C
C     This utility normalizes an input vector so that the largest
C     component out the output vector (in absolute value) is 1.
C
C$ Examples
C
C     Suppose you would like to determine if the vector V lies in
C     the half-space whose edge contains the origin and that
C     is pointed to by the vector N normal to the edge of the
C     half-space.  This routine allows you to first normalize V so
C     that floating point over and underflows can be avoided.
C
C        CALL NRMLIZ ( V, DIM, V )
C
C        IF ( VDOTG( V, N, DIM ) .GT. 0.0D0 ) THEN
C
C           WRITE (*,*)  'V lies in the half-space'
C
C        ELSE
C
C           WRITE (*,*)  'V  lies outside the half-space.'
C
C        END IF
C
C$ Restrictions
C
C     None.
C
C$ Author_and_Institution
C
C     W.L. Taber      (JPL)
C
C$ Literature_References
C
C     None.
C
C$ Version
C
C-    SPICELIB Version 1.0.0, 09-JUN-1997 (WLT)
C
C
C-&
 
C$ Index_Entries
C
C     Normalize a vector
C
C-&
 
      DOUBLE PRECISION      BIG
      INTEGER               I
 
C
C     We handle the 3-dimensional and 2-dimensional cases separately
C     in the hope that the expense of checking dimensions is worth
C     the savings gained by avoiding loop overhead.
C
C     First we take care of the 3-dimensional case.
C
      IF ( DIM  .EQ. 3 ) THEN
 
         BIG = MAX ( DABS(V(1)), DABS(V(2)), DABS(V(3)) )
 
         IF ( BIG .EQ. 0.0D0 ) THEN
 
            VOUT(1) = 0.0D0
            VOUT(2) = 0.0D0
            VOUT(3) = 0.0D0
 
         ELSE
 
            VOUT(1) = V(1) / BIG
            VOUT(2) = V(2) / BIG
            VOUT(3) = V(3) / BIG
 
         END IF
C
C     Try the two dimensional case next.
C
      ELSE IF ( DIM  .EQ. 2 ) THEN
 
         BIG = MAX ( DABS(V(1)), DABS(V(2)) )
 
         IF ( BIG .EQ. 0.0D0 ) THEN
 
            VOUT(1) = 0.0D0
            VOUT(2) = 0.0D0
 
         ELSE
 
            VOUT(1) = V(1) / BIG
            VOUT(2) = V(2) / BIG
 
         END IF
 
C
C     Finally the general case
C
      ELSE
 
         BIG = 0.0D0
 
         DO I = 1, DIM
 
            BIG = MAX ( BIG, DABS(V(I)) )
 
         END DO
 
         IF ( BIG .EQ. 0.0D0 ) THEN
 
            DO I = 1, DIM
               VOUT(I) = 0.0D0
            END DO
 
 
         ELSE
 
            DO I = 1, DIM
               VOUT(I) = V(I)/BIG
            END DO
 
         END IF
 
      END IF
 
      RETURN
      END
