C$Procedure      PINHFS ( Point in half-space )
 
      INTEGER FUNCTION PINHFS ( POINT, EPOINT, NORMAL )
 
C$ Abstract
C
C    Given a half-space and a point, determine whether the point
C    lies in the half-space, in the edge of the half-space or
C    in the opposite half-space.
C
C$ Copyright
C
C     Copyright (1999), California Institute of Technology.
C     U.S. Government sponsorship acknowledged.
C
C$ Required_Reading
C
C     None.
C
C$ Keywords
C
C     GEOMETRY
C
C$ Declarations
 
      IMPLICIT NONE
      DOUBLE PRECISION      POINT  ( 3 )
      DOUBLE PRECISION      EPOINT ( 3 )
      DOUBLE PRECISION      NORMAL ( 3 )
 
C$ Brief_I/O
C
C     VARIABLE  I/O  DESCRIPTION
C     --------  ---  --------------------------------------------------
C     POINT      I   a point in the space.
C     EPOINT     I   a point on the edge of the half-space
C     NORMAL     I   an edge normal pointing into the half-space
C
C     Returns 1 if in the half-space; 0 for edge; or -1 in opposite
C
C$ Detailed_Input
C
C     POINT      is an arbitrary point in space.  POINT(1) is
C                the x-coordinate of the point, POINT(2) is the
C                y-coordinate of the point, POINT(3) is the
C                z-coordinate.
C
C     EPOINT     is an X, Y, Z point in the edge of the 
C                half-space.
C
C     NORMAL     is a vector normal to the edge of the half-space
C                that points into the half-space.
C
C$ Detailed_Output
C
C     The function returns 1 if the point lies in the open half
C     space, 0 if the point lies in the edge, and -1 if it lies
C     in the opposite half-space
C
C$ Parameters
C
C     None.
C
C$ Files
C
C     None.
C
C$ Exceptions
C
C     Error free.
C
C$ Particulars
C
C     Given a point in the edge of a half-space and a vector pointing
C     into the interior of the half-space as shown below
C
C
C         + + + + + + + + + + + + + + + + +
C          + + + + + + + + + + + + + + + +
C         + + + + + +   + + + Half Space+ +
C          + + + + +     + + + + + + + + +
C         + + + + +   ^   + + + + + + + + +
C          + + + + +  |  + + + + + + + + +
C         + + + + +   |   + + + + + + + + +
C          + + + + +  |  + + + + + + + + +
C         + + + + +   |   + + + + + + + + +
C          + + + + +  |  + + + + + + + + +
C         ============0====================
C          - - - - - - - - - - - - - - - -
C         - - - - - - - - Opposite- - - -
C          - - - - - - - - - - - - - - - -
C         - - - - - - - - - - - - - - - - -
C          - - - - - - - - - - - - - - - -
C         - - - - - - - - - - - - - - - - -
C
C     this routine returns 1 if the point lies in the half-space
C     pointed to by the normal vector.  It returns 0 if the
C     point lies in the edge.  It returns -1 if the point lies
C     in the opposite half-space.
C
C     If NORMAL is the zero vector, the function returns the
C     value 0.
C
C$ Examples
C
C     Suppose you have been given points on the vertices of
C     a tetrahedron and the normals to the faces to the
C     tetrahedron.  Given an arbitrary point P, you can
C     use this routine to determine whether or not P is in the
C     interior of the tetrahedron.  Let P1, P2, P3, P4 be the
C     vertices of the tetrahedron and let N1, N2, N3 and N4 be
C     the normals to the faces of the tetrahedron (Ni is normal
C     to the face Pi, Pi+1, Pi+2 and Ni points into the interior
C     of the tetrahedron.  To determine if POINT is inside the
C     tetrahedron the following computation will suffice.
C
C
C        INSIDE =      PINHFS ( POINT, P1, N1 ) .GT. 0
C                .AND. PINHFS ( POINT, P2, N2 ) .GT. 0
C                .AND. PINHFS ( POINT, P3, N3 ) .GT. 0
C                .AND. PINHFS ( POINT, P4, N4 ) .GT. 0
C
C$ Restrictions
C
C     None.
C
C$ Author_and_Institution
C
C     J.A. Bytof      (JPL)
C     W.L. Taber      (JPL)
C
C$ Literature_References
C
C     None.
C
C$ Version
C
C-    SPICELIB Version 2.1.0, 02-SEP-2004 (EDW)
C
C         Edits to improved runtime speed, additions
C         to comments and headers.
C
C-    SPICELIB Version 2.0.0, 01-FEB-1999 (JAB)
C
C         Added code to account for points very close
C         to the plane; return value is set to 0 (point in edge)
C         in this case.
C
C-    SPICELIB Version 1.0.0, 28-OCT-1996 (WLT)
C
C
C-&
 
C$ Index_Entries
C
C     Determine if a point is in a half-space.
C     Determine where a point lies in space divided by a plane
C
C-&
C
C     Spicelib Functions
C

C
C     Local Variables
C
 
      DOUBLE PRECISION      SCALE
      DOUBLE PRECISION      TEST
      DOUBLE PRECISION      TOL
      DOUBLE PRECISION      V      ( 3 )
 
 
C
C     TOL is set to a more or less arbitrary small value.
C
      TOL = 1.D-6
 
C
C     Compute the vector from the edge point to the input point.
C
C        _   _____   ______
C        V = POINT - EPOINT
C
      V(1) = POINT(1) - EPOINT(1)
      V(2) = POINT(2) - EPOINT(2)
      V(3) = POINT(3) - EPOINT(3)


C
C     We re-scale the normal in the hopes of gaining a slightly
C     more robust half-space test.
C
      SCALE = MAX ( DABS(NORMAL(1)), DABS(NORMAL(2)), DABS(NORMAL(3)) )

      IF ( SCALE .EQ. 0.0D0 ) THEN

         PINHFS = 0

      ELSE

C
C        For the sake of speed, explicitly compute the
C        quantity:
C
C           < V, NORMAL/SCALE>.
C
         TEST = V(1)*(NORMAL(1)/SCALE) + 
     .          V(2)*(NORMAL(2)/SCALE) +
     .          V(3)*(NORMAL(3)/SCALE)


         IF ( DABS(TEST) .LE. TOL ) THEN

C
C          If TEST has value within TOL of 0, decide the
C          point exists on the plane.
C
            PINHFS = 0

         ELSE IF ( TEST .GT. 0.0D0 ) THEN
C
C           If the inner product TEST has positive value, the
C           point exists in the halfspace.
C 

            PINHFS = 1

         ELSE

C
C           Only remaining possibility, the particle
C           lies in the opposite half space.
C
            PINHFS = -1

         END IF
 
      END IF
 
      RETURN
      END
