C$Procedure      PINHLF ( Point in half-plane )
 
      INTEGER FUNCTION PINHLF ( POINT, EPOINT, NORMAL )
 
C$ Abstract
C
C    Given a half-plane and a point, determine whether the point
C    lies in the half-plane, in the edge of the half-plane or
C    in the opposite half-plane.
C
C$ Copyright
C
C     Copyright (1997), California Institute of Technology.
C     U.S. Government sponsorship acknowledged.
C
C$ Required_Reading
C
C     None.
C
C$ Keywords
C
C     GEOMETRY
C
C$ Declarations
 
      IMPLICIT NONE
      DOUBLE PRECISION      POINT  ( 2 )
      DOUBLE PRECISION      EPOINT ( 2 )
      DOUBLE PRECISION      NORMAL ( 2 )
 
C$ Brief_I/O
C
C     VARIABLE  I/O  DESCRIPTION
C     --------  ---  --------------------------------------------------
C     POINT      I   a point in the plane.
C     EPOINT     I   a point on the edge of the half-plane
C     NORMAL     I   an edge normal pointing into the half-plane
C
C     Returns 1 if in the half-plane; 0 for edge; or -1 in opposite
C
C$ Detailed_Input
C
C     POINT      is an arbitrary point in the plane.  POINT(1) is
C                the x-coordinate of the point, POINT(2) is the
C                y-coordinate of the point.
C
C     EPOINT     is a point in the edge of the half-plane.
C
C     NORMAL     is a vector normal to the edge of the half-plane
C                that points into the half-plane.
C
C$ Detailed_Output
C
C     The function returns 1 if the point lies in the open half
C     plane, 0 if the point lies in the edge, and -1 if it lies
C     in the opposite half-plane
C
C$ Parameters
C
C     None.
C
C$ Files
C
C     None.
C
C$ Exceptions
C
C     Error free.
C
C     1) If NORMAL is the zero vector, the function returns the
C        value 0.
C
C$ Particulars
C
C     Given a point in the edge of a half-plane and a vector pointing
C     into the interior of the half-plane as shown below
C
C
C         + + + + + + + + + + + + + + + + +
C          + + + + + + + + + + + + + + + +
C         + + + + + +   + + + Half Plane+ +
C          + + + + +     + + + + + + + + +
C         + + + + +   ^   + + + + + + + + +
C          + + + + +  |  + + + + + + + + +
C         + + + + +   |   + + + + + + + + +
C          + + + + +  |  + + + + + + + + +
C         + + + + +   |   + + + + + + + + +
C          + + + + +  |  + + + + + + + + +
C         ============0====================
C          - - - - - - - - - - - - - - - -
C         - - - - - - - - Opposite- - - -
C          - - - - - - - - - - - - - - - -
C         - - - - - - - - - - - - - - - - -
C          - - - - - - - - - - - - - - - -
C         - - - - - - - - - - - - - - - - -
C
C     this routine returns 1 if the point lies in the half plane
C     pointed to by the normal vector.  It returns 0 if the
C     point lies in the edge.  It returns -1 if the point lies
C     in the opposite half-plane.
C
C
C$ Examples
C
C     Suppose you have been given points on the vertices of
C     a parallelogram and the normals to the sides to the
C     parallelogram.  Given an arbitrary point P, you can
C     use this routine to determine whether or not P is in the
C     interior of the parallelogram.  Let P1, P2, P3, P4 be the
C     vertices of the parallelogram and let N1, N2, N3 and N4 be
C     the normals to the edges of the parallelgram (Ni is normal
C     to the edge from Pi to Pi+1 and Ni points into the interior
C     of the parallelogram.  To determine if POINT is inside the
C     parallelogram the following computation will suffice.
C
C
C        INSIDE =      PINHLF ( POINT, P1, N1 ) .GT. 0
C                .AND. PINHLF ( POINT, P2, N2 ) .GT. 0
C                .AND. PINHLF ( POINT, P3, N3 ) .GT. 0
C                .AND. PINHLF ( POINT, P4, N4 ) .GT. 0
C
C$ Restrictions
C
C     None.
C
C$ Author_and_Institution
C
C     W.L. Taber      (JPL)
C
C$ Literature_References
C
C     None.
C
C$ Version
C
C-    SPICELIB Version 1.1.0, 02-SEP-2004 (EDW)
C
C         Edits to improved runtime speed, additions
C         to comments and headers; added use of TOL parameter
C         to account for points very close to the plane; 
C         return has value 0 (point in edge) in this case.
C
C-    SPICELIB Version 1.0.0, 28-OCT-1996 (WLT)
C
C
C-&
 
C$ Index_Entries
C
C     Determine if a point is in a half-plane.
C     Determine where a point lies in a plane divided by a line
C
C-&


C
C     Local Variables
C
      DOUBLE PRECISION      SCALE
      DOUBLE PRECISION      TEST
      DOUBLE PRECISION      V      ( 2 )
      DOUBLE PRECISION      TOL


C
C     TOL is set to a more or less arbitrary small value.
C
      TOL = 1.D-6
 
C
C     Compute the vector from the edge point to the input point.
C
C        _   _____   ______
C        V = POINT - EPOINT
C
      V(1) = POINT(1) - EPOINT(1)
      V(2) = POINT(2) - EPOINT(2)

C
C     We re-scale the normal in the hopes of gaining a slightly
C     more robust half-plane test.
C
      SCALE = MAX ( DABS(NORMAL(1)), DABS(NORMAL(2)) )
 
      IF ( SCALE .EQ. 0.0D0 ) THEN
 
         PINHLF = 0
 
      ELSE
 
C
C        For the sake of speed, explicitly compute the
C        quantity:
C
C           < V, NORMAL/SCALE>.
C
         TEST = V(1)*(NORMAL(1)/SCALE) + 
     .          V(2)*(NORMAL(2)/SCALE) 
 
 
         IF ( DABS(TEST) .LE. TOL ) THEN

C
C          If TEST has value within TOL of 0, decide the
C          point exists on the plane.
C
            PINHLF = 0

         ELSE IF ( TEST .GT. 0.0D0 ) THEN
C
C           If the inner product TEST has positive value, the
C           point exists in the halfspace.
C 

            PINHLF = 1

         ELSE

C
C           Only remaining possibility, the particle
C           lies in the opposite half space.
C
            PINHLF = -1

         END IF
 
      END IF
 
      RETURN
      END
