C$Procedure   PINSEG   ( line segment point closest to another point )
 
      SUBROUTINE PINSEG ( A, B, P, C, CLEN )
 
C$ Abstract
C
C     Given a line segment AB, and the line it defines,
C     determine the point C on line AB that is closest to 
C     point P.
C
C
C$ Copyright
C
C     Copyright (1999), California Institute of Technology.
C     U.S. Government sponsorship acknowledged.
C
C$ Required_Reading
C
C     None.
C
C$ Keywords
C
C     LINE, POINT
C
C$ Declarations
 
      IMPLICIT              NONE
 
      DOUBLE PRECISION      CLEN
      DOUBLE PRECISION      A      ( 3 )
      DOUBLE PRECISION      B      ( 3 )
      DOUBLE PRECISION      C      ( 3 )
      DOUBLE PRECISION      P      ( 3 )
 
 
C$ Brief_I/O
C
C     VARIABLE  I/O  DESCRIPTION
C     --------  ---  --------------------------------------------------
C     A          I   Coordinates of first end of line segment.
C     B          I   Coordinates of second end of line segment.
C     P          I   Coordinates of point not on line segment.
C     C          O   Coordinates of point on line closest to P.
C     CLEN       O   Distance from point A to point C in terms
C                    of the A-B distance.
C
C$ Detailed_Input
C
C     A          Coordinates of first end of line segment.
C
C     B          Coordinates of second end of line segment.
C
C     P          Coordinates of point not on line segment.
C
C$ Detailed_Output
C
C     C          Coordinates of point on line closest to P.
C
C     CLEN       Distance from point A to point C in terms
C                of the A-B distance.  If the point is between
C                A and B, the value of CLEN is between 0 and 1.
C                If C is not between A and B, but closest to A,
C                the value of CLEN is negative.  Otherwise,
C                the value of CLEN is greater than one.
C
C$ Parameters
C
C     None.
C
C$ Files
C
C     None.
C
C$ Exceptions
C
C     1) The routine signals the error "SPICE(DEGENLINESEG)"
C        if position A equals position B, i.e. 
C            __
C            AB = {0}
C
C$ Particulars
C
C     The routine is partially optimized for run-time speed.
C     VSUB and VADD calls replaced with explicit calculations.
C
C$ Examples
C
C     TBD
C
C$ Restrictions
C
C     None.
C
C$ Author_and_Institution
C
C     J.A. Bytof      (JPL)
C
C$ Literature_References
C
C     None.
C
C$ Version
C
C-    SPICELIB Version 1.0.0, 12-MAR-1999 (JAB)
C
C-&
 
C$ Index_Entries
C
C     line
C
C-&
 
      DOUBLE PRECISION      VDOT
      DOUBLE PRECISION      VNORM
      LOGICAL               RETURN


      DOUBLE PRECISION      ABLEN
      DOUBLE PRECISION      ACLEN
      DOUBLE PRECISION      BCDOT
      DOUBLE PRECISION      AB     ( 3 )
      DOUBLE PRECISION      AP     ( 3 )
      DOUBLE PRECISION      AC     ( 3 )

C
C     Standard SPICE error handling.
C
      IF ( RETURN() ) THEN
         RETURN
      END IF

      CALL CHKIN  ( 'PINSEG' )

 
C
C     Generate the vector A to B.
C
      AB(1) = B(1) - A(1)
      AB(2) = B(2) - A(2)
      AB(3) = B(3) - A(3)

      ABLEN =  VNORM ( AB )

      IF ( ABLEN .EQ. 0.D0 ) THEN

         CALL SETMSG ( 'Degenerate line segment. '
     .     // 'Endpoints have the same coordinates.' )
         CALL SIGERR ( 'SPICE(DEGENLINESEG)'         )
         CALL CHKOUT ( 'PINSEG'                      )
         RETURN

      END IF
 
C
C     Generate the vector A to P.
C
      AP(1) = P(1) - A(1)
      AP(2) = P(2) - A(2)
      AP(3) = P(3) - A(3)
 
C
C     Project the vector AP onto AB.
C
      CALL VPROJ ( AP, AB, AC )
 
C
C     Get length of the projection vector.
C
      ACLEN =  VNORM ( AC )
      CLEN  = ACLEN / ABLEN
 
C
C     What is the dot product of vectors AB and AC?
C
      BCDOT = VDOT ( AB, AC )
 
      C(1) = A(1) + AC(1)
      C(2) = A(2) + AC(2)
      C(3) = A(3) + AC(3)

C
C     If the dot product is less than zero, the
C     vectors AB and AC are anti-parallel,  and set 
C     the sign of the A to C distance to a negative
C     value.
C
 
      IF ( BCDOT .LT. 0.D0 ) THEN
         CLEN = -CLEN
      END IF
 
      CALL CHKOUT ( 'PINSEG' )

      RETURN
      END
