C$Procedure PLBORE ( Plate Intersected by Ray )

      SUBROUTINE PLBORE ( POSOBS, BSIGHT, PLID, BODY, XYZHIT, FOUND )
 
C$ Abstract
C
C     Determine the ID code of the plate and body fixed
C     coordinates of the point where an arbitrary ray intersects
C     the plate model.
C
C$ Copyright
C
C     Copyright (1999), California Institute of Technology.
C     U.S. Government sponsorship acknowledged.
C
C$ Required_Reading
C
C      None.
C
C$ Keywords
C
C      SHAPES
C
C$ Declarations
 
      IMPLICIT NONE
 
      INCLUDE              'pltmax.inc'
 
      DOUBLE PRECISION      POSOBS   ( 3 )
      DOUBLE PRECISION      BSIGHT   ( 3 )
      INTEGER               PLID
      INTEGER               BODY
      DOUBLE PRECISION      XYZHIT   ( 3 )
      LOGICAL               FOUND
 
C$ Brief_I/O
C
C     VARIABLE  I/O  DESCRIPTION
C     --------  ---  --------------------------------------------------
C     POSOBS     I   observer location in body fixed coordinates
C     BSIGHT     I   ray direction in body fixed coordinates
C     PLID       O   ID code of the plate intersected by the boresight
C     BODY       O   ID of the body corresponding to the plate PLID
C     XYZHIT     O   Point on plate model where ray hits
C     FOUND      O   TRUE if there is a plate, FALSE otherwise.
C
C$ Detailed_Input
C
C     POSOBS     observer location in body fixed coordinates
C
C     BSIGHT     ray direction from observer in body fixed coordinates
C
C
C$ Detailed_Output
C
C     PLID       is the plate ID of the first plate that is intersected
C                by the boresight of the instrument specified by INSTID.
C                In the event that the boresight intersects the surface
C                model at an edge or vertex, the plate whose normal is
C                closest to anti-parallel to the boresight direction is
C                selected.  If this criteria does not identify uniquely
C                the plate to chose, the plate whose center is closest
C                to the instrument is chose.  If this still does not
C                resolve which plate to chose, then the plate with 
C                the smallest ID value (among those in contention) 
C                is returned.
C
C     BODY       is the NAIF ID code for the body that the boresight
C                first intersects and to which the plate specified by
C                PLID belongs.
C
C     XYZHIT     is the closest point on the plate model where the ray
C                from the observer's location intersects.
C
C     FOUND      is a logical flag that indicates whether or not
C                the boresight vector does indeed intersect.  If
C                the boresight intersects a plate FOUND is TRUE.
C                Otherwise FOUND is FALSE.
C
C$ Parameters
C
C     None.
C
C$ Files
C
C     None.
C
C$ Exceptions
C
C     1) If a required kernel (see Particulars) below cannot be located
C        an error will be diagnosed by one of the routines called
C        by this routine.
C
C     2) ARRAYTOOSMALL - more
C
C$ Particulars
C
C     Given a set of objects whose surfaces are modeled by
C     a set of covering triangular plates, this routine returns
C     the first plate in the list of plates that is intersected
C     by the boresight of the specified instrument.
C
C     A substantial amount of preparation is required in the
C     users program before this routine can be called.
C
C     The user must "load" a shape model for the object of interest
C
C     Once these preparations have been performed this routine will
C     return the plate first intersected by the boresight (if one
C     exists.
C
C     Note, coding for this routine includes sections optimized
C     speed.
C
C$ Examples
C
C     When time permits.
C
C$ Restrictions
C
C     None.
C
C$ Author_and_Institution
C
C     J.A. Bytof      (JPL)
C     W.L. Taber      (JPL)
C
C$ Literature_References
C
C     Woo, A. "Fast Ray-Box Intersection", Graphic Gems I, 395-396.
C
C$ Version
C
C-    SPICELIB Version 1.1.0, 19-AUG-2004 (EDW)
C
C        Implemented "Fast Ray-Box Intersection" algorithm.
C        Renamed routine PLBORE from PLBORE_3.
C
C-    SPICELIB Version 1.0.0, 25-FEB-1999 (JAB)
C
C        Based on PLBORE and PLBORE_2.
C
C-&
 
C$ Index_Entries
C
C     plate and plate model point intersected by ray
C
C-&

C
C     SPICELIB Functions
C 
      DOUBLE PRECISION      VDOT
      DOUBLE PRECISION      VNORM
 
      LOGICAL               RETURN
      LOGICAL               FAILED

      INTEGER               PINHFS
 
C
C     Parameters
C
      INTEGER              NVXLST
      PARAMETER          ( NVXLST = 10000 )

C
C     Local Variables
C
      DOUBLE PRECISION      NORML  ( 3 )
      DOUBLE PRECISION      TOCENT ( 3 )
      DOUBLE PRECISION      VOXORI ( 3 )
      DOUBLE PRECISION      VOXFAR ( 3 )
      DOUBLE PRECISION      PLANE  ( 3 )
      DOUBLE PRECISION      MAXT   ( 3 )
      DOUBLE PRECISION      ORIGIN ( 3 )
      DOUBLE PRECISION      COORD  ( 3 )
      DOUBLE PRECISION      BDSITE ( 3 )
      DOUBLE PRECISION      CENTR  ( 3 )
      DOUBLE PRECISION      EDGES  ( 3, 3 )
      DOUBLE PRECISION      OBSMAT ( 3, 3 )
      DOUBLE PRECISION      POINTS ( 3, 3 )
      DOUBLE PRECISION      COSANG
      DOUBLE PRECISION      DIST
      DOUBLE PRECISION      NEAR
      DOUBLE PRECISION      VOXSIZ
      DOUBLE PRECISION      SCALE
      DOUBLE PRECISION      TEMP
      DOUBLE PRECISION      MAG
      DOUBLE PRECISION      TOL

      INTEGER               PLATID  ( MAXPLT )
      INTEGER               VOXLST  ( NVXLST )
      INTEGER               NPLATE
      INTEGER               I
      INTEGER               J
      INTEGER               NVXTOT
      INTEGER               NVXOUT
      INTEGER               NVX
      INTEGER               BCOUNT
      INTEGER               BODEX
      INTEGER               CBOD
      INTEGER               TEST
      INTEGER               TMP
      INTEGER               NVOX   ( 3 )
      INTEGER               QUAD   ( 3 )
      INTEGER               LEFT
      INTEGER               RIGHT
      INTEGER               MIDDLE
      INTEGER               PLANID
      INTEGER               PNTR
      
      LOGICAL               HITS
      LOGICAL               HAVE
      LOGICAL               INSIDE


C
C     Currently, the platelib system is restricted to
C     load only one kernel during any session.
C     Hardcode this fact. Were multiple plate kernels 
C     supported, the correct call to determine the number
C     of bodies with loaded plates models:
C
C      CALL PLTOT ( BCOUNT, TEMP1, TEMP2 )
C
      SAVE  BCOUNT
      DATA  BCOUNT  / 1 /

      DATA  RIGHT   / 0 /
      DATA  LEFT    / 1 /
      DATA  MIDDLE  / 2 /
 
C
C     Standard SPICE error handling.
C
      IF ( RETURN() ) THEN
         RETURN
      END IF
 
      CALL CHKIN ( 'PLBORE' )
 
C
C     Until we have better knowledge we assume there is no intersection
C
      BODY  =  0
      PLID  =  0
      FOUND = .FALSE.
      HAVE  = .FALSE.
      INSIDE= .TRUE.
      NEAR  = 9.D99
 
C
C     Obtain the unit vector of the ray from the observer.
C
      CALL VHAT ( BSIGHT, BDSITE )
 
C
C     For each body, check the coverage with the field of view.
C
      DO BODEX = 1, BCOUNT

C
C        Possibly, perform these geometry calculations in the
C        driver routine so they occur only once. However, ease
C        or use may require no change.
C

C
C        For each body, retrieve the voxel grid origin in model
C        units and calculate the farthest extent of the
C        voxel grid in voxel space.
C
         CALL VOXSUM ( BODEX, NVOX, VOXSIZ, VOXORI, NVXTOT, NVX )

C
C        The observer position in voxel space.
C
         CALL TOGRID ( POSOBS, VOXORI, VOXSIZ, ORIGIN )

C
C        The grid origin in voxel space lies at (0,0,0). Calculate
C        the farthest extent of the voxel grid in voxel space, i.e.
C        cast the grid dimension to double precision.
C
         VOXFAR(1) =  DBLE( NVOX(1) )
         VOXFAR(2) =  DBLE( NVOX(2) )
         VOXFAR(3) =  DBLE( NVOX(3) )

C
C        Determine the intersection between the BSIGHT vector
C        and the voxel grid using the Digital Differential 
C        Analyzer Method.
C
C        Find the candidate plane.
C
         DO I = 1, 3

            IF (  ORIGIN(I) .LT. 0.D0 ) THEN
            
               QUAD(I) = LEFT
               PLANE(I)= 0.D0
               INSIDE  = .FALSE.
            
            ELSE IF  ( ORIGIN(I) .GT. VOXFAR(I) ) THEN
            
               QUAD(I) = RIGHT
               PLANE(I)= VOXFAR(I)
               INSIDE  = .FALSE.

            ELSE

               QUAD(I) = MIDDLE

            END IF
            
         END DO

C
C        Check whether POSOBS exists within the voxel grid.
C
         IF ( .NOT. INSIDE ) THEN

C
C           Calculate T distances to candidate planes.
C
            DO I = 1, 3

               IF (  QUAD(I) .NE. MIDDLE .AND. BSIGHT(I) .NE. 0 ) THEN
            
                  MAXT(I) = (PLANE(I) - ORIGIN(I) )/BSIGHT(I)

               ELSE

                  MAXT(I) = -1.D0

               END IF
            
            END DO

C
C           Determine the largest of the MAXT for final choice of
C           intersection.
C
            PLANID = 1
         
            DO I = 2, 3

               IF ( MAXT(PLANID) .LT. MAXT(I) ) THEN
                  PLANID = I
               END IF

            END DO

C
C           Confirm the final candidate exists inside the grid.
C
            IF( MAXT(PLANID) .LT. 0.D0 ) THEN

               FOUND = .FALSE.

               CALL CHKOUT ( 'PLBORE' )
               RETURN

            END IF

            DO I = 1, 3

               IF ( PLANID .NE. I ) THEN

C
C                 Calculate the expected intersection coordinate
C                 with the voxel grid over each degree-of-freedom.
C
                  COORD(I) = ORIGIN(I) + MAXT(PLANID) * BSIGHT(I)
               
C
C                 If the calculated COORD is outside the box,
C                 a true intersection does not exist. Return
C                 FALSE.
C
                  IF( COORD(I) .LT. 0.D0
     .                .OR.
     .                COORD(I) .GT. VOXFAR(I) )
     .            THEN
     
                     FOUND = .FALSE.
               
                     CALL CHKOUT ( 'PLBORE' )
                     RETURN

                  END IF

               ELSE
            
                  COORD(I) = PLANE(I)
               
               END IF

            END DO

         ELSE

C
C           Observer point exists inside voxel grid, no
C           calculations needed.
C
            COORD(1) = ORIGIN(1)
            COORD(2) = ORIGIN(2)
            COORD(3) = ORIGIN(3)
            
         END IF

C
C        A tiny step down the boresight to overcome round-off 
C        problems. Such problems can occur if the value for
C        the intersection point exceeds the box bounds
C        due to round-off.
C
         MAG      = VNORM( BSIGHT )
         TOL      = 10D-4 
         COORD(1) = COORD(1) + ( BSIGHT(1)* TOL )/MAG
         COORD(2) = COORD(2) + ( BSIGHT(2)* TOL )/MAG
         COORD(3) = COORD(3) + ( BSIGHT(3)* TOL )/MAG

C
C        Need an error check on return size of VOXLST.
C
         CALL XDDA ( COORD, BSIGHT, NVOX, NVXLST, NVXOUT, VOXLST )

         PNTR = 1

C
C        Return a list of all plates contained in the voxels
C        in VOXLST.
C
         DO I=1, NVXOUT

            CALL VOXPLT( VOXLST(I), NPLATE, PLATID(PNTR) )

C
C           NPLATE returns zero or greater.
C              
            PNTR = PNTR + NPLATE

         END DO

C
C        So far we haven't done any of the initialization work
C        for BOUNDS for this body.  Look up the body identifier
C        from the index
C
         CALL DX2BID ( BODEX, CBOD )
 
C
C        If something went wrong up above there is no point in
C        going on from here.
C
         IF ( FAILED() ) THEN
            CALL CHKOUT ( 'PLBORE' )
            RETURN
         END IF
 
C
C        We are going to need to subtract the location of the observer
C        from vertices of a plate.  To speed things up a tiny bit,
C        we'll make 3 copies of the observer's location so that
C        we make a single subroutine call to handle the 3 subtractions.
C
         CALL VEQU ( POSOBS, OBSMAT(1,1) )
         CALL VEQU ( POSOBS, OBSMAT(1,2) )
         CALL VEQU ( POSOBS, OBSMAT(1,3) )
 
C
C        Now examine each plate in the PLATID list for this body. 
C        PNTR has the value of the index available for data in PLATID,
C        so the final location of data is at index PNTR - 1.
C
         DO I = 1, PNTR - 1

C
C           Retrive the appropriate plate.
C
            J = PLATID(I)

C
C           Get the J'th plate center and normal.
C
            CALL NTHPLC ( J, CENTR )
            CALL NTHPLN ( J, NORML )

C
C           Only if this plate is visible do we do anything else
C           with this plate. i.e. POSOBS exists in the halfspace
C           defined by by CENTR and NORML.
C
            IF ( PINHFS( POSOBS, CENTR, NORML) .EQ. 1 ) THEN

C
C              Compute the position of the observer relative to the
C              center of the plate. Optimize for speed.
C
               TOCENT(1) = POSOBS(1) - CENTR(1)
               TOCENT(2) = POSOBS(2) - CENTR(2)
               TOCENT(3) = POSOBS(3) - CENTR(3)

C
C              Retrieve the model coordinates of the J'th plate's  
C              three vertices. Compute the edges of the tetrahedral 
C              angle with the observer as the apex and the
C              vertices as members of the edge rays.  Finally
C              see if the boresight lies inside the tetrahedron.
C
               CALL NTHPLV ( J,          POINTS            )
               CALL VSUBG  ( POINTS,     OBSMAT, 9,  EDGES )
               CALL INSANG ( BDSITE,
     .                       EDGES(1,1), EDGES(1,2), EDGES(1,3),
     .                       HITS,       SCALE )
 
C
C              If the boresight intersects this plate, see if we already
C              had an intersection.  If so see if this plate is closer
C              than the last intersection we had.
C
               IF ( HITS .AND. HAVE ) THEN
 
C
C                 If this intersection point is closer than
C                 the last one, this plate must be in front.
C
                  IF      ( SCALE .LT. NEAR ) THEN
 
                     BODY   = CBOD
                     NEAR   = SCALE
                     COSANG = VDOT ( BDSITE, NORML )
 
                     CALL NTHPLI ( J, TMP, PLID )
 
                  ELSE IF ( SCALE .EQ. NEAR ) THEN
C
C                    In this case we must have an edge point.
C                    We pick the plate that whose normal direction
C                    is closest to anti-parallel to the boresight
C                    direction.
C
                     CALL VHAT ( NORML, NORML )
 
                     TEMP = VDOT ( BDSITE, NORML )
 
                     IF ( TEMP .LT. COSANG ) THEN

                        COSANG = TEMP
                        BODY   = CBOD
                        DIST   = VNORM ( TOCENT )
 
                        CALL NTHPLI ( J, TMP, PLID )
 
                     ELSE IF ( TEMP .EQ. COSANG ) THEN
C
C                       Hmmmm.  We've got two parallel plates that
C                       share an edge or vertex.  Pick the one
C                       with the closest center.
C
                        TEMP = VNORM ( TOCENT )
 
                        IF      ( TEMP .LT. DIST ) THEN
 
                           DIST = TEMP
                           CALL NTHPLI ( J, TMP, PLID )
 
                        ELSE IF ( TEMP .EQ. DIST ) THEN
C
C                          The centers of the plates are equidistant.
C                          That's the end of the geometric tests.
C                          We just take the one that had the lower
C                          plate ID number.
C
                           CALL NTHPLI ( J, TMP, TEST )
 
                           IF ( TEST .LT. PLID ) THEN
                              PLID = TEST
                              BODY = CBOD
                           END IF
 
                        END IF
 
                     END IF
 
                  END IF
 
               ELSE IF ( HITS .AND. .NOT. HAVE ) THEN
 
                  HAVE   = .TRUE.
                  NEAR   = SCALE
                  COSANG = VDOT ( BDSITE, NORML )
                  DIST   = VNORM( TOCENT )
                  BODY   = CBOD
 
                  CALL NTHPLI ( J, TMP, PLID )
  
               END IF
 
            END IF

C
C           Fetch the next plate for this body.
C
         END DO
 
C
C        We are done with the current body.  Do the same set of
C        steps for the next body in the list of bodies that have
C        plate models stored in the plate manager.
C
      END DO
 
C
C     We've done all of the tedious steps. If we have an intersection
C     put the information into FOUND.
C
      FOUND = HAVE
 
C
C     Compute intersection between the ray and plate, if the
C     intersection exists.
C
C         ______   ______        ______
C         XYZHIT = POSOBS + NEAR*BDSITE
C
      IF ( FOUND ) THEN
         CALL VLCOM ( 1.D0, POSOBS, NEAR, BDSITE, XYZHIT )
      END IF

C
C     That's all folks
C
      CALL CHKOUT ( 'PLBORE' )
      RETURN
 
      END
 