C$Procedure PLINFO ( plate model, return general information )
 
      SUBROUTINE PLINFO ( PLID, BODY, 
     .                    VIDS, VERTS, NORMAL, CENTER, ALBEDO  )
 
C$ Abstract
C
C     Return plate geometry and albedo information.
C
C$ Copyright
C
C     Copyright (1997), California Institute of Technology.
C     U.S. Government sponsorship acknowledged.
C
C$ Required_Reading
C
C     None.
C
C$ Keywords
C
C     PLATE
C
C$ Declarations
 
      IMPLICIT              NONE
 
      INTEGER               BODY
      INTEGER               PLID
      INTEGER               VIDS     ( 3 )
 
      DOUBLE PRECISION      ALBEDO
      DOUBLE PRECISION      NORMAL   ( 3 )
      DOUBLE PRECISION      CENTER   ( 3 )
      DOUBLE PRECISION      VERTS    ( 3, 3 )
 
 
C$ Brief_I/O
C
C     VARIABLE  I/O  DESCRIPTION
C     --------  ---  --------------------------------------------------
C     PLID       I   ID number of plate.
C     BODY       I   NAIF ID code of body.
C     VIDS       O   ID codes of the plate's vertices.
C     VERTS      O   XYZ coordinates of the plate's vertices.
C     NORMAL     O   Normal vector components of the plate.
C     CENTER     O   Center of plate's vector components.
C     ALBEDO     O   Plate's albedo.
C
C$ Detailed_Input
C
C     PLID       is the plate ID number.
C
C     BODY       is the NAIF ID code of the body.
C
C
C$ Detailed_Output
C
C     VIDS       are the ID codes for the plates vertices.
C
C     VERTS      are the body-fixed XYZ coordinates of plate's three
C                vertices, where (x1,y1,z1), (x2,y2,z2), (x3,y3,z3)
C                are stored in the order ((VERTS(I,J),I=1,3),J=1,3).
C
C     NORMAL     The outward-pointing normal vector of the plate in
C                the body-fixed frame.  The length of the vector
C                is equal to the area of the plate.
C
C     CENTER     The vector from the center of the body
C                to the center of the plate in body-fixed coordinates.
C
C     ALBEDO     The albedo of the plate.  If has not been determined,
C                the value returned is -1.D0, otherwise a value between
C                0.D0 and 1.D0 is returned.
C
C$ Parameters
C
C     None.
C
C$ Files
C
C     See: $Restrictions.
C
C$ Exceptions
C
C     1) If information required by this routine has not been
C        loaded prior to calling this routine, the omission will
C        be diagnosed by a routine in the call tree of this routine.
C
C$ Particulars
C
C     Given a plate ID and a NAIF body ID, this routine calls
C     routines in the plate manager and returns the information
C     stored there.
C
C$ Examples
C
C     The following code fragment determines the total surface area
C     of the plate model.
C
C   C
C   C     Load plate model.
C   C
C         CALL RDPLAT ( 'eros.plt' )
C
C   C
C   C     Obtain the number of plates in the Eros model.
C   C
C         CALL PLTNUM ( 2000433, NPLATE )
C
C   C     The SPICELIB routine 'VNORM' returns the length of
C   C     the vector 'NORMAL', which is set to the area of each plate.
C
C         AREA = 0.D0
C
C         DO I = 1, NPLATE
C            CALL PLINFO ( I, 2000433,
C        .                 VIDS, VERTS, NORMAL, CENTER, ALBEDO )
C            AREA = AREA + VNORM ( NORMAL )
C         END DO
C
C         WRITE (*,*) 'Surface area of model = ', AREA, ' KM^2'
C
C$ Restrictions
C
C     1) DB kernel(s) must be loaded containing plate model
C        data for the body specified by the input BODY.
C
C$ Author_and_Institution
C
C     J.A. Bytof      (JPL)
C
C$ Literature_References
C
C     None.
C
C$ Version
C
C-    SPICELIB Version 1.0.0, 20-JUN-1997 (JAB)
C
C
C-&
 
C$ Index_Entries
C
C     get vertex, normal, center and albedo information for a plate
C
C-&
 
C
C     Local variables
C
      INTEGER               PLDEX
      INTEGER               BODEX
      INTEGER               PIDOUT
 
C
C     Obtain the plate's body index and plate index numbers
C     from its plate ID and body NAIF ID.
C
 
      CALL BID2DX ( BODY,  BODEX         )
      CALL PLTIDX ( PLID,  BODEX,  PLDEX )
 
C
C     Obtain vertex ID's of the specified plate from the
C     plate manager.
C
 
      CALL NTHPVL ( PLDEX, PIDOUT, VIDS  )
 
C
C     Obtain the body-fixed coordinates of the plate's vertices.
C
 
      CALL NTHPLV ( PLDEX, VERTS )
 
C
C     Obtain the body-fixed components of the plate's normal vector.
C
 
      CALL NTHPLN ( PLDEX, NORMAL )
 
C
C     Obtain the body-fixed components of the plate's center.
C
 
      CALL NTHPLC ( PLDEX, CENTER )
 
C
C     Obtain the plate's albedo value.
C

      CALL NTHALB ( PLDEX, PIDOUT, ALBEDO )
 
      RETURN
      END
