C$Procedure      PLINVU_2 ( Plates in View After Offset Rotation )
 
      SUBROUTINE PLINVU_2 ( INSTID, OFFROT, ET,     ROOM,   MAXBDS,
     .                      NBODS,  BODIES, BOUNDS, INFOV,
     .                      PLIDS,  VISAMT, LITAMT                 )
 
C$ Abstract
C
C     Determine a list of plates that lie in the field of view of
C     an instrument after an offset rotation is applied.
C
C$ Copyright
C
C     Copyright (2000), California Institute of Technology.
C     U.S. Government sponsorship acknowledged.
C
C$ Required_Reading
C
C      None.
C
C$ Keywords
C
C      SHAPES
C
C$ Declarations
 
      IMPLICIT NONE
 
      INCLUDE               'pltmax.inc'
 
      INTEGER               INSTID
      DOUBLE PRECISION      ET
      INTEGER               ROOM
      INTEGER               MAXBDS
      INTEGER               NBODS
      INTEGER               BODIES  ( * )
      INTEGER               BOUNDS  ( 2, * )
      INTEGER               INFOV
      INTEGER               PLIDS   ( * )
      DOUBLE PRECISION      VISAMT  ( * )
      DOUBLE PRECISION      LITAMT  ( * )
      DOUBLE PRECISION      OFFROT  ( 3, 3 )
 
C$ Brief_I/O
C
C     VARIABLE  I/O  DESCRIPTION
C     --------  ---  --------------------------------------------------
C     INSTID     I   id-code for some instrument
C     OFFROT     I   offset rotation matrix
C     ET         I   epoch at which to locate plates in the FOV
C     ROOM       I   maximum number of plate ids that can be returned
C     MAXBDS     I   maximum number of bodies that can be returned
C     INFOV      O   total number of plates inside the field of view
C     NBODS      O   total number of bodies that have plates in FOV
C     BODIES     O   a list of body ids for bodies in the FOV
C     BOUNDS     O   start and stop list of plates for each body
C     PLIDS      O   plate ids for plates in the field of view
C     VISAMT     O   the amount of a plate visible in the FOV
C     LITAMT     O   the amount of a plate that is illuminated
C
C$ Detailed_Input
C
C     INSTID     is the id-code for some instrument. The calling
C                program must load and instrument kernel for this
C                instrument specifying the field of view of the
C                instrument and its orientation relative to some
C                reference frame.
C
C     OFFROT     offset rotation to be applied in the instrument frame.
C
C     ET         is an epoch expressed in ephemeris seconds past
C                the J2000 epoch (TDB) at which the user wishes
C                to determine a collection of plates in the field
C                of view of the specified instrument.
C
C     ROOM       is the maximum number of plate ids that can be
C                returned.
C
C                It is recommended that the user determine this
C                value from the NAIF include file 'pltmax.inc'.
C
C     MAXBDS     is the maximum number of bodies (having modeled
C                surfaces) that may appear in the field of view.
C
C                It is recommended that the user determine this
C                value from the NAIF include file 'pltmax.inc'.
C
C
C$ Detailed_Output
C
C     NBODS      Number of different bodies having plates in the
C                field of view of the specified instrument.
C
C     BODIES     is a list of body ids for bodies having one or more
C                surface plates that intersect the field of view
C                and whose normals point "toward" the instrument.
C
C     BOUNDS     is a list of start and stop indexes for the plates
C                associated with a given body that lie in the field
C                of view of the specified instrument.
C
C                The plate identifiers for the body with id-code
C                BODIES(I) (I <= NBODS) are contained in the array PLIDS
C
C                  from PLIDS( BOUNDS(1,I) )
C                  to   PLIDS( BOUNDS(2,I) )
C
C     INFOV      is the umber of plates found in the field of
C                view.  Plates whose normals are pointed
C                away from the observer are not included in
C                the list.
C
C     PLIDS      is a list of plate identifiers for plates that lie
C                in the field of view of the specified instrument.
C
C     VISAMT     is the fraction (0.0 to 1.0) of a plate's area which
C                is visible to the observer.  A value of exactly 0.0
C                could mean one of three things: the plate faces
C                away from the observer, or it is not in the field of
C                view, or it is completely occluded by other plates.
C
C     LITAMT     is the fraction (0.0 to 1.0) of a plate's visible
C                area that is illuminated, accounting for solar position
C                and shadowing by other plates. If a plate faces away
C                from the observer, or is completely outside the FOV,
C                -1.0 is returned.  If shadow processing is turned off,
C                -9.0 is returned.
C
C
C$ Parameters
C
C     None.
C
C$ Files
C
C     None.
C
C$ Exceptions
C
C     1) If the "shape" of the field of view for the specified
C        instrument is not polygonal or circular, the error
C        'SPICE(UNSUPPORTEDFOV)' will be signalled.
C
C     2) If insufficient room is provided for the array BODIES to
C        list all of the bodies that intersect the field of view
C        of the instrument, the error 'SPICE(OUTOFROOM)' is
C        signalled.
C
C     3) If insufficient room is provided for the array PLIDS to
C        list all of the plates that intersect the field of view,
C        the error 'SPICE(OUTOFROOM)' is signalled.
C
C     4) If a required kernel (see Particulars) below cannot be located
C        an error will be diagnosed by one of the routines called
C        by this routine.
C
C$ Particulars
C
C     Given a set of objects whose surfaces are modeled by
C     a set of covering triangular plates, this routine returns
C     a list of plates from these objects that lie within the
C     field of view of an instrument at a particular epoch.
C
C     A substantial amount of preparation is required in the
C     users program before this routine can be called.
C
C     The user must "load"
C
C        1) Instrument kernels for the instruments of interest
C        2) C-kernels that relate the orientation of the instrument
C           mounting platform to some other reference frame
C        3) shape models for all objects of interest
C        4) spk files for: the objects on which the observing
C           instruments are mounted; the objects with loaded
C           shape models; the sun.
C        5) PCK files that give the orientation of each of the
C           objects whose shapes are modeled by triangular tilings
C        6) An SCLK kernel
C        7) a "frames" kernel that connects the bodyfixed frames
C           of the various modeled surfaces to the appropriate
C           PCK objects.
C
C     Once these preparations have been performed this routine will
C     return a complete list of plates from all modeled surfaces
C     that lie within the field of view of the specified instrument.
C
C     Moreover, the extent that the parallel arrays VISAMT and
C     LITAMT will contain the fraction of each plate (in terms of
C     area) that lies in the field of view and how much of that
C     plate is illuminated by the sun.
C
C     At the moment, this routine does not solve for obscuration
C     or shadowing of one plate by another.  Moreover, the current
C     version of this routine models circular fields of view
C     as regular 16-sided polygons.
C
C$ Examples
C
C     When time permits.
C
C$ Restrictions
C
C     None.
C
C$ Author_and_Institution
C
C     J.A. Bytof      (JPL)
C     W.L. Taber      (JPL)
C
C$ Literature_References
C
C     None.
C
C$ Version
C
C-    Version 2.0.3, 17-JAN-2000 (JAB)
C
C        Before calling PNTOCC, copy Sun position
C        into a three dimensional array.
C
C-    Version 2.0.2, 20-OCT-1999 (JAB)
C
C        Fixed output array indexing error.
C
C        Added IF-test filter for plates facing away from
C        observer.
C
C        Corrected header comments describing output arrays.
C
C        Substituted a big chunk of code with a call to INITFV
C        (field of view initialization).
C
C-    Version 2.0.1, 25-FEB-1999 (JAB)
C
C        Based on PLINVU, added offset rotation capability.
C
C-    Version 2.0.0, 03-FEB-1999 (JAB)
C
C        Incorporated occlusion processing.
C
C-    Beta Version 1.2.0, 03-SEP-1998 (WLT)
C
C        Irina Mikheeva of LPL at the University of Arizona identified
C        an error in the previous version of this routine. In that
C        version, the transformation from the instrument frame to the
C        bodyfixed frame was not performed correctly in the case that
C        the frame associated with the user specified BODY could be
C        identified.  The code has been updated so that the
C        computation of this frame in different branches of the
C        routine are now equivalent.
C
C        In particular in the code below, the three lines
C
C           CALL SXFORM ( FRAME,  'J2000', ET,      FRM2J2 )
C           CALL SXFORM ( 'J2000', FRNAME, ET-LT,   TSIPM  )
C           CALL MXMG   ( TSIPM,   FRM2J2, 6, 6, 6, XFORM )
C
C        have replaced the erroneous line
C
C           CALL SXFORM ( FRAME, FRNAME, ET-LT, XFORM )
C
C        of the previous edition of the routine.
C
C-    Beta Version 1.1.0, 17-JUN-1997 (WLT)
C
C        Fixed the call to CIDFRM (it was OIDFRM before I fixed it).
C
C-    Beta Version 1.0.0, 04-JUN-1997 (WLT)
C
C
C-&
 
C$ Index_Entries
C
C     portion of an object in a field of view
C     plates in field of view
C
C-&
C
C     SPICELIB Functions
C
 
 
      LOGICAL               RETURN
      LOGICAL               FAILED
 
C
C     Parameters
C
 
      INTEGER               LBPOOL
      PARAMETER           ( LBPOOL = -5 )
 
      INTEGER               LNGEST
      PARAMETER           ( LNGEST = 50 )
 
      INTEGER               PTHSIZ
      PARAMETER           ( PTHSIZ = LNGEST + 2 )
 
 
      INTEGER               FVROOM
      PARAMETER           ( FVROOM = 40 )
 
      INTEGER               VTXSIZ
      PARAMETER           ( VTXSIZ = 3*LNGEST )
 
      INTEGER               WDSIZE
      PARAMETER           ( WDSIZE = 32 )
 
      INTEGER               SUN
      PARAMETER           ( SUN    = 10 )
 
      DOUBLE PRECISION      NSIDES
      PARAMETER           ( NSIDES = 16.0D0 )
 
C
C     Local Variables
C
 
      CHARACTER*(WDSIZE)    FRAME
      CHARACTER*(WDSIZE)    FRNAME
 
      DOUBLE PRECISION      BDSITE ( 3 )
      DOUBLE PRECISION      BSIGHT ( 3 )
      DOUBLE PRECISION      EDGE   ( 3, FVROOM )
      DOUBLE PRECISION      FOV    ( 3, FVROOM )
      DOUBLE PRECISION      FOVNRM ( 3, FVROOM )
      DOUBLE PRECISION      SIDE   ( 3, FVROOM )
      DOUBLE PRECISION      FOVFRC
      DOUBLE PRECISION      FRM2J2 ( 6, 6 )
      DOUBLE PRECISION      LITFRC
      DOUBLE PRECISION      LT
      DOUBLE PRECISION      LTS
      DOUBLE PRECISION      POSOBS ( 3 )
      DOUBLE PRECISION      ROT    ( 3, 3 )
      DOUBLE PRECISION      SOLAR  ( 6 )
      DOUBLE PRECISION      STATE  ( 6 )
      DOUBLE PRECISION      STEMP  ( 6 )
      DOUBLE PRECISION      SUNDIR ( 3 )
      DOUBLE PRECISION      TSIPM  ( 6, 6 )
      DOUBLE PRECISION      VERTXS ( VTXSIZ )
      DOUBLE PRECISION      VISFRC
      DOUBLE PRECISION      XFORM  ( 6, 6 )
 
      INTEGER               BCOUNT
      INTEGER               BODEX
      INTEGER               BODY
      INTEGER               FOVSIZ
      INTEGER               FRCODE
      INTEGER               I
      INTEGER               J
      INTEGER               NPLATE
      INTEGER               OBS
      INTEGER               PATHS ( 2, LBPOOL:PTHSIZ )
      INTEGER               PLATES
      INTEGER               PLTID
      INTEGER               TMP
      INTEGER               USED
      INTEGER               VERTS
      INTEGER               FSTIDX
      INTEGER               LSTIDX
      INTEGER               NVERT
      INTEGER               PRANGE   ( 2 )
      INTEGER               VRANGE   ( 2 )
 
      LOGICAL               AWAY
      LOGICAL               DOINIT
      LOGICAL               FOUND
 
C
C     Standard SPICE error handling.
C
      IF ( RETURN() ) THEN
         RETURN
      END IF
 
      CALL CHKIN ( 'PLINVU_2' )
 
C
C     Now handle some of the initializations.  We haven't used
C     any of the output arrays yet and there are currently no
C     bodies whose plates are known to be in the field of view.
C
      USED  = 0
      INFOV = 0
      NBODS = 0
 
      CALL PTHINI ( 3, LNGEST, PTHSIZ, PATHS, VERTXS )
 
C
C     Get the observer ID associated with this instrument
C
      OBS = INSTID / 1000
 
C
C     Get FOV vectors and normals.
C
 
      CALL INITFV ( INSTID, FVROOM, FRAME, BSIGHT, FOVSIZ, FOV, FOVNRM )
 
C
C     Determine the number of bodies that have plates
C     models loaded.
C
      CALL PLTOT ( BCOUNT, PLATES, VERTS )
 
C
C     For each body, check the coverage with the field of view.
C
      DO BODEX = 1, BCOUNT
C
C        So far we haven't done any of the initialization work
C        for BOUNDS for this body.  If any plates for this body
C        are discovered to be in the field of view we'll do
C        the initialization work then.  Right now simply note that
C        it needs to be done.
C
         DOINIT = .TRUE.
C
C        Look up the body index
C
         CALL DX2BID ( BODEX, BODY )
 
C
C        See if we can discern the frame name associated with this
C        body.
C
         CALL CIDFRM ( BODY, FRCODE, FRNAME, FOUND )
 
         IF ( FOUND ) THEN
C
C           We can do this the easy (and less error prone way). We
C           need the state of the observer in the apparent bodyfixed
C           frame of the body.  We also need the state of the sun
C           in the apparent body fixed frame relative to the body
C           corrected for light time to the body from the observer.
C
            CALL SPKEZ ( BODY, ET,    FRNAME, 'LT+S', OBS,  STATE, LT  )
            CALL SPKEZ ( SUN,  ET-LT, FRNAME, 'LT+S', BODY, SOLAR, LTS )
 
            CALL VMINUS( STATE, POSOBS )
C
C           Get the transformation that transforms vectors in FRAME
C           to the bodyfixed frame of this body.
C
            CALL SXFORM ( FRAME,  'J2000', ET,      FRM2J2 )
            CALL SXFORM ( 'J2000', FRNAME, ET-LT,   TSIPM  )
            CALL MXMG   ( TSIPM,   FRM2J2, 6, 6, 6, XFORM )
 
         ELSE
C
C           We don't have the bodyfixed frame identification loaded.
C           We'll try to do this the old-fashioned way. First get the
C           state of the body relative to the observer and the state
C           of the sun relative to the body in the J2000 frame
C           with appropriate corrections made for light time.
C
            CALL SPKEZ( BODY, ET,    'J2000', 'LT+S', OBS,  STATE, LT  )
            CALL SPKEZ( SUN,  ET-LT, 'J2000', 'LT+S', BODY, STEMP, LTS )
C
C           Construct the transformation from the FRAME of the
C           instrument to the bodyfixed frame.
C
            CALL SXFORM ( FRAME,  'J2000', ET,      FRM2J2 )
            CALL TISBOD ( 'J2000', BODY,   ET-LT,   TSIPM  )
            CALL MXMG   ( TSIPM,   FRM2J2, 6, 6, 6, XFORM )
C
C           Transform the state of the sun into the bodyfixed frame.
C           And get the apparent state of the observer in the
C           bodyfixed frame.
C
            CALL MXVG   ( TSIPM, STEMP, 6, 6, SOLAR )
            CALL MXVG   ( TSIPM, STATE, 6, 6, STEMP )
            CALL VMINUS ( STEMP, POSOBS )
 
         END IF
C
C        If something went wrong up above there is no point in
C        going on from here.
C
         IF ( FAILED() ) THEN
            CALL CHKOUT ( 'PLINVU_2' )
            RETURN
         END IF
 
C
C        Get the rotation component of the frame transformation
C
         DO I = 1, 3
            DO J = 1, 3
               ROT (I,J) = XFORM(I,J)
            END DO
         END DO
C
C        Rotate all of the FOV components into the bodyfixed frame.
C
         CALL MXV ( ROT, BSIGHT, BDSITE )
 
         DO I = 1, FOVSIZ
            CALL MXV ( ROT, EDGE(1,I), FOV(1,I) )
         END DO
 
C
C        Obtain the outward normal vector for each FOV plane.
C
 
         DO I = 1, FOVSIZ-1
            CALL VSUB  ( FOV(1,I+1), FOV(1,I),  SIDE(1,I)   )
            CALL UCRSS ( FOV(1,I),   SIDE(1,I), FOVNRM(1,I) )
         END DO
 
         CALL VSUB  ( FOV(1,FOVSIZ), FOV(1,1),       SIDE(1,FOVSIZ)   )
         CALL UCRSS ( FOV(1,FOVSIZ), SIDE(1,FOVSIZ), FOVNRM(1,FOVSIZ) )
 
C
C        Now examine each plate for this body.
C
         CALL PLTBND ( BODEX, NPLATE, PRANGE, NVERT, VRANGE )
 
         FSTIDX = PRANGE(1)
         LSTIDX = PRANGE(2)
 
         DO I = 1, 3
            SUNDIR(I) = SOLAR(I)
         END DO
 
         DO J = FSTIDX, LSTIDX
 
            CALL PNTOCC ( J,
     .                    FOVSIZ,  FOV,    FOVNRM,
     .                    POSOBS,  SUNDIR,
     .                    FOVFRC,  VISFRC, LITFRC,
     .                    AWAY                       )
 
 
C
C           Only include plates that are facing towards
C           the observer.
C
            IF ( .NOT. AWAY ) THEN
 
C
C              Check if there's room in the output arrays.
C
 
               IF ( FOVFRC .GT. 0.D0 ) THEN
 
                  USED = USED + 1
 
                  IF ( USED .GE. ROOM ) THEN
 
                     CALL SETMSG ( 'Sorry, but you''ve used up '
     .               //            'all of the room in the '
     .               //            'output array before we could '
     .               //            'output all of the plates in '
     .               //            'view.  The amount of room '
     .               //            'supplied for output must be '
     .               //            'increased.  The current '
     .               //            'number of slots provided is '
     .               //            '#. See the header of PLINVU_2 '
     .               //            'for instructions on '
     .               //            'selecting an appropriate '
     .               //            'size for the output arrays. ' )
 
                     CALL ERRINT ( '#', ROOM )
                     CALL SIGERR ( 'SPICE(OUTOFROOM)'  )
                     CALL CHKOUT ( 'PLINVU_2' )
                     RETURN
 
                  END IF
 
C
C                 If we've already added this body onto the list
C                 of output bodies, we can just append things to
C                 the current list.
C
                  CALL NTHPLI ( J, TMP, PLTID )
 
                  IF ( DOINIT ) THEN
C
C                    Make sure that there's room for this body
C                    in the list of bodies.
C
                     IF ( NBODS .GE. MAXBDS ) THEN
 
                        CALL SETMSG ( 'There are at least # '
     .                  //            'bodies that have plates '
     .                  //            'present in the field of '
     .                  //            'view of instrument #. '
     .                  //            'However, the number of '
     .                  //            'bodies for which space '
     .                  //            'has been provided by the '
     .                  //            'caller of PLINVU is #. '
     .                  //            'See the header for PLINVU '
     .                  //            'for a discussion of the '
     .                  //            'space required for the '
     .                  //            'outputs of PLINVU. ' )
 
                        CALL ERRINT ( '#', NBODS + 1 )
                        CALL ERRINT ( '#', MAXBDS    )
                        CALL SIGERR ( 'SPICE(OUTOFROOM)'  )
                        CALL CHKOUT ( 'PLINVU_2' )
                        RETURN
 
                     END IF
 
C
C                 We won't need to do the initialization work
C                 for this body again.  Set DOINIT to note this.
C
                     DOINIT          = .FALSE.
 
                     NBODS           = NBODS + 1
                     BODIES(  NBODS) = BODY
                     BOUNDS(1,NBODS) = USED
 
                  END IF
 
                  BOUNDS(2,NBODS) = USED
                  PLIDS (USED)    = PLTID
                  LITAMT(USED)    = LITFRC
                  VISAMT(USED)    = VISFRC
 
               END IF
 
 
            END IF
C
C     Fetch the next plate for this body.
C
 
         END DO
 
C
C        We are done with the current body.  Do the same set of
C        steps for the next body in the list of bodies that have
C        plate models stored in the plate manager.
C
 
      END DO
 
C
C     We've done all of the tedious steps.  One last thing to do:
C     note the total number of plates that showed up in the field
C     of view.
C
 
      INFOV = USED
 
C
C     That's all folks
C
 
      CALL CHKOUT ( 'PLINVU_2' )
      RETURN
 
      END
