C$Procedure PLNORM ( Plate Normal)
 
      SUBROUTINE PLNORM ( P, NORMAL )
 
C$ Abstract
C
C    Compute the normal vector of a triangular "plate" in 3-space.
C
C$ Copyright
C
C     Copyright (1997), California Institute of Technology.
C     U.S. Government sponsorship acknowledged.
C
C$ Required_Reading
C
C     None.
C
C$ Keywords
C
C     GEOMETRY
C
C$ Declarations
 
      IMPLICIT              NONE
 
      DOUBLE PRECISION      P         ( 3, 3 )
      DOUBLE PRECISION      NORMAL    ( 3 )
 
C$ Brief_I/O
C
C     VARIABLE  I/O  DESCRIPTION
C     --------  ---  --------------------------------------------------
C     P          I   The vertices of a triangle in 3-space.
C     NORMAL     O   The components of the normal of the triangle
C
C
C$ Detailed_Input
C
C     P          is an array containing the coordinates of the
C                vertices of a triangle in 3-space. The coordinates
C                of the I'th vertex are given by the triple:
C
C                 (  P(1,I), P(2,I), P(3,I)  )
C
C$ Detailed_Output
C
C     NORMAL     is the normal vector of a triangular plate
C                based on the cross product of the vector from
C                the first vertex to the second with the vector
C                from the second vertex to the third.
C
C$ Parameters
C
C     None.
C
C$ Files
C
C     None.
C
C$ Exceptions
C
C     Error free.
C
C$ Particulars
C
C     This routine computes the normal vector to a triangular plate
C     from the cross product of the two vectors defined by the
C     vertices.  The length of the returned normal vector is
C     scaled to be equal in value to the area of the plate.
C
C$ Examples
C
C     Suppose you need to compute the normal vector of a triangle
C     in 3-space.  Here's how you could use this routine to
C     accomplish the task.  (Note that unless you're vertices
C     are automatically loaded in to the array P, this is
C     probably more trouble than it's worth.)
C
C
C        P(1,1) = x-coordinate of first vertex
C        P(2,1) = y-coordinate of first vertex
C        P(3,1) = z-coordinate of first vertex
C
C        P(1,2) = x-coordinate of second vertex
C        P(2,2) = y-coordinate of second vertex
C        P(3,2) = z-coordinate of second vertex
C
C        P(1,3) = x-coordinate of third vertex
C        P(2,3) = y-coordinate of third vertex
C        P(3,3) = z-coordinate of third vertex
C
C     Now to get the normal vector just call PLNORM
C
C        CALL PLNORM ( P, NORMAL )
C
C
C$ Restrictions
C
C     None.
C
C$ Author_and_Institution
C
C     J.A. Bytof      (JPL)
C     W.L. Taber      (JPL)
C
C$ Literature_References
C
C     None.
C
C$ Version
C
C-    SPICELIB Version 1.0.0, 20-JUN-1997 (JAB)
C
C
C-&
 
C$ Index_Entries
C
C     Compute the normal vector of a triangle
C
C-&
C
C     Local Variables
C
      INTEGER               I
 
      DOUBLE PRECISION      V1 ( 3 )
      DOUBLE PRECISION      V2 ( 3 )
      DOUBLE PRECISION      V3 ( 3 )
 
C
C     Form vectors V1 and V2, which follow two sides of the triangle.
C
 
      DO I = 1, 3
         V1(I) = P(I,2) - P(I,1)
         V2(I) = P(I,3) - P(I,2)
      END DO
 
C
C     Determine the cross-product vector of V1 and V2.
C
      CALL VCRSS ( V1, V2, V3 )
 
C
C     Multiply the length of V3 by one-half, making the
C     length of the normal vector to the plate equal to its area.
C
      CALL VSCL  ( 0.5D0, V3, NORMAL )
 
      RETURN
      END
