C$Procedure      PLSANG ( plate solar angle )
 
      SUBROUTINE PLSANG ( PLID,  BODY,   OBS,   ET, CORR,
     .                    ANGLE, VISAMT, LITAMT          )
 
C$ Abstract
C
C     Return the angle between the vector to the sun and the
C     vector to an observer from the center of a given plate.
C
C$ Copyright
C
C     Copyright (1999), California Institute of Technology.
C     U.S. Government sponsorship acknowledged.
C
C$ Required_Reading
C
C     None.
C
C$ Keywords
C
C     PLATE
C
C$ Declarations
 
      IMPLICIT NONE
 
      INTEGER               PLID
      INTEGER               BODY
      INTEGER               OBS
 
      DOUBLE PRECISION      ET
 
      CHARACTER*(*)         CORR
 
      DOUBLE PRECISION      ANGLE
      DOUBLE PRECISION      VISAMT
      DOUBLE PRECISION      LITAMT
 
C$ Brief_I/O
C
C     VARIABLE  I/O  DESCRIPTION
C     --------  ---  --------------------------------------------------
C     PLID       I   ID code for plate of interest.
C     BODY       I   ID code for the body associated with the plate.
C     OBS        I   ID code of the observer.
C     ET         I   epoch in seconds past the J2000 epoch.
C     CORR       I   corrections to apply due to finite speed of light.
C     ANGLE      O   angle between sun and observer from plate center.
C     VISAMT     O   fraction of plate area visible from the observer.
C     LITAMT     O   fraction of visible plate area illuminated
C                    by the Sun,
C
C$ Detailed_Input
C
C     PLID       is the plate ID code.
C
C     BODY       is the NAIF ID of the body associated with the plate
C                specified by PLID.
C
C     OBS        is the NAIF ID of the observer.
C
C     ET         is the epoch (ephemeris time) at which the
C                angle is computed.
C
C     CORR       is a flag that indicates whether or not light-time
C                and stellar aberration should be applied in the
C                computation of the output angle.  Allowed values
C                are: 'NONE', and 'ALL'.  (This list may be expanded
C                in future releases of this routine.)  The case of
C                the letters in CORR is not significant. ('None' and
C                'NONE' are regarded as equivalent.)
C
C                If the value of CORR is 'NONE', all directions computed
C                are geometric.  If the value of CORR is 'ALL'
C                light-time and stellar aberration correction are
C                are applied in all computations.
C
C                For most users, the value 'NONE' will be suitable
C                for computing the retrieved angle.  The formal error
C                made as a result of using 'NONE' instead of all will
C                typically be on the order of 0.01 degrees.
C
C$ Detailed_Output
C
C     ANGLE      is the angle in radians between the vector from
C                the plate center to the sun and the vector from
C                the plate center to the observer.  Note that
C                light-time or stellar aberrations are applied
C                based upon the value of CORR.
C
C     VISAMT     is the fraction of the plate area that is visible
C                from the observer; ranges from 0.0 to 1.0.
C
C     LITAMT     is the fraction of the visible plate area that is
C                illuminated by sun; ranges from 0.0 to 1.0.  If
C                the plate can't be seen at all from the viewpoint,
C                LITAMT is set to 0.0.
C
C$ Parameters
C
C     None.
C
C$ Files
C
C     See: $Restrictions.
C
C$ Exceptions
C
C     1) If the value of CORR is not one of the recognized strings,
C        the error 'SPICE(BADCORRSPEC)' will be signalled.
C
C     2) If information required by this routine has not been
C        loaded prior to calling this routine, the omission will
C        be diagnosed by a routine in the call tree of this routine.
C
C$ Particulars
C
C      Given the body id and plate id for some plate in a triangulation
C      of the surface of some object, this routine returns the angle
C      between the direction to the sun and the direction to an observer
C      as seen from the center of the plate.
C
C$ Examples
C
C     The following code fragment illustrates the use of this routine
C     to find the plate in a shape model in which the angle between
C     the sun and NEAR spacecraft is minimized. (We assume that the
C     set of plate identifiers is a sequence of consecutive integers
C     starting at 1.)
C
C   C
C   C     Load plate model.
C   C
C         CALL RDPLAT ( 'eros.plate.dbk' )
C
C   C
C   C     Load ephemerides.
C   C
C         CALL SPKLEF ( 'eros.bsp',  HAND1 )
C         CALL SPKLEF ( 'near.bsp',  HAND2 )
C         CALL SPKLEF ( 'de403.bsp', HAND3 )
C
C   C
C   C     Load asteroid attitude file.
C   C
C         CALL PCKLOF ( 'astatt.bpc', HAND4 )
C
C   C
C   C     Get the number of plates in the Eros model.
C   C
C         CALL PLSIZE ( NBODS,  TOTPLT, TOTVRT, BDLIST,
C        .              BDPNUM, BDPBND, BDVNUM, BDVBND )
C
C         EROS   = 2000433
C         EROSDX = ISRCHI ( EROS, NBODS, BDLIST )
C         NPLATE = BDPNUM ( EROSDX )
C
C         Set an initial (out of range) minimum value for the angle.
C
C         MINANG = 10.0D0
C
C         DO PLID = 1, NPLATE
C
C            CALL PLSANG ( PLID,  EROS, -93, ET, CORR, ...
C        .                 ANGLE, VISAMT, LITAMT )
C
C            IF ( VISAMT .GT. 0.0D0 .AND. ANGLE .LT. MINANG ) THEN
C               MINAT = PLID
C            END IF
C
C         END DO
C
C         WRITE (*,*) 'The angle between the sun and spacecraft is '
C         WRITE (*,*) 'smallest for plate: ', MINAT
C
C$ Restrictions
C
C     1) DB kernel(s) must be loaded containing plate model data
C        for the body specified by the input BODY..
C
C     2) SP kernel(s) must be loaded containing ephemeris data
C        for the plate model body or bodies, the target body, the
C        Sun and possibly plate model satellites, spacecraft, and
C        intermediate bodies.
C
C     3) PC kernel(s) must be loaded containing orientation data
C        for the body specified by the input BODY.
C
C$ Author_and_Institution
C
C     J.A. Bytof      (JPL)
C     W.L. Taber      (JPL)
C
C$ Literature_References
C
C     None.
C
C$ Version
C
C-    SPICELIB Version 2.0.0, 07-DEC-1998 (JAB)
C
C        Added capability to determine viewpoint occlusion
C        and shadowing ofthe plate by other plates.
C
C-    SPICELIB Version 1.0.0, 19-JUN-1997 (WLT)
C
C-&
 
C$ Index_Entries
C
C     angle between sun and observer from plate center
C
C-&
 
C$ Revisions
C
C-&
 
C
C     SPICELIB functions
C
 
      LOGICAL               RETURN
      DOUBLE PRECISION      VSEP
 
C
C     Local variables
C
 
      INTEGER               WDSIZE
      PARAMETER           ( WDSIZE = 32 )
 
      INTEGER               FVROOM
      PARAMETER           ( FVROOM = 1 )
 
 
      CHARACTER*(WDSIZE)    MYCORR
 
      INTEGER               BODEX
      INTEGER               PLDEX
      INTEGER               FOVSIZ
 
      LOGICAL               AWAY
 
      DOUBLE PRECISION      CENTER ( 6 )
      DOUBLE PRECISION      CNTINR ( 6 )
 
      DOUBLE PRECISION      LT
      DOUBLE PRECISION      LTS
      DOUBLE PRECISION      FOV    ( 3, FVROOM )
      DOUBLE PRECISION      FOVNRM ( 3, FVROOM )
      DOUBLE PRECISION      FOVFRC
      DOUBLE PRECISION      LITFRC
      DOUBLE PRECISION      VISFRC
      DOUBLE PRECISION      NORMAL ( 3 )
      DOUBLE PRECISION      OBSCNT ( 6 )
      DOUBLE PRECISION      OBSPOS ( 6 )
      DOUBLE PRECISION      SUNPOS ( 6 )
      DOUBLE PRECISION      SCNOBS ( 6 )
      DOUBLE PRECISION      SCNSSB ( 6 )
      DOUBLE PRECISION      SSBBOD ( 6 )
      DOUBLE PRECISION      SSBOBS ( 6 )
      DOUBLE PRECISION      STBOD  ( 6 )
      DOUBLE PRECISION      STOBS  ( 6 )
      DOUBLE PRECISION      STSUN  ( 6 )
      DOUBLE PRECISION      TIPM   ( 3, 3 )
      DOUBLE PRECISION      TOBDFX ( 6, 6 )
      DOUBLE PRECISION      TOINRT ( 6, 6 )
 
C
C     Standard SPICE error handling.
C
 
      IF ( RETURN () ) THEN
         RETURN
      ELSE
         CALL CHKIN ( 'PLSANG' )
      END IF
C
C     See what kind of corrections should be applied.
C
      CALL LJUST ( CORR,   MYCORR )
      CALL UCASE ( MYCORR, MYCORR )
 
      IF (       MYCORR .NE. 'ALL'
     .     .AND. MYCORR .NE. 'NONE' )  THEN
 
         CALL SETMSG ( 'The corrections to apply in the '
     .   //            'computations performed by PLSANG must '
     .   //            'be ''ALL'' or ''NONE''.  The '
     .   //            'corrections specified were ''#''. ' )
 
         CALL ERRCH  ( '#', CORR )
         CALL SIGERR ( 'SPICE(BADCORRSPEC)'  )
         CALL CHKOUT ( 'PLSANG' )
         RETURN
 
      END IF
 
C
C     Get body index of target, given its NAIF ID.
C
 
      CALL BID2DX ( BODY, BODEX )
 
C
C     Get the plate index for this plate.
C
 
      CALL PLTIDX ( PLID, BODEX, PLDEX )
 
C
C     Get the outward pointing normal vector of the requested
C     plate in plate model coordinates.
C
 
      CALL NTHPLN ( PLDEX, NORMAL )
 
C
C     Get the position of the center of the plate in plate
C     model coordinates.
C
      CALL NTHPLC ( PLDEX, CENTER )
 
C
C     We either are going to apply every correction we can think
C     of or we're just going to take the quick and dirty approach.
C
      IF ( MYCORR .EQ. 'ALL' ) THEN
C
C        This is the more involved case.  We've got a lot of light-time
C        and aberration corrections to apply.  Fortunately, SPICELIB
C        has a big bag of routines well suited to this.
C
C        First of all the bodyfixed velocity of the plate center is
C        zero.
C
         CENTER(4) = 0.0D0
         CENTER(5) = 0.0D0
         CENTER(6) = 0.0D0
C
C        Next get the light time corrected position of the center
C        of the body.
C
         CALL SPKEZ  ( BODY, ET, 'J2000',  'LT', OBS,  STBOD, LT )
C
C        Get the state of the observer and body relative to the solar
C        system barycenter.  The BODY state should be corrected for
C        the light time we just computed.
C
         CALL SPKSSB ( OBS,  ET,    'J2000', SSBOBS )
         CALL SPKSSB ( BODY, ET-LT, 'J2000', SSBBOD )
C
C        Get the light time corrected state transformation matrix from
C        inertial to bodyfixed coordinates and invert the resulting
C        matrix.
C
         CALL TISBOD ( 'J2000', BODY, ET-LT, TOBDFX  )
         CALL INVSTM ( TOBDFX,  TOINRT               )
C
C        Compute the inertial state of the center of the plate
C        relative to the center of the body.
C
         CALL MXVG   ( TOINRT,  CENTER, 6, 6, CNTINR )
C
C        Combine the inertial state of the center of the plate to
C        get the inertial state of the center of the plate with
C        respect to the observer and with respect to the solar system
C        barycenter.
C
         CALL VADDG ( CNTINR,  STBOD,  6, SCNOBS  )
         CALL VADDG ( CNTINR,  SSBBOD, 6, SCNSSB  )
C
C        Now get the aberration corrected position of the center
C        of the plate with respect to the observer. Negate this
C        to get the apparent position of the observer with respect
C        to the center of the plate.
C
         CALL STELAB ( SCNOBS,   SSBOBS(4),  OBSCNT )
         CALL VMINUS ( OBSCNT,   OBSPOS )
C
C        Get the state of the sun corrected for light time and
C        stellar aberration with respect to the center of the plate.
C
         CALL SPKAPP ( 10, ET-LT, 'J2000', SCNSSB, 'LT+S', SUNPOS, LTS )
 
      ELSE
C
C        This is the easy case.  No corrections are applied at all.
C        Just get everything into bodyfixed coordinates.
C
         CALL SPKEZ ( OBS, ET, 'J2000', 'NONE', BODY,  STOBS, LT  )
         CALL SPKEZ ( 10,  ET, 'J2000', 'NONE', BODY,  STSUN, LTS )
C
C        Now get the rotation from J2000 to bodyfixed coordinates.
C
         CALL TIPBOD ( 'J2000', BODY, ET, TIPM )
C
C        Convert the position of the observer and sun to bodyfixed
C        coordinates.
C
         CALL MXV  ( TIPM,   STOBS,  OBSPOS )
         CALL VSUB ( OBSPOS, CENTER, OBSPOS )
 
         CALL MXV  ( TIPM,   STSUN,  SUNPOS )
         CALL VSUB ( SUNPOS, CENTER, SUNPOS )
 
      END IF
 
 
C
C     One way or the other we now have the position of the
C     sun and observer relative to the center of the plate.
C     Compute the angle, visible amount, and lit amount.
C
      ANGLE = VSEP ( SUNPOS, OBSPOS )
 
C
C     There is no field of view needed, in this case.
C
 
      FOVSIZ = 0
 
      CALL PNTOCC ( PLDEX,
     .              FOVSIZ,  FOV,    FOVNRM,
     .              OBSPOS,  SUNPOS,
     .              FOVFRC,  VISFRC, LITFRC,
     .              AWAY                      )
 
      IF ( .NOT. AWAY ) THEN
 
         VISAMT = VISFRC
         LITAMT = LITFRC
 
      ELSE
 
C
C     Plate faces away from observer.
C
 
         VISAMT = 0.0D0
         LITAMT = 0.0D0
 
      END IF
 
C
C     That's all folks.
C
 
      CALL CHKOUT ( 'PLSANG' )
      RETURN
      END
