C$Procedure PLTANG ( plate model, angle of incidence to target )
 
      SUBROUTINE PLTANG ( PLID, BODY, TARGET, ET, CORR, ANGLE )
 
C$ Abstract
C
C     Return the angle of incidence at a plate center to a specified
C     target body.
C
C$ Copyright
C
C     Copyright (1997), California Institute of Technology.
C     U.S. Government sponsorship acknowledged.
C
C$ Required_Reading
C
C     None.
C
C$ Keywords
C
C     PLATE
C
C$ Declarations
 
      IMPLICIT NONE
 
      INTEGER               PLID
      INTEGER               BODY
      INTEGER               TARGET
      DOUBLE PRECISION      ET
      CHARACTER*(*)         CORR
      DOUBLE PRECISION      ANGLE
 
C$ Brief_I/O
C
C     VARIABLE  I/O  DESCRIPTION
C     --------  ---  --------------------------------------------------
C     PLID       I   ID code for plate of interest
C     BODY       I   ID code for the body associated with the plate
C     TARGET     I   ID code of the target body
C     ET         I   epoch in seconds past the J2000 epoch
C     CORR       I   corrections to apply due to finite speed of light
C     ANGLE      O   angle between sun and observer from plate center
C
C$ Detailed_Input
C
C     PLID       is the plate ID code.
C
C     BODY       is the NAIF ID of the body associated with the plate
C                specified by PLID.
C
C     TARGET     is the NAIF ID of the target object.
C
C     ET         is the epoch (ephemeris time) at which the
C                angle is computed.
C
C     CORR       is a flag that indicates whether or not light-time
C                and stellar aberration should be applied in the
C                computation of the output angle.  Allowed values
C                are: 'NONE', and 'ALL'.  (This list may be expanded
C                in future releases of this routine.)  The case of
C                the letters in CORR is not significant. ('None' and
C                'NONE' are regarded as equivalent.)
C
C                If the value of CORR is 'NONE', all directions computed
C                are geometric.  If the value of CORR is 'ALL'
C                light-time and stellar aberration correction are
C                are applied in all computations.  Note that center of
C                the plate is regarded as the observer when performing
C                light-time computations.
C
C                For most users, the value 'NONE' will be suitable
C                for computing the retrieved angle.  The formal error
C                made as a result of using 'NONE' instead of all will
C                typically be on the order of 0.01 degrees.  However,
C                it is likely that the direction of a surface normal
C                will be more than 0.01 degrees away from the "true"
C                normal to the surface at the center of the plate.
C
C$ Detailed_Output
C
C     ANGLE      is the angle in radians between the plate normal and
C                target body as seen from the center of the plate,
C                otherwise known as the "angle of incidence".
C
C$ Parameters
C
C     None.
C
C$ Files
C
C     See: $Restrictions.
C
C$ Exceptions
C
C     1) If the value of CORR is not one of the recognized strings,
C        the error 'SPICE(BADCORRSPEC)' will be signalled.
C
C     2) If information required by this routine has not been
C        loaded prior to calling this routine, the omission will
C        be diagnosed by a routine in the call tree of this routine.
C
C$ Particulars
C
C      Given the body id and plate id for some plate in a triangulation
C      of the surface of some object, this routine returns the angle
C      between the normal to the plate and the vector from the
C      center of the plate to the specified target body.
C
C$ Examples
C
C     The following code fragment illustrates the use of
C     this routine to determine number of plates for which the
C     angle of incidence of sun is less than 20 degrees and the
C     angle of incidence of the observer is less than 45 degrees.
C
C
C   C
C   C     Load plate model.
C   C
C         CALL RDPLAT ( 'eros.plt' )
C
C   C
C   C     Load ephemerides.
C   C
C         CALL SPKLEF ( 'eros.bsp',  HAND1 )
C         CALL SPKLEF ( 'near.bsp',  HAND2 )
C         CALL SPKLEF ( 'de403.bsp', HAND3 )
C
C   C
C   C     Load asteroid attitude file.
C   C
C         CALL PCKLOF ( 'astatt.bpc', HAND4 )
C
C   C
C   C     Get the number of plates in the Eros model.
C   C
C         CALL PLTNUM ( 2000433, NPLATE )
C
C   C
C   C     Determine how many plates are visible
C   C     from the NEAR spacecraft with an angle of incidence
C   C     of less than 45 degrees and with a solar
C   C     angle of incidence of less than 20 degrees.
C   C
C
C        NFOUND = 0
C
C        ANGSC  = 45.D0 * RPD()
C        ANGSUN = 20.D0 * RPD()
C
C        DO I = 1, NPLATE
 
C           CALL PLTANG ( I, 2000433, -93, ET, ANG )
 
C           IF ( ANG .LT. ANGSC ) THEN
 
C              CALL PLTANG ( I, 2000433, 10, ET, ANG )
 
C              IF ( ANG .LT. ANGSUN ) THEN
C                 NFOUND = NFOUND + 1
C              END IF
 
C           END IF
C        END DO
C
C
C$ Restrictions
C
C     1) DB kernel(s) must be loaded containing plate model data
C        for the body specified by the input BODY..
C
C     2) SP kernel(s) must be loaded containing ephemeris data
C        for the plate model body or bodies, the target body, the
C        Sun and possibly plate model satellites, spacecraft, and
C        intermediate bodies.
C
C     3) PC kernel(s) must be loaded containing orientation data
C        for the body specified by the input BODY.
C
C$ Author_and_Institution
C
C     J.A. Bytof      (JPL)
C     W.L. Taber      (JPL)
C
C$ Literature_References
C
C     None.
C
C$ Version
C
C-    SPICELIB Version 1.0.0, 19-JUN-1997 (WLT)
C
C-&
 
C$ Index_Entries
C
C     get angle of incidence to a body at a plate center
C
C-&
 
C$ Revisions
C
C-&
 
C
C     SPICELIB functions
C
 
      LOGICAL               RETURN
      DOUBLE PRECISION      VSEP
 
C
C     Local variables
C
      INTEGER               WDSIZE
      PARAMETER           ( WDSIZE = 32 )
 
 
      CHARACTER*(WDSIZE)    MYCORR
 
 
      INTEGER               I
      INTEGER               J
      INTEGER               BODEX
      INTEGER               PLDEX
 
      DOUBLE PRECISION      CENTER ( 6 )
      DOUBLE PRECISION      CNTINR ( 6 )
 
      DOUBLE PRECISION      LT
      DOUBLE PRECISION      NORMAL ( 3 )
      DOUBLE PRECISION      SCNSSB ( 6 )
      DOUBLE PRECISION      SSBBOD ( 6 )
      DOUBLE PRECISION      STARG  ( 6 )
      DOUBLE PRECISION      TIPM   ( 3, 3 )
      DOUBLE PRECISION      ROT    ( 3, 3 )
      DOUBLE PRECISION      TOBDFX ( 6, 6 )
      DOUBLE PRECISION      TOINRT ( 6, 6 )
 
 
C
C     Standard SPICE error handling.
C
 
      IF ( RETURN () ) THEN
         RETURN
      ELSE
         CALL CHKIN ( 'PLTANG' )
      END IF
 
C
C     See what kind of corrections should be applied.
C
      CALL LJUST ( CORR,   MYCORR )
      CALL UCASE ( MYCORR, MYCORR )
 
      IF (       MYCORR .NE. 'ALL'
     .     .AND. MYCORR .NE. 'NONE' )  THEN
 
         CALL SETMSG ( 'The corrections to apply in the '
     .   //            'computations performed by PLSANG must '
     .   //            'be ''ALL'' or ''NONE''.  The '
     .   //            'corrections specified were ''#''. ' )
 
         CALL ERRCH  ( '#', CORR )
         CALL SIGERR ( 'SPICE(BADCORRSPEC)'  )
         CALL CHKOUT ( 'PLTANG' )
         RETURN
 
      END IF
 
C
C     Get body index, given its NAIF ID.
C
 
      CALL BID2DX ( BODY, BODEX )
 
C
C     Get the name of the body associated with the plate.
C
 
      CALL PLTIDX ( PLID, BODEX, PLDEX )
 
C
C     Get the outward pointing normal vector of the requested
C     plate in plate model coordinates.
C
 
      CALL NTHPLN ( PLDEX, NORMAL )
 
C
C     Get the position of the center of the plate in plate
C     model coordinates.
C
      CALL NTHPLC ( PLDEX, CENTER )
 
C
C     We either are going to apply every correction we can think
C     of or we're just going to take the quick and dirty approach.
C
      IF ( MYCORR .EQ. 'ALL' ) THEN
 
C
C        First of all, the bodyfixed velocity of the plate center is
C        zero.
C
         CENTER(4) = 0.0D0
         CENTER(5) = 0.0D0
         CENTER(6) = 0.0D0
C
C        Get the position and velocity of the center of the plate
C        relative to the center of the body in J2000 coordinates.
C        TISBOD returns the state transformation matrix form J2000
C        to the bodyfixed frame.  INVSTM inverts this transformation.
C
         CALL TISBOD ( 'J2000', BODY,    ET,  TOBDFX )
         CALL INVSTM ( TOBDFX,  TOINRT               )
C
C        Now we can just multiply to get the J2000 state of the
C        center of the plate.
C
         CALL MXVG   ( TOINRT,  CENTER, 6, 6, CNTINR )
C
C        Rotate the NORMAL vector to the plate into the J2000
C        frame.
C
         DO I = 1,3
            DO J = 1,3
               ROT(I,J) = TOINRT(I,J)
            END DO
         END DO
 
         CALL MXV ( ROT, NORMAL, NORMAL )
C
C        Get the state of the body of the plate relative to the
C        solar system  barycenter.
C
         CALL SPKSSB ( BODY, ET, 'J2000', SSBBOD )
C
C        Add the state of the center of the body to the state of the
C        center of the plate to get the state of the plate center
C        relative to the solar system barycenter.
C
         CALL VADDG ( CNTINR,  SSBBOD,  6, SCNSSB  )
 
C
C        Get the state of the target corrected for light time and
C        stellar aberration with respect to the center of the plate.
C
         CALL SPKAPP ( TARGET, ET, 'J2000', SCNSSB, 'LT+S', STARG, LT )
 
      ELSE
C
C        Just compute the geometric state of the target relative to
C        the center of the plate in bodyfixed coordinates.
C
         CALL SPKGEO ( TARGET,  ET, 'J2000', BODY, STARG, LT )
         CALL TIPBOD ( 'J2000', BODY, ET,  TIPM    )
         CALL MXV    ( TIPM,    STARG, STARG )
         CALL VSUB   ( STARG,  CENTER, STARG )
 
      END IF
C
C     We now have the "state" of the target relative to the center
C     of the plate relative to the same frame as the normal vector
C     to the plate.  All we need is the angle between these vectors.
C
 
      ANGLE = VSEP ( NORMAL, STARG )
 
      CALL CHKOUT ( 'PLTANG' )
 
      RETURN
      END
