C$Procedure  PNTOCC  ( Plate occlusion processor )
 
      SUBROUTINE PNTOCC ( PLDEX,  FOVSIZ,  
     .                    FOV,    FOVNRM,
     .                    OBS,    SUN,
     .                    FOVAMT, VISAMT, 
     .                    LITAMT, AWAY   )
 
C$ Abstract
C
C     Determine the visibility of a plate from a spacecraft and
C     the extent to which it is in shadow.
C
C$ Copyright
C
C     Copyright (2000), California Institute of Technology.
C     U.S. Government sponsorship acknowledged.
C
C$ Required_Reading
C
C     None.
C
C$ Keywords
C
C     OCCLUSION
C
C$ Declarations
 
      IMPLICIT              NONE
 
      INTEGER               PLDEX
      INTEGER               FOVSIZ
 
      DOUBLE PRECISION      FOV       ( 3, * )
      DOUBLE PRECISION      FOVNRM    ( 3, * )
      DOUBLE PRECISION      OBS       ( 3 )
      DOUBLE PRECISION      SUN       ( 3 )
      DOUBLE PRECISION      FOVAMT
      DOUBLE PRECISION      VISAMT
      DOUBLE PRECISION      LITAMT
 
      LOGICAL               AWAY
 
 
C$ Brief_I/O
C
C     VARIABLE  I/O  DESCRIPTION
C     --------  ---  --------------------------------------------------
C     PLDEX      I   Plate index.
C     FOVSIZ     I   Number of sides in field of view.
C     FOV        I   Field of view vectors in body-fixed coordinates.
C     FOVNRM     I   Inward-pointing normal vectors to the side planes
C                    of the field of view volume.
C     OBS        I   Observer's position in body-fixed coordinates.
C     SUN        I   Sun's direction in body-fixed coordinates.
C     FOVAMT     O   Fraction of plate area which is in the field
C                    of view.
C     VISAMT     O   Fraction of plate area which is in the field
C                    of view and is not hidden from view by
C                    other plates.
C     LITAMT     O   Fraction of visible plate area which is illuminated
C                    by the Sun, accounting for Sun position and
C                    shadowing by other plates.
C     AWAY       O   Logical variable that is true if plate faces away
C                    from observer.
C
C$ Detailed_Input
C
C     PLDEX      Plate index.
C
C     FOVSIZ     Number of sides in field of view.  If this is
C
C     FOV        Field of view vectors in body-fixed coordinates.
C
C     FOVNRM     Inward-pointing normal vectors to the side planes
C                of the field of view volume.
C
C     OBS        Observer's position in body-fixed coordinates.
C
C     SUN        Sun's direction in body-fixed coordinates.
C
C$ Detailed_Output
C
C     FOVAMT     Fraction (0.0 to 1.0) of plate area inside
C                the field of view volume.
C
C     VISAMT     Fraction (0.0 to 1.0) of plate area inside
C                the field of view volume and not hidden from the
C                observer by other plates.
C
C     LITAMT     is the fraction (0.0 to 1.0) of a plate's visible
C                area illuminated, accounting for solar position
C                and shadowing by other plates. If a plate faces away
C                from the observer, or is completely outside the FOV,
C                -1.0 is returned.  If shadow processing is turned off,
C                -9.0 is returned.
C
C     AWAY       Logical variable with value TRUE if plate faces away
C                from observer.
C
C$ Parameters
C
C     None.
C
C$ Files
C
C     None.
C
C$ Exceptions
C
C     NOT Error free.
C
C$ Particulars
C
C     Later.
C
C$ Examples
C
C     Later.
C
C$ Restrictions
C
C     None.
C
C$ Author_and_Institution
C
C     J.A. Bytof      (JPL)
C
C$ Literature_References
C
C     None.
C
C$ Version
C
C-    SPICELIB Version 2.0.0, 17-JAN-2000 (JAB)
C
C        Corrected errors in counting plate grid points that
C        determine the fraction of the plate that is
C        visible and shadowed.  Improved some comments.
C
C-    SPICELIB Version 1.0.1, 23-MAR-1999 (JAB)
C
C        Implemented first-order horizon mask processing.
C
C-    SPICELIB Version 1.0.0, 03-FEB-1999 (JAB)
C
C        First version.
C-&
 
C$ Index_Entries
C
C     plate occlusion processor
C
C-&
 
 
      INCLUDE               'pltmax.inc'
 
 
C
C     Local variables.
C
 
      INTEGER               MAXGRD
      PARAMETER           ( MAXGRD = 200 )
 
      INTEGER               LBPOOL
      PARAMETER           ( LBPOOL = -5 )
 
      INTEGER               LNGEST
      PARAMETER           ( LNGEST = 50 )
 
      INTEGER               PTHSIZ
      PARAMETER           ( PTHSIZ = LNGEST + 2 )
 
      INTEGER               VTXSIZ
      PARAMETER           ( VTXSIZ = 3*LNGEST )
 
 
      INTEGER               I
      INTEGER               J
      INTEGER               NDIV
      INTEGER               PMASK
      INTEGER               NGRID
      INTEGER               FOVCNT
      INTEGER               LITCNT
      INTEGER               DRKCNT
      INTEGER               VISCNT
      INTEGER               HITCNT
      INTEGER               NPLATE
      INTEGER               PATH
      INTEGER               PTHIN
      INTEGER               TRIDIV
      INTEGER               PLIST    ( MAXPLT )
      INTEGER               PATHS    ( 3, LBPOOL:PTHSIZ )
 
      LOGICAL               HIT
      LOGICAL               RETURN
      LOGICAL               APPROX
      LOGICAL               PINFOV
      LOGICAL               DOSHAD
      LOGICAL               GRMADE
      LOGICAL               RAYOCC
      LOGICAL               FOVFLG    ( MAXGRD )
      LOGICAL               LITFLG    ( MAXGRD )
      LOGICAL               VISFLG    ( MAXGRD )
 
      DOUBLE PRECISION      DPR
      DOUBLE PRECISION      VDOT
      DOUBLE PRECISION      VSEP
      DOUBLE PRECISION      DIST
      DOUBLE PRECISION      POLYA
      DOUBLE PRECISION      TOTAL
      DOUBLE PRECISION      INAREA
      DOUBLE PRECISION      INVIEW
      DOUBLE PRECISION      PTHARA
      DOUBLE PRECISION      MAXELE
      DOUBLE PRECISION      COELEV
      DOUBLE PRECISION      COEOBS
      DOUBLE PRECISION      COESUN
      DOUBLE PRECISION      TOOBS     ( 3 )
      DOUBLE PRECISION      NORML     ( 3 )
      DOUBLE PRECISION      CENTR     ( 3 )
      DOUBLE PRECISION      VERTXS    ( VTXSIZ )
      DOUBLE PRECISION      POINTS    ( 3, 3 )
      DOUBLE PRECISION      GRDXYZ    ( 3, MAXGRD )
      DOUBLE PRECISION      GRDCEN    ( 3, MAXGRD )
      DOUBLE PRECISION      RAYDIR    ( 3, MAXGRD )
 
 
C
C     Initialize variables and counters.
C
 
      AWAY = .FALSE.
 
      DIST = 0.D0
 
      FOVCNT = -999
      VISCNT = -999
      LITCNT = -999
      DRKCNT = -999
      HITCNT = -999
 
C
C     Standard SPICELIB error handling.
C
 
      IF ( RETURN () ) THEN
         RETURN
      END IF
 
      CALL CHKIN  ( 'PNTOCC' )
 
C
C     Initialize grid point creation flag.
C
 
      GRMADE = .FALSE.
 
C
C     Get the plate's unit normal vector and center.
C
 
      CALL NTHPUN ( PLDEX, NORML )
      CALL NTHPLC ( PLDEX, CENTR )
 
C
C     Get the vector from the plate's center to the observer.
C
 
      CALL VSUB ( OBS, CENTR, TOOBS )
 
C
C     Return now if the plate is facing away from the observer.
C
 
      IF ( VDOT( TOOBS, NORML ) .LT. 0.D0 ) THEN
 
         FOVAMT =  0.D0
         VISAMT =  0.D0
         LITAMT = -1.D0
         AWAY   = .TRUE.
 
         CALL CHKOUT ( 'PNTOCC' )
 
         RETURN
 
      END IF
 
C
C     Get the coordinates of the plate's vertices.
C
 
      CALL NTHPLV ( PLDEX, POINTS )
 
 
C****************************************************************
C************  Field of View Processing   ***********************
C****************************************************************
 
C
C     If FOVSIZ is zero, field of view will not be processed.
C
 
      IF ( FOVSIZ .EQ. 0 ) THEN
 
         FOVAMT = 1.D0
 
      ELSE
 
         CALL PTHINI ( 3, LNGEST, PTHSIZ, PATHS, VERTXS )
 
         CALL PTHMAK ( 3, 3, POINTS, PATHS, VERTXS, PATH )
 
C
C     Determine the extent of plate in the field of view.
C
 
 
         CALL POLYFV_2 ( OBS,    FOVSIZ,  FOVNRM,   PATH,
     .                   PATHS,  VERTXS,  PTHIN          )
 
C
C     If some part of the path is in the field of view
C     the value of PTHIN will be non-zero.
C
 
         IF ( PTHIN .NE. 0 ) THEN
 
            INAREA = PTHARA ( PTHIN, PATHS, VERTXS )
            TOTAL  = POLYA  ( 3,     3,     POINTS )
 
            INVIEW = INAREA / TOTAL
 
C
C     We don't need the portion that's inside the
C     field of view any more.  Remove it from the
C     path list.
C
 
            CALL PTHKIL ( PTHIN, PATHS, VERTXS )
 
C
C     Is plate totally inside the field of view?
C
 
            IF ( APPROX( INVIEW, 1.D0, 1.D-6 ) ) THEN
 
               FOVAMT = 1.D0
 
            ELSE
 
C
C     Generate the plate's grid point coordinates
C     and initialize the field of view occlusion flag array.
C
 
C
C     TRIDIV returns the current triangle division parameter.
C
 
               NDIV = TRIDIV( 0 )
 
               CALL SUBTRI ( POINTS, NDIV, NGRID, GRDXYZ, GRDCEN )
 
               DO I = 1, NGRID
                  FOVFLG(I) = .FALSE.
                  VISFLG(I) = .FALSE.
               END DO
 
               GRMADE = .TRUE.
 
C
C     Find grid points that are outside the FOV.
C
               FOVCNT = 0
 
               DO I = 1, NGRID
                  IF ( PINFOV( FOVSIZ, FOVNRM, OBS, GRDCEN(1,I)) ) THEN
                     FOVFLG(I) = .TRUE.
                     VISFLG(I) = .TRUE.
                     FOVCNT    = FOVCNT + 1
                  END IF
               END DO
 
C
C     No grid points lie in the field of view.
C
               IF ( FOVCNT .EQ. 0  ) THEN
 
                  FOVAMT  = 0.D0
                  VISAMT  = 0.D0
                  LITAMT = -1.D0
 
                  CALL CHKOUT ( 'PNTOCC' )
 
                  RETURN
 
               END IF
 
               FOVAMT = DBLE(FOVCNT) / DBLE(NGRID)
 
            END IF
 
         ELSE
 
C
C     Plate is totally outside the field of view.
C
            FOVAMT =  0.D0
            VISAMT =  0.D0
            LITAMT = -1.D0
 
            CALL CHKOUT ( 'PNTOCC' )
 
            RETURN
 
         END IF
 
      END IF
 
 
C****************************************************************
C************  Plate Mask Processing  ***************************
C****************************************************************
 
 
C
C     Obtain the plate's elevation mask values.
C
 
      CALL NTHMSK ( PLDEX, PMASK, MAXELE )
 
C
C     If PMASK = 0, the plate is not occludable.
C
 
      IF ( PMASK .EQ. 0 ) THEN
 
         VISAMT = FOVAMT
 
C
C     If shadow processing turned off, LITAMT is given a missing
C     value flag.
C
 
         IF ( .NOT. DOSHAD( 0 ) ) THEN
 
            LITAMT = -9.D0
 
         ELSE
 
            IF ( VDOT( SUN, NORML ) .GT. 0.D0 ) THEN
               LITAMT =  1.D0
            ELSE
               LITAMT =  0.D0
            END IF
 
         END IF
 
         CALL CHKOUT ( 'PNTOCC' )
         RETURN
 
      END IF
 
 
C****************************************************************
C************  Visibility Processing  ***************************
C****************************************************************
 
C
C     What is the angular separation between the plate's
C     normal vector and the horizon mask?
C
 
      COELEV = 90.D0 - MAXELE
 
C
C     What is the observer's co-elevation with respect to
C     the plate?
C
 
      COEOBS = DPR() * VSEP ( TOOBS, NORML )
 
C
C     Could the horizon mask interfere with the line of
C     sight to the observer.
C
 
      IF ( COEOBS .GT. COELEV ) THEN
 
C
C     Plate might be partially or toally hidden from observer.
C
 
C
C     Generate plate grid points.
C
 
         IF ( .NOT. GRMADE ) THEN
 
            NDIV = TRIDIV( 0 )
            CALL SUBTRI ( POINTS, NDIV, NGRID, GRDXYZ, GRDCEN )
 
C
C     The plate lies totally within the field of view.
C
 
            DO I = 1, NGRID
               FOVFLG(I) = .TRUE.
            END DO
 
            FOVCNT = NGRID
 
            GRMADE = .TRUE.
 
         END IF
 
C
C     Initialize the visibility flag array.  We are
C     only interested in grid points that are inside the
C     field of view.
C
 
         DO I = 1, NGRID
            VISFLG(I) = FOVFLG(I)
         END DO
 
         VISCNT = FOVCNT
 
C
C     Query the spatial index and list the plates that might
C     hide plate PLDEX from the observer.
C
 
         CALL TRIVOX ( PLDEX, TOOBS, NPLATE, PLIST )
 
 
C
C     For each grid point inside the field of view,
C     generate a ray to the observer.
C
 
         DO I = 1, NGRID
            IF ( VISFLG(I) ) THEN
               CALL VSUB ( OBS, GRDCEN(1,I), RAYDIR(1,I) )
            END IF
         END DO
 
C
C     Loop over all possible occluding plates and visible
C     grid points.
C
 
         HITCNT = 0
 
         DO J = 1, NPLATE
 
            DO I = 1, NGRID
 
C
C     If grid point remains visible, test for occlusion.
C
 
               IF ( VISFLG(I) ) THEN
 
C
C     Does the ray intersect a plate?
C
 
                  HIT = RAYOCC( GRDCEN(1,I), RAYDIR(1,I),
     .                          PLIST(J),    DIST         )
 
 
                  IF ( HIT )  THEN
 
                     VISFLG(I) = .FALSE.
                     HITCNT = HITCNT + 1
 
                  END IF
 
               END IF
 
            END DO
 
         END DO
 
         VISCNT = (FOVCNT - HITCNT)
         VISAMT = DBLE(VISCNT) / DBLE(NGRID)
 
C
C     Return now if no visible points are left.
C
 
         IF ( VISCNT .EQ. 0 ) THEN
            LITAMT = -1.D0
            CALL CHKOUT ( 'PNTOCC' )
            RETURN
         END IF
 
      ELSE
 
C
C    Observer can see all of plate within field of view.
C
 
        VISAMT = FOVAMT
        VISCNT = FOVCNT
 
      END IF
 
 
C
C     Shadow Processing
C

C
C     If shadow processing is turned off, exit the subroutine.
C
 
      IF ( .NOT. DOSHAD( 0 ) ) THEN
         LITAMT = -9.D0
         CALL CHKOUT ( 'PNTOCC' )
         RETURN
      END IF
 
 
C
C     Does the plate face the Sun?
C
      COESUN = DPR() * VSEP ( SUN, NORML )
 
      IF ( COESUN .GE. 90.D0 ) THEN
 
         LITAMT = 0.D0
 
      ELSE IF ( COESUN .GT. COELEV ) THEN
 
C
C        Initialize lighting flag arra y.
C
         IF ( .NOT. GRMADE ) THEN
            NDIV = TRIDIV( 0 )
            CALL SUBTRI ( POINTS, NDIV, NGRID, GRDXYZ, GRDCEN )
            DO I = 1, NGRID
               FOVFLG(I) = .TRUE.
               VISFLG(I) = .TRUE.
            END DO
            FOVCNT = NGRID
            VISCNT = NGRID
         END IF
 
         GRMADE = .TRUE.
 
         DO I = 1, NGRID
            LITFLG(I) = VISFLG(I)
         END DO
 
C
C     Get a list of plates that might shadow this plate from
C     the sun.
C
         CALL TRIVOX ( PLDEX, SUN, NPLATE, PLIST )
 
C
C     Loop over all potentially shadowing plates and visible
C     grid points.
C
         DRKCNT = 0
 
         DO J = 1, NPLATE
 
            DO I = 1, NGRID
 
C
C              If the grid point is visible, test for shadowing.
C
               IF ( LITFLG(I) ) THEN
 
C
C                 Does the ray intersect a plate?
C
                  HIT = RAYOCC( GRDCEN(1,I), SUN, PLIST(J), DIST )
 
                  IF ( HIT )  THEN
 
                     LITFLG(I) = .FALSE.
                     DRKCNT  = DRKCNT + 1
                  END IF
 
               END IF
 
            END DO
 
         END DO
 
         LITCNT = (VISCNT - DRKCNT)
 
         IF ( VISCNT .GT. 0 ) THEN
            LITAMT = DBLE(LITCNT) / DBLE(VISCNT)
         ELSE
            LITAMT = 0.D0
         END IF
 
      ELSE
 
         LITAMT = 1.D0
 
      END IF
 
C
C     Standard SPICE error handling.
C 
      CALL CHKOUT ( 'PNTOCC' )
 
      RETURN
      END

C12345678901234567890123456789012345678901234567890123456789012345678912
