C$Procedure POLYA ( Polygon area )
 
      DOUBLE PRECISION FUNCTION POLYA ( DIM, N, PTS )
 
C$ Abstract
C
C    Determine the area of a convex polygon that lies in a
C    plane specified by the vertices of the polygon.
C
C$ Copyright
C
C     Copyright (1997), California Institute of Technology.
C     U.S. Government sponsorship acknowledged.
C
C$ Required_Reading
C
C     None.
C
C$ Keywords
C
C     GEOMETRY
C
C$ Declarations
 
      IMPLICIT NONE
      INTEGER               DIM
      INTEGER               N
      DOUBLE PRECISION      PTS ( DIM, * )
 
C$ Brief_I/O
C
C     VARIABLE  I/O  DESCRIPTION
C     --------  ---  --------------------------------------------------
C     DIM        I   Dimension of the space of the polygon (2 or 3).
C     N          I   Number of vertices of the polygon.
C     PTS        I   The vertices of the polygon
C
C     The function returns the area of the polygon.
C
C$ Detailed_Input
C
C     DIM        is the dimension of the space in which the vertices
C                of the polygon are given.  Allowed values are 2 and 3.
C
C     N          is the number of vertices of the polygon
C
C     PTS        are the coordinates of the vertices of the polygon.
C                The edges of the polygon are made up of the segments
C
C                   PTS(1:DIM,1)   to PTS(1:DIM,2)
C                   PTS(1:DIM,2)   to PTS(1:DIM,3)
C                                .
C                                .
C                                .
C
C                   PTS(1:DIM,N-1) to PTS(1:DIM,N)
C                   PTS(1:DIM,N)   to PTS(1:DIM,1)
C
C                 where PTS(1:DIM,I) refers to the I'th vertex
C
C$ Detailed_Output
C
C     The function returns the area of the polygon spanned by the
C     points in the array PTS.
C
C$ Parameters
C
C     None.
C
C$ Files
C
C     None.
C
C$ Exceptions
C
C     1) If DIM is not 2 or 3, the error SPICE(BADDIMENSION) is
C        signalled and the function returns the value 0.
C
C$ Particulars
C
C     This routine is a macro for the pseudo code below:
C
C
C        VSUM(1) = 0.0D0
C        VSUM(2) = 0.0D0
C        VSUM(3) = 0.0D0
C
C        DO I = 1, N
C
C           J = I+1
C
C           IF ( J .GT. N ) THEN
C              J = J - N
C           END IF
C
C           Compute the vector quantity:
C
C           VSUM = VSUM +  PTS(1,I) x PTS(1,J)
C
C        END DO
C
C        AREA = 1/2 NORM( VSUM )
C
C$ Examples
C
C     Suppose you need the area of a triangle given by the points
C     P1, P2, and P3 in the 3-space.  The code below will perform
C     the area computation.
C
C
C        DOUBLE PRECISION      P1 ( 3 )
C        DOUBLE PRECISION      P2 ( 3 )
C        DOUBLE PRECISION      P3 ( 3 )
C        DOUBLE PRECISION      T1 ( 3, 3 )
C
C        Set the values of P1, P2, and P3.  Then we move them
C        into the point array T1.
C
C        CALL MOVED ( P1, 3, T1(1,1) )
C        CALL MOVED ( P2, 3, T1(1,2) )
C        CALL MOVED ( P3, 3, T1(1,3) )
C
C        AREA = POLYA ( 3, 3, T1 )
C
C
C     To compute the area of a quadrilateral in the plane whose vertices
C     are given by the points Q1, Q2, Q3, Q4, use the block of code
C     below.
C
C
C        DOUBLE PRECISION      Q1 ( 2 )
C        DOUBLE PRECISION      Q2 ( 2 )
C        DOUBLE PRECISION      Q3 ( 2 )
C        DOUBLE PRECISION      Q4 ( 2 )
C
C        DOUBLE PRECISION      QUAD ( 2, 4 )
C
C        CALL MOVED ( Q1, 2, QUAD(1,1) )
C        CALL MOVED ( Q2, 2, QUAD(1,2) )
C        CALL MOVED ( Q3, 2, QUAD(1,3) )
C        CALL MOVED ( Q4, 2, QUAD(1,4) )
C
C        AREA = POLYA ( 2, 4, QUAD )
C
C$ Restrictions
C
C     None.
C
C$ Author_and_Institution
C
C     W.L. Taber      (JPL)
C
C$ Literature_References
C
C     None.
C
C$ Version
C
C-    SPICELIB Version 1.0.0, 3-MAR-1997 (WLT)
C
C
C-&
 
C$ Index_Entries
C
C     Compute the area of a polygon in the plane
C     Compute the area of a polygon in space
C
C-&
C
C     Spicelib Functions
C
      DOUBLE PRECISION      VNORM
 
C
C     Local Variables
C
 
      DOUBLE PRECISION      SUM
      DOUBLE PRECISION      TMP   ( 3 )
      DOUBLE PRECISION      V     ( 3, 2 )
      DOUBLE PRECISION      VSUM  ( 3 )
 
 
      INTEGER               FRST
      INTEGER               SCND
      INTEGER               I
 
 
      IF      ( DIM .EQ. 2 ) THEN
 
         IF ( N .LT. 3 ) THEN
            POLYA = 0.0D0
            RETURN
         END IF
 
         SUM  = 0.0D0
         FRST = 2
         SCND = 1
 
         CALL VSUBG ( PTS(1,2), PTS(1,1), 2, V(1,1) )
 
         DO I = 3, N
 
            CALL SWAPI ( FRST, SCND )
            CALL VSUBG ( PTS(1,I), PTS(1,1), 2, V(1,SCND) )
C
C           Compute the cross product of V(1,FRST) with V(2,SCND)
C
 
            SUM = SUM + ( V(1,FRST)*V(2,SCND) - V(2,FRST)*V(1,SCND) )
 
 
         END DO
 
         POLYA = 0.5D0 * ABS( SUM )
 
 
      ELSE IF ( DIM .EQ. 3 ) THEN
 
         IF ( N .LT. 3 ) THEN
            POLYA = 0.0D0
            RETURN
         END IF
C
C        Set the initial vector sum.
C
         VSUM(1) = 0.0D0
         VSUM(2) = 0.0D0
         VSUM(3) = 0.0D0
 
         FRST    = 2
         SCND    = 1
C
C        Compute the vector from the first vertex to the second.
C
         CALL VSUB ( PTS(1,2), PTS(1,1), V(1,1) )
 
         DO I = 3, N
 
            CALL SWAPI( FRST, SCND )
C
C           Compute the vector from the first vertex to the I'th.
C
            CALL VSUB ( PTS(1,I), PTS(1,1), V(1,SCND) )
 
C
C           Compute the cross product of V(1,FRST) with V(1,SCND)
C
            CALL VCRSS ( V(1,FRST), V(1,SCND), TMP  )
            CALL VADD  ( TMP,       VSUM,      VSUM )
 
         END DO
 
         POLYA = 0.5D0 * VNORM ( VSUM )
 
      ELSE
 
         POLYA = 0.0D0
 
         CALL CHKIN ( 'POLYA' )
         CALL SETMSG ( 'The dimension of the space for computing '
     .   //            'the area must be 2 or 3.  You''ve '
     .   //            'specified a dimension of #. ' )
 
         CALL ERRINT ( '#', DIM )
         CALL SIGERR ( 'SPICE(BADDIMENSION)'  )
         CALL CHKOUT ( 'POLYA' )
         RETURN
 
 
      END IF
 
      RETURN
      END
