C$Procedure      POLYCT ( Polygon cut )
 
      SUBROUTINE POLYCT ( PATH,  PATHS, VERTXS,
     .                    REFPT, NORML, PTHIN, PTHOUT )
 
C$ Abstract
C
C     Given a plane polygon (either in 2-space or 3-space) described
C     by a series of vertices, determine the portion of the polygon
C     lies inside a user specified half-space and the portion that
C     lies outside the user specified half space.
C
C$ Copyright
C
C     Copyright (1997), California Institute of Technology.
C     U.S. Government sponsorship acknowledged.
C
C$ Required_Reading
C
C      None.
C
C$ Keywords
C
C      GEOMETRY
C
C$ Declarations
 
      IMPLICIT NONE
 
      INTEGER               LBPOOL
      PARAMETER           ( LBPOOL = -5 )
 
      INTEGER               PATH
      INTEGER               PATHS  ( 2, LBPOOL:* )
      DOUBLE PRECISION      VERTXS ( * )
      DOUBLE PRECISION      REFPT  ( * )
      DOUBLE PRECISION      NORML  ( * )
      INTEGER               PTHIN
      INTEGER               PTHOUT
 
C$ Brief_I/O
C
C     VARIABLE  I/O  DESCRIPTION
C     --------  ---  --------------------------------------------------
C     PATH       I   Is the head node of a path
C     REFPT      I   A point on the edge of the half-plane or space
C     NORML      I   A vector pointing to the interior of the half-space
C     PATHS     I/O  Is the linked list component of an initialized list
C     VERTXS    I/O  Is the coordinate component of an initialized list
C     PTHIN      O   The points making up the vertices of intersection
C     PTHOUT     O   the points making up the vertices of intersection
C
C$ Detailed_Input
C
C     PATH       is the node associated with the first vertex of a path
C                in the path list made up of PATHS and VERTXS.
C
C                Note that although the value of PATH is not modified
C                by this routine, upon exit from POLYCT, PATH no longer
C                is the node of a path in the path list.
C
C     REFPT      is a point on the edge of the half-plane or half-space.
C
C     NORML      is a vector pointing into the interior of the
C                half-plane or half-space
C
C     PATHS      is the linked list portion an initialized path list
C                that holds the node values of paths.
C
C     VERTXS     is the vertex portion of an initialized path list
C                that holds the coordinates of path vertices.
C
C$ Detailed_Output
C
C     PATHS      is the linked list portion the path list after
C                the input PATH has been cut into two separate
C                paths---one on each side of the boundary.
C
C     VERTXS     is the vertex portion of the path list after
C                the input PATH has been cut into two separate
C                paths---one on each side of the boundary.
C
C     PTHIN      is the head node associated with the portion of the
C                path that lies inside the half-space pointed
C                to by NORML
C
C     PTHOUT     is the head node associated with the portion of the
C                path that lies inside the half-space that NORML
C                points away from.
C
C$ Parameters
C
C     None.
C
C$ Files
C
C     None.
C
C$ Exceptions
C
C     1) If the dimension, DIM, supplied is not 2 or 3, the error
C        'SPICE(BADDIMENSION)' is signalled.
C
C
C$ Particulars
C
C     A line (or plane) in the plane (or in space) forms a boundary
C     between two open half-planes (half-spaces).  Given a normal
C     vector to the boundary and a point in the boundary, the two
C     open regions separated by the boundary can be named interior and
C     exterior.  The normal vector points into the interior. It points
C     away from the exterior.
C
C     Given a convex polygon, (represented here by an ordered sequence
C     of vertices) it can be decomposed into two distinct convex
C     polygons.  One that intersects the interior region and one
C     that intersects the exterior region.
C
C     This routine returns the components of this decomposition
C     that have a non-zero area.
C
C
C     Note that if the input normal vector is parallel to the
C     i'th coordinate axis, special care is taken at
C     points of intersection of the boundary and path. The i'th
C     coordinate of intersection points will equal the i'th
C     coordinate of REFPTN.
C
C$ Examples
C
C     Suppose you would like to find the intersection of two
C     overlapping triangles in the plane.  This routine together
C     with the routine PTSHLF allow you to easily construct the
C     vertices of the intersection region.
C
C     Let the points of P1, P2, and P3 be the vertices of the first
C     triangle, and let Q1, Q2, and Q3 be the vertices of the second.
C
C     Load the points into the arrays PATH and POLY2.
C
C     PATH ( 1, 1 )  = P1(1)
C     PATH ( 1, 2 )  = P2(1)
C     PATH ( 1, 3 )  = P3(1)
C
C     PATH ( 2, 1 )  = P1(2)
C     PATH ( 2, 2 )  = P2(2)
C     PATH ( 2, 3 )  = P3(2)
C
C     SIZE = 3
C
C     POLY2 ( 1, 1 ) = Q1(1)
C     POLY2 ( 1, 2 ) = Q2(1)
C     POLY2 ( 1, 3 ) = Q3(1)
C
C     POLY2 ( 2, 1 ) = Q1(2)
C     POLY2 ( 2, 2 ) = Q2(2)
C     POLY2 ( 2, 3 ) = Q3(2)
C
C     Find the intersections with each of the interior half-planes
C     of the triangle Q1, Q2, Q3.
C
C     DO I = 1, 3
C
C        Construct the next half-plane of the triangle with vertices
C        Q1, Q2, and Q3
C
C        J1 = I
C        J2 = I+1
C        J3 = I+2
C
C        IF ( J2 .GT. 3 ) THEN
C           J2 = J2 - 3
C        END IF
C
C        IF ( J3 .GT. 3 ) THEN
C           J3 = J3 - 3
C        END IF
C
C        CALL PTSHLF ( POLY2(1,J1),  POLY2(1,J2), POLY2(1,J3), NORML )
C
C        Find the intersection with the current half-plane.
C
C        CALL POLYCT ( 2,
C                      SIZE,  PATH,  POLY2(1,J1), NORML,
C                      SIZIN, PTHIN, SIZOUT,      PTHOUT )
C
C        Copy the interior portion back into PATH.
C
C        DO J = 1, SIZIN
C           PATH(1,J) = PTHIN(1,J)
C           PATH(2,J) = PTHIN(2,J)
C        END DO
C
C        SIZE = SIZIN
C
C     END DO
C
C     PATH now contains the vertices of the intersection.
C
C$ Restrictions
C
C     None.
C
C$ Author_and_Institution
C
C     W.L. Taber      (JPL)
C
C$ Literature_References
C
C     None.
C
C$ Version
C
C-    SPICELIB Version 1.1.0, 13-APR-1999 (WLT)
C
C        Fixed a the problem of not incrementing the number of
C        points on one side of the half space/plane when all of
C        the remaining points can be moved to an output path.
C
C-    SPICELIB Version 1.0.0, 2-MAR-1997 (WLT)
C
C
C-&
 
C$ Index_Entries
C
C     Determine the intersection of a polygon and a half-plane
C     Determine the intersection of a polygon and a half-space
C
C-&
 
 
C
C     Spicelib Functions
C
      DOUBLE PRECISION      VDOT
      DOUBLE PRECISION      VDOTG
 
      INTEGER               LNKHL
      INTEGER               LNKTL
      INTEGER               PINHFS
      INTEGER               PINHLF
      INTEGER               PTHDIM
 
      LOGICAL               RETURN
 
C
C     Local Variables
C
C     Here are the mnemonics for the integer codes for inside, outside
C     and on the edge of a half-space.
C
      INTEGER               OUT
      PARAMETER           ( OUT  = -1 )
 
      INTEGER               EDGE
      PARAMETER           ( EDGE = 0 )
 
      INTEGER               IN
      PARAMETER           ( IN   = 1 )
 
      INTEGER               NA
      PARAMETER           ( NA   = 100 )
C
C     The following are mnemonics for the integer codes used to indicate
C     that while traversing a path you are moving away or toward
C     the boundary of the half-space.
C
      INTEGER               AWAY
      PARAMETER           ( AWAY   = 0 )
 
      INTEGER               TOWARD
      PARAMETER           ( TOWARD = 1 )
 
C
C     The following parameter help us manually traverse the linked
C     list component of the path list.
C
      INTEGER               FORWRD
      PARAMETER           ( FORWRD  = 1 )
 
      INTEGER               BCKWRD
      PARAMETER           ( BCKWRD  = 2 )
 
 
      LOGICAL               AFTER
      PARAMETER           ( AFTER = .TRUE. )
 
 
 
      DOUBLE PRECISION      DIR    ( 3 )
      DOUBLE PRECISION      INTRSC ( 3 )
      DOUBLE PRECISION      LP     ( 3 )
      DOUBLE PRECISION      SIGN
      DOUBLE PRECISION      T
      DOUBLE PRECISION      V      ( 3 )
      DOUBLE PRECISION      VERTEX ( 3 )
      DOUBLE PRECISION      W      ( 3 )
 
      INTEGER               AT
      INTEGER               AXIS
      INTEGER               DIM
      INTEGER               FROM
      INTEGER               HEAD
      INTEGER               I
      INTEGER               LAST
      INTEGER               NEW
      INTEGER               NEXT
      INTEGER               NOTZRO
      INTEGER               NOW
      INTEGER               PREV
      INTEGER               PRIOR
      INTEGER               PUT  ( OUT : IN )
      INTEGER               SIZE ( OUT : IN )
      INTEGER               TAIL
      INTEGER               TO
      INTEGER               TWOAGO
      INTEGER               WHERE
 
      LOGICAL               FND
      LOGICAL               PUTALL
      LOGICAL               SHARPN
C
C     Standard SPICE error handling.
C
      IF ( RETURN() ) THEN
         RETURN
      END IF
 
      CALL CHKIN ( 'POLYCT' )
 
C
C     Find the dimension of the input path.
C
      DIM    =  PTHDIM ( PATHS )
 
C
C     Check the one possible error=--an unsupported vector space
C     dimension.
C
      IF ( DIM .NE. 2 .AND. DIM .NE. 3 ) THEN
 
         CALL SETMSG ( 'The dimension supplied to POLYCT was #. '
     .   //            'Only dimensions 2 and 3 are supported. ' )
         CALL ERRINT ( '#', DIM )
         CALL SIGERR ( 'SPICE(BADDIMENSION)' )
         CALL CHKOUT ( 'POLYCT' )
         RETURN
 
      END IF
 
C
C     Get the head and tail of the current path, if they are the
C     same there's nothing do except wipe out the current path.
C
      HEAD = LNKHL ( PATH, PATHS )
      TAIL = LNKTL ( HEAD, PATHS )
 
C
C     Right now, we don't know what the head nodes of the IN and
C     OUT paths will be.  We initialize these nodes to zero.
C     In addition we set the counters for the number of points
C     in each region to zero.
C
      PTHIN        = 0
      PTHOUT       = 0
 
      SIZE( OUT  ) = 0
      SIZE( EDGE ) = 0
      SIZE( IN   ) = 0
 
      PUT ( IN   ) = 0
      PUT ( OUT  ) = 0
 
      IF ( TAIL .EQ. HEAD ) THEN
 
         CALL PTHKIL ( HEAD, PATHS )
         CALL CHKOUT ( 'POLYCT' )
         RETURN
 
      END IF
 
      AT   = HEAD
C
C     Determine whether or not the normal is parallel to  one
C     of the coordinate axes.  If it is we will want to ``sharpen''
C     our segment crossing estimates.
C
      AXIS   = 0
      NOTZRO = 0
 
      DO I = 1, DIM
         IF ( NORML(I) .NE. 0.0D0 ) THEN
            AXIS    = I
            NOTZRO  = NOTZRO + 1
         END IF
      END DO
 
      SHARPN = NOTZRO .EQ. 1
 
 
C
C     Get the first vertex, and move it to the end of the path.
C
C              AT         NEXT                   TAIL
C              o--------->o--------->o--- ... -->o
C              |
C              v
C              |
C              `--------------------------------------------.
C                                                           |
C                                                           v
C                         o--------->o--- ... -->o--------->o
C                         NEXT                   TAIL       NEW
C
C
C     Note that the same number of vertices are in the path
C     after this transformation.
C
      CALL PTHVRT ( AT,   PATHS, VERTXS, FND,   VERTEX, PREV, NEXT )
      CALL PTHINS ( TAIL, AFTER, VERTEX, PATHS, VERTXS, NEW        )
      CALL PTHRPT ( AT,   PATHS, PREV,   NEXT )
 
      TAIL = NEW
 
C
C     There are no coordinates for the last point we've examined
C     in the input path.  But we need some value.  We set the
C     coordinates to zero. Moreover, the last point must be
C     different from the current VERTEX.  So we modify the
C     first coordinate if necessary.
C
      LP (1) = 0.0D0
      LP (2) = 0.0D0
      LP (3) = 0.0D0
 
      IF ( VERTEX(1) .EQ. 0.0D0 ) THEN
         LP(1) = 1.0D0
      END IF
 
C
C     There isn't a LAST region that a vertex was in so we set the
C     variable that records the LAST region to NA (not applicable).
C     Similarly, right now the path is not moving towards or away
C     from the boundary.
C
      LAST  = NA
      PRIOR = NA
 
C
C     There are two possible situations: The path lives in 2-space
C     or the path lives in 3-space.
C
C     We handle the two cases in almost identical fashion.  We
C     don't use the general purpose vector manipulation routines
C     and do everything in a single case so that we can avoid
C     some of the numerical inefficiencies of the general case
C     routines.
C
 
 
      IF      ( DIM .EQ. 2 ) THEN
 
C
C        Determine where the first vertex lies relative to the
C        edge of the half-plane.
C
         WHERE = PINHLF ( VERTEX, REFPT, NORML )
 
C
C        We will circularly examine all the points of the path. The
C        first point (which we just examined) will correspond to the
C        point whose NEXT node is negative.  The path list still
C        contains the same points.  However, the first one is now
C        at NEXT.
C
         DO WHILE ( NEXT .GT. 0 )
C
C           Put the last vertex taken from the input PATH into the
C           appropriate output path. (but only if it is different
C           from the vertex that preceded it---this is why we
C           had to set LP to some point different from the first
C           vertex prior to the first pass through the loop).
C
            IF (      LP(1) .NE. VERTEX(1)
     .           .OR. LP(2) .NE. VERTEX(2) ) THEN
 
               SIZE(WHERE) = SIZE(WHERE) + 1
 
               IF   ( WHERE .NE. EDGE ) THEN
 
 
                  CALL PTHINS ( PUT(WHERE),  AFTER, VERTEX,
     .                                       PATHS, VERTXS, PUT(WHERE) )
 
               ELSE
 
                  CALL PTHINS ( PUT(OUT), AFTER, VERTEX,
     .                                    PATHS, VERTXS, PUT(OUT) )
 
                  CALL PTHINS ( PUT(IN),  AFTER, VERTEX,
     .                                    PATHS, VERTXS, PUT(IN)  )
 
               END IF
 
            END IF
 
C
C           Record the coordinates of the vertex and where it
C           was in relation to the half-plane.
C
            LP(1)  = VERTEX(1)
            LP(2)  = VERTEX(2)
            TWOAGO = LAST
            LAST   = WHERE
 
C
C           Until we know otherwise we need to assert that we should NOT
C           put the remainder of the input path into one of the output
C           paths.
C
            PUTALL = .FALSE.
 
C
C           Copy and remove the next vertex from the input path.
C
            AT = NEXT
            CALL PTHVRT ( AT, PATHS, VERTXS, FND, VERTEX, PREV, NEXT )
            CALL PTHRPT ( AT, PATHS, PREV,   NEXT )
 
C
C           See where this vertex lies in relation to the boundary.
C
            WHERE = PINHLF ( VERTEX, REFPT, NORML )
 
C
C           If we are in the same part of the plane as last time,
C           we don't have to look for an intersection.
C
            IF    ( WHERE .EQ. LAST ) THEN
 
C
C              If we do not have a point in the boundary, we
C              perform some computations that should eventually
C              allow us to skip checking most of the vertices.
C
               IF ( SIZE(EDGE) .EQ. 0 ) THEN
C
C                 See if the path is currently moving away from
C                 or towards the boundary (we regard parallel to
C                 the boundary as moving towards the boundary).
C
                  CALL   VSUBG ( VERTEX, LP, 2, DIR )
                  SIGN = VDOTG ( NORML, DIR, 2   ) * DBLE(WHERE)
 
                  IF ( SIGN .GT. 0.0D0 ) THEN
                     NOW = AWAY
                  ELSE
                     NOW = TOWARD
                  END IF
 
                  IF (      NOW   .EQ. AWAY
     .                .AND. PRIOR .EQ. TOWARD )  THEN
C
C                     Here's the situation.
C
C                     1) We have not crossed the boundary.
C                     2) On the last pass through this bit of
C                        code we were moving toward or parallel
C                        to the boundary.
C                     3) We are now moving away from the boundary.
C                     4) We have a convex path.
C
C                     Conclusion: This path lies entirely on one
C                     side of the boundary.  Set the PUTALL flag
C                     which allows us (a bit later on) to move
C                     all of the path that remains into the
C                     appropriate output path.
C
                      PUTALL = NEXT .GT. 0
 
                  END IF
 
                  PRIOR = NOW
 
               END IF
 
 
            ELSE IF ( WHERE + LAST .EQ. 0 ) THEN
 
C
C              Recall that WHERE is 1, 0, -1 depending on whether
C              the current point is in the open half-plane pointed
C              to by the normal, in the edge, or in the interior of
C              the half-plane opposite to the normal.  Since it cannot
C              be the case that WHERE = LAST = 0 (we'd have executed
C              the first branch above) we must have the current point
C              and the last point on opposite sides of the edge of
C              the half-plane.  We need to compute the intersection
C              with that edge.
C
C              If we let V be the vector from the last point to
C              the current point, then we need to find that t such than
C
C                 LP + t V
C
C              lies in the edge of the half-plane.  That is, we need
C              to find the value of t such than the equation below
C              is satisfied.
C
C                  < LP + tV - REFPT, NORML > = 0
C
C                                            < REFPT - LP, NORML >
C              But this is true only for t = ---------------------
C                                               < V, NORML >
C
C              We compute t and construct an intersection point.
C
               CALL VSUBG ( VERTEX, LP, 2, V )
               CALL VSUBG ( REFPT,  LP, 2, W )
 
               T = VDOTG ( W, NORML, 2 ) /  VDOTG ( V, NORML, 2 )
 
               CALL V2LCOM ( 1.0D0, LP, T, V, INTRSC )
 
C
C              If the normal vector is parallel to one
C              of the coordinate axes, we will want to ``sharpen''
C              up our intersection estimate.
C
               IF ( SHARPN ) THEN
                  INTRSC(AXIS) = REFPT(AXIS)
               END IF
 
C
C              Put the intersection into both lists.
C
               CALL PTHINS( PUT(IN ), AFTER, INTRSC, PATHS, VERTXS,
     .                      PUT(IN )  )
               CALL PTHINS( PUT(OUT), AFTER, INTRSC, PATHS, VERTXS,
     .                      PUT(OUT)  )
 
C
C              We now have one more point in the edge of the
C              two half-planes.
C
               SIZE(EDGE)  = SIZE(EDGE) + 1
 
            END IF
 
C
C           Now move the current vertex (and if appropriate the
C           remaining vertices) into the appropriate list.
C
 
            IF ( WHERE .NE. EDGE ) THEN
 
C
C              Look to see if we can move all of the remaining
C              nodes in a single shot.  If we have two or more
C              points in the boundary, and the last point was not
C              a boundary point, we can classify all remaining
C              points.
C
               IF (       SIZE(EDGE) .GE. 2
     .              .AND. NEXT       .GT. 0 ) THEN
C
C                 There are two ways we can have two or more points
C                 in the boundary
C
C                 1. There is an edge of the polygon contained in
C                    the edge of the half-plane.  In this
C                    case, by convexity all vertices of
C                    the polygon lie in the same
C                    half-plane.  We already know where
C                    one of these is, the side
C                    corresponding to WHERE.
C
C                 2. If the polygon does not have an edge contained in
C                    the edge of the half plane then it can
C                    cross the edge of the half plane
C                    either 0 or 2 times.  We've reached
C                    two.  This means the polygon cannot
C                    again leave the half plane WHERE it
C                    is now.
C
                  PUTALL = .TRUE.
 
               ELSE IF (      WHERE .EQ. TWOAGO
     .                  .AND. LAST  .EQ. EDGE ) THEN
C
C                 This is a rare case.  The LAST vertex was on
C                 the boundary; this vertex and the vertex
C                 two vertices ago lie in the same half-space.
C                 Consequently, the entire path must lie
C                 in a single half-space.  We can put everything that's
C                 left into half space WHERE our current vertex lives.
C
                  PUTALL = NEXT .GT. 0
 
               END IF
 
            END IF
 
C
C           If the PUTALL logical is set to TRUE we can move all
C           of the remaining vertices into the appropriate output
C           path.  If it isn't set, we'll cycle back to the top
C           of the loop and put the current vertex in the
C           appropriate path.
C
            IF ( PUTALL ) THEN
C
C              Move the remaining portion of the original PATH to
C              the appropriate output.
C
               CALL PTHINS ( PUT(WHERE),  AFTER, VERTEX,
     .                                    PATHS, VERTXS, PUT(WHERE) )

               SIZE(WHERE) = SIZE(WHERE) + 1

C
C              If the NEXT node is the TAIL node, there's nothing
C              left to do.
C
               IF ( NEXT .NE. TAIL ) THEN
                  FROM        = NEXT
                  TO          = PATHS(BCKWRD,TAIL)
                  SIZE(WHERE) = SIZE(WHERE) + 1
 
                  CALL LNKXSL ( FROM,  TO,   PATHS )
                  CALL LNKILA ( PUT(WHERE),  FROM, PATHS )
               END IF
C
C              Kill off everything that remains of the original
C              path and set NEXT to zero so that we won't make
C              another pass through the big loop.
C
               CALL PTHKIL ( TAIL, PATHS )
               NEXT = 0
 
            END IF
 
         END DO
 
 
      ELSE IF ( DIM .EQ. 3 ) THEN
C
C        Determine where the first vertex lies relative to the
C        edge of the half-space.
C
         WHERE = PINHFS ( VERTEX, REFPT, NORML )
 
C
C        We will circularly examine all the points of the path. The
C        first point (which we just examined) will correspond to the
C        point whose NEXT node is negative.  The path list still
C        contains the same points.  However, the first one is now
C        at NEXT.
C
         DO WHILE ( NEXT .GT. 0 )
C
C           Put the last vertex taken from the input PATH into the
C           appropriate output path. (but only if it is different
C           from the vertex that preceded it---this is why we
C           had to set LP to some point different from the first
C           vertex prior to the first pass through the loop).
C
            IF (      LP(1) .NE. VERTEX(1)
     .           .OR. LP(2) .NE. VERTEX(2)
     .           .OR. LP(3) .NE. VERTEX(3) ) THEN
 
               SIZE(WHERE) = SIZE(WHERE) + 1
 
               IF   ( WHERE .NE. EDGE ) THEN
 
                  CALL PTHINS ( PUT(WHERE),  AFTER, VERTEX,
     .                                       PATHS, VERTXS, PUT(WHERE) )
 
               ELSE
 
                  CALL PTHINS ( PUT(OUT), AFTER, VERTEX,
     .                                    PATHS, VERTXS, PUT(OUT) )
 
                  CALL PTHINS ( PUT(IN),  AFTER, VERTEX,
     .                                    PATHS, VERTXS, PUT(IN)  )
 
               END IF
 
            END IF
 
C
C           Record the coordinates of the vertex and where it
C           was in relation to the half-space.
C
            LP(1)  = VERTEX(1)
            LP(2)  = VERTEX(2)
            LP(3)  = VERTEX(3)
            TWOAGO = LAST
            LAST   = WHERE
 
C
C           Until we know otherwise we need to assert that we should NOT
C           put the remainder of the input path into one of the output
C           paths.
C
            PUTALL = .FALSE.
 
C
C           Copy and remove the next vertex from the input path.
C
            AT = NEXT
            CALL PTHVRT ( AT, PATHS, VERTXS, FND, VERTEX, PREV, NEXT )
            CALL PTHRPT ( AT, PATHS, PREV,   NEXT )
 
C
C           See where this vertex lies in relation to the boundary.
C
            WHERE = PINHFS ( VERTEX, REFPT, NORML )
 
C
C           If we are in the same part of the plane as last time,
C           we don't have to look for an intersection.
C
            IF    ( WHERE .EQ. LAST ) THEN
 
C
C              If we do not have a point in the boundary, we
C              perform some computations that should eventually
C              allow us to skip checking most of the vertices.
C
               IF ( SIZE(EDGE) .EQ. 0 ) THEN
C
C                 See if the path is currently moving away from
C                 or towards the boundary (we regard parallel to
C                 the boundary as moving towards the boundary).
C
                  CALL   VSUB ( VERTEX, LP, DIR )
                  SIGN = VDOT ( NORML,      DIR ) * DBLE(WHERE)
 
                  IF ( SIGN .GT. 0.0D0 ) THEN
                     NOW = AWAY
                  ELSE
                     NOW = TOWARD
                  END IF
 
                  IF (      NOW   .EQ. AWAY
     .                .AND. PRIOR .EQ. TOWARD )  THEN
C
C                     Here's the situation.
C
C                     1) We have not crossed the boundary.
C                     2) On the last pass through this bit of
C                        code we were moving toward or parallel
C                        to the boundary.
C                     3) We are now moving away from the boundary.
C                     4) We have a convex path.
C
C                     Conclusion: This path lies entirely on one
C                     side of the boundary.  Set the PUTALL flag
C                     which allows us (a bit later on) to move
C                     all of the path that remains into the
C                     appropriate output path.
C
                     PUTALL = NEXT .GT. 0
 
                  END IF
 
                  PRIOR = NOW
 
               END IF
 
 
            ELSE IF ( WHERE + LAST .EQ. 0 ) THEN
 
C
C              Recall that WHERE is 1, 0, -1 depending on whether
C              the current point is in the open half-space pointed
C              to by the normal, in the edge, or in the interior of
C              the half-space opposite to the normal.  Since it cannot
C              be the case that WHERE = LAST = 0 (we'd have executed
C              the first branch above) we must have the current point
C              and the last point on opposite sides of the edge of
C              the half-space.  We need to compute the intersection
C              with that edge.
C
C              If we let V be the vector from the last point to
C              the current point, then we need to find that t such than
C
C                 LP + t V
C
C              lies in the edge of the half-space.  That is, we need
C              to find the value of t such than the equation below
C              is satisfied.
C
C                  < LP + tV - REFPT, NORML > = 0
C
C                                            < REFPT - LP, NORML >
C              But this is true only for t = ---------------------
C                                               < V, NORML >
C
C              We compute t and construct an intersection point.
C
               CALL VSUB ( VERTEX, LP, V )
               CALL VSUB ( REFPT,  LP, W )
 
               T = VDOT ( W, NORML ) /  VDOT ( V, NORML )
 
               CALL VLCOM ( 1.0D0, LP, T, V, INTRSC )
 
C
C              If the normal vector is parallel to one
C              of the coordinate axes, we will want to ``sharpen''
C              up our intersection estimate.
C
               IF ( SHARPN ) THEN
                  INTRSC(AXIS) = REFPT(AXIS)
               END IF
 
C
C              Put the intersection into both lists.
C
               CALL PTHINS( PUT(IN ), AFTER, INTRSC, PATHS, VERTXS,
     .                      PUT(IN )  )
               CALL PTHINS( PUT(OUT), AFTER, INTRSC, PATHS, VERTXS,
     .                      PUT(OUT)  )
 
C
C              We now have one more point in the edge of the
C              two half-spaces.
C
               SIZE(EDGE)  = SIZE(EDGE) + 1
 
            END IF
 
C
C           Now move the current vertex (and if appropriate the
C           remaining vertices) into the appropriate list.
C
            IF ( WHERE .NE. EDGE ) THEN
 
C
C              Look to see if we can move all of the remaining
C              nodes in a single shot.  If we have two or more
C              points in the boundary, and the last point was not
C              a boundary point, we can classify all remaining
C              points.
C
               IF (       SIZE(EDGE) .GE. 2
     .              .AND. NEXT       .GT. 0 ) THEN
C
C                 There are two ways we can have two or more points
C                 in the boundary
C
C                 1. There is an edge of the polygon contained in
C                    the edge of the half-space.  In this
C                    case, by convexity all vertices of
C                    the polygon lie in the same
C                    half-space.  We already know where
C                    one of these is, the side
C                    corresponding to WHERE.
C
C                 2. If the polygon does not have an edge contained in
C                    the edge of the half plane then it can
C                    cross the edge of the half plane
C                    either 0 or 2 times.  We've reached
C                    two.  This means the polygon cannot
C                    again leave the half plane WHERE it
C                    is now.
C
                  PUTALL = NEXT .GT. 0
 
               ELSE IF (      WHERE .EQ. TWOAGO
     .                  .AND. LAST  .EQ. EDGE ) THEN
C
C                 This is a rare case.  The LAST vertex was on
C                 the boundary; this vertex and the vertex
C                 two vertices ago lie in the same half-space.
C                 Consequently, the entire path must lie
C                 in a single half-space.  We can put everything that's
C                 left into half space WHERE our current vertex lives.
C
                  PUTALL = NEXT .GT. 0
 
               END IF
 
            END IF
 
C
C           If the PUTALL logical is set to TRUE we can move all
C           of the remaining vertices into the appropriate output
C           path.  If it isn't set, we'll cycle back to the top
C           of the loop and put the current vertex in the
C           appropriate path.
C
            IF ( PUTALL ) THEN
C
C              Move the remaining portion of the original PATH to
C              the appropriate output.
C
               CALL PTHINS ( PUT(WHERE),  AFTER, VERTEX,
     .                                    PATHS, VERTXS, PUT(WHERE) )

               SIZE(WHERE) = SIZE(WHERE) + 1

C
C              If the NEXT node is the TAIL node, there's nothing
C              left to do.
C
               IF ( NEXT .NE. TAIL ) THEN
 
                  FROM        = NEXT
                  TO          = PATHS(BCKWRD,TAIL)
                  SIZE(WHERE) = SIZE(WHERE) + 1
 
                  CALL LNKXSL ( FROM,  TO,   PATHS )
                  CALL LNKILA ( PUT(WHERE),  FROM, PATHS )
 
               END IF
 
C
C              Kill off everything that remains of the original
C              path and set NEXT to zero so that we won't make
C              another pass through the big loop.
C
               CALL PTHKIL ( TAIL, PATHS )
               NEXT = 0
 
            END IF
 
 
         END DO
 
      END IF
 
C
C     We only return paths that intersect a non-boundary region.
C
 
      IF ( SIZE(IN) .GT. 0 ) THEN
         PTHIN  = LNKHL ( PUT(IN),  PATHS )
      ELSE IF ( PUT(IN) .NE. 0 ) THEN
         CALL PTHKIL ( PUT(IN), PATHS )
      END IF
 
      IF ( SIZE(OUT) .GT. 0 ) THEN
         PTHOUT = LNKHL ( PUT(OUT), PATHS )
      ELSE IF ( PUT(OUT) .NE. 0 ) THEN
         CALL PTHKIL ( PUT(OUT), PATHS )
      END IF
 
      CALL CHKOUT ( 'POLYCT' )
      RETURN
      END
