C$Procedure      POLYFV ( Polygon in Field of View )
 
      SUBROUTINE POLYFV ( APEX, NFOV,  FOVEDG,
     .                    PATH, PATHS, VERTXS,
     .                    PTHIN )
 
C$ Abstract
C
C    Given a polygonal path and field of view, determine the
C    intersection of the path and field of view.
C
C$ Copyright
C
C     Copyright (1997), California Institute of Technology.
C     U.S. Government sponsorship acknowledged.
C
C$ Required_Reading
C
C     None.
C
C$ Keywords
C
C     None.
C
C$ Declarations
 
      IMPLICIT NONE
 
      INTEGER               LBPOOL
      PARAMETER           ( LBPOOL = -5 )
 
      DOUBLE PRECISION      APEX   ( 3 )
      INTEGER               NFOV
      DOUBLE PRECISION      FOVEDG ( 3, * )
      INTEGER               PATH
      INTEGER               PATHS  ( 2, LBPOOL:* )
      DOUBLE PRECISION      VERTXS ( * )
      INTEGER               PTHIN
 
C$ Brief_I/O
C
C     VARIABLE  I/O  DESCRIPTION
C     --------  ---  --------------------------------------------------
C     APEX       I   Is the point where the "sides" of the FOV intersect
C     NFOV       I   Is the number corners to the FOV
C     FOVEDG     I   Is an array of vectors from APEX to each FOV corner
C     PATH       I   Is the head node of a path
C     PATHS     I/O  Is the linked list component of an initialized list
C     VERTXS    I/O  Is the coordinate component of an initialized list
C     PTHIN      O   The points making up the vertices of intersection
C
C$ Detailed_Input
C
C     APEX       is the point in space from which the "pyramid" that
C                represents the field of view emanates.
C
C     NFOV       is the number of corners in the field of view.  Only
C                polygonal fields of view are supported by this routine.
C
C     FOVEDG     is an array of vectors that point along the edges of
C                the field of view.  The vectors are ordered so that
C                the segments
C
C                    FOVEDG(1:3,1) to FOVEDG(1:3,2)
C                    FOVEDG(1:3,2) to FOVEDG(1:3,3)
C                    FOVEDG(1:3,3) to FOVEDG(1:3,4)
C                          .
C                          .
C                          .
C                    FOVEDG(1:3,NFOV-1) to FOVEDG(1:3,NFOV)
C                    FOVEDG(1:3,NFOV)   to FOVEDG(1:3,1)
C
C                each lie in a distinct side of the field of view.
C
C                It is assumed that all vectors in FOVEDG are no-zero.
C                In addition it is assumed that any three edge vectors
C                are linearly independent.
C
C                No checks are performed to ensure these conditions
C                hold.
C
C     PATH       is the node associated with the first vertex of a path
C                in the path list made up of PATHS and VERTXS.
C
C                Note that although the value of PATH is not modified
C                by this routine, upon exit from POLYCT, PATH no longer
C                is the node of a path in the path list.
C
C     PATHS      is the linked list portion an initialized path list
C                that holds the node values of paths.
C
C     VERTXS     is the vertex portion of an initialized path list
C                that holds the coordinates of path vertices.
C
C
C
C$ Detailed_Output
C
C     PATHS      is the linked list portion the path list after
C                the input PATH has been cut into two separate
C                paths---one on each side of the boundary.
C
C     VERTXS     is the vertex portion of the path list after
C                the input PATH has been cut into two separate
C                paths---one on each side of the boundary.
C
C     PTHIN      is the head node associated with the portion of the
C                path that lies inside the FOV specified by APEX, NFOV
C                and FOVEDG
C
C
C$ Parameters
C
C     None.
C
C$ Files
C
C     None.
C
C$ Exceptions
C
C     NOT Error free.
C
C$ Particulars
C
C     This routine determines the intersection of a polygonal path
C     with a "pyramid-shaped" field of view.
C
C$ Examples
C
C     Suppose you want to determine how much of a polygonal plate
C     (used to approximate a portion of an objects surface) lies
C     in the field of view of an instrument.
C
C$ Restrictions
C
C     It is assumed that the coordinates of the vertices of the
C     input path are relative to the same basis vectors as used
C     for APEX and FOVEDG.
C
C$ Author_and_Institution
C
C     W.L. Taber      (JPL)
C
C$ Literature_References
C
C     None.
C
C$ Version
C
C-    SPICELIB Version 1.0.0, 28-MAY-1997 (WLT)
C
C
C-&
 
C$ Index_Entries
C
C     Intersection of a polygon and field of view
C
C-&
C
C     SPICELIB Functions
C
      DOUBLE PRECISION      VDOT
      LOGICAL               RETURN
 
C
C     Local Variables
C
      DOUBLE PRECISION      NORML  ( 3 )
      DOUBLE PRECISION      PNORML ( 3 )
      DOUBLE PRECISION      TEST
 
      INTEGER               I
      INTEGER               MYPATH
      INTEGER               NEXT
      INTEGER               PTHOUT
 
      LOGICAL               UNKNWN
 
 
 
C
C     Standard SPICE error handling.
C
      IF ( RETURN() ) THEN
         RETURN
      END IF
 
      CALL CHKIN ( 'POLYFV' )
 
      IF ( NFOV .LT. 3 ) THEN
 
         CALL SETMSG ( 'A polygonal field of view requires at '
     .   //            'least 3 edges. However, #, edges were '
     .   //            'supplied to POLYFV. ' )
 
         CALL ERRINT ( '#', NFOV )
         CALL SIGERR ( 'SPICE(BADFOVSIZE)'  )
         CALL CHKOUT ( 'POLYFV' )
         RETURN
 
      END IF
C
C     Determine the "sign" to give to the construction of the
C     normals to the FOV sides.
C
      CALL VCRSS  ( FOVEDG(1,1), FOVEDG(1,2), NORML )
      I = 3
 
      UNKNWN = .TRUE.
 
      DO WHILE ( UNKNWN )
 
         TEST = VDOT( NORML, FOVEDG(1,I) )
 
         IF ( TEST .GT. 0.0D0 ) THEN
            NEXT   =  2
            UNKNWN = .FALSE.
         ELSE IF ( TEST .LT. 0.0D0 ) THEN
            NEXT   =  NFOV
            UNKNWN = .FALSE.
         END IF
 
         I = I + 1
 
         IF ( I .GT. NFOV ) THEN
            CALL SETMSG ( 'The field of view specified is '
     .      //            'degenerate.  All edge vectors lie in '
     .      //            'the same plane. ' )
            CALL SIGERR ( 'SPICE(FLATFOV)'  )
            CALL CHKOUT ( 'POLYFV' )
            RETURN
         END IF
 
      END DO
 
      MYPATH = PATH
 
      DO I = 1, NFOV
C
C        Construct the normals to the next side of the FOV
C
         CALL VCRSS  ( FOVEDG(1,I), FOVEDG(1,NEXT), PNORML )
 
         CALL NRMIZE ( PNORML,  3,                  NORML  )
 
C
C        Determine where the PATH
C
         CALL POLYCT ( MYPATH,  PATHS, VERTXS,  APEX, NORML,
     .                 PTHIN,   PTHOUT )
 
 
C
C        We don't need the portion of the path that is on the
C        outside of the last plane.  Get rid of it so we have
C        more room in the path list.
C
         CALL PTHKIL ( PTHOUT, PATHS )
 
         IF ( PTHIN .EQ. 0 ) THEN
            CALL CHKOUT ( 'POLYFV' )
            RETURN
         END IF
 
         MYPATH = PTHIN
         NEXT   = NEXT + 1
 
         IF ( NEXT .GT. NFOV ) THEN
            NEXT = NEXT - NFOV
         END IF
 
      END DO
 
      CALL CHKOUT ( 'POLYFV' )
      RETURN
 
      END
