C$Procedure      POLYFV_2 ( Polygon in Field of View )
 
      SUBROUTINE POLYFV_2 ( APEX, NFOV,  FOVNRM,
     .                      PATH, PATHS, VERTXS,
     .                      PTHIN                )
 
C$ Abstract
C
C     Given a polygonal path and field of view, determine the
C     intersection of the path and field of view.
C
C$ Copyright
C
C     Copyright (1999), California Institute of Technology.
C     U.S. Government sponsorship acknowledged.
C
C$ Required_Reading
C
C     None.
C
C$ Keywords
C
C     PLATE
C
C$ Declarations
 
      IMPLICIT NONE
 
      INTEGER               LBPOOL
      PARAMETER           ( LBPOOL = -5 )
 
      DOUBLE PRECISION      APEX   ( 3 )
      INTEGER               NFOV
      DOUBLE PRECISION      FOVNRM ( 3, * )
      INTEGER               PATH
      INTEGER               PATHS  ( 2, LBPOOL:* )
      DOUBLE PRECISION      VERTXS ( * )
      INTEGER               PTHIN
 
C$ Brief_I/O
C
C     VARIABLE  I/O  DESCRIPTION
C     --------  ---  --------------------------------------------------
C     APEX       I   Is the point where the "sides" of the FOV intersect
C     NFOV       I   Is the number corners to the FOV
C     FOVNRM     I   Is an array of vectors normal to the FOV sides
C     PATH       I   Is the head node of a path
C     PATHS     I/O  Is the linked list component of an initialized list
C     VERTXS    I/O  Is the coordinate component of an initialized list
C     PTHIN      O   The points making up the vertices of intersection
C
C$ Detailed_Input
C
C     APEX       is the point in space from which the "pyramid" that
C                represents the field of view emanates.
C
C     NFOV       is the number of corners in the field of view.  Only
C                polygonal fields of view are supported by this routine.
C
C     FOVNRM     is an array of vectors that are normal and
C                point inwared from each side of the view volume.
C
C                It is assumed that all vectors in FOVNRM are non-zero.
C                In addition it is assumed that any three normal vectors
C                are linearly independent.
C
C                No checks are performed to ensure these conditions
C                hold.  The subroutine INITFV should supply correct
C                normal vectors for each pointing instance.
C
C     PATH       is the node associated with the first vertex of a path
C                in the path list made up of PATHS and VERTXS.
C
C                Note that although the value of PATH is not modified
C                by this routine, upon exit from POLYCT, PATH no longer
C                is the node of a path in the path list.
C
C     PATHS      is the linked list portion an initialized path list
C                that holds the node values of paths.
C
C     VERTXS     is the vertex portion of an initialized path list
C                that holds the coordinates of path vertices.
C
C
C
C$ Detailed_Output
C
C     PATHS      is the linked list portion the path list after
C                the input PATH has been cut into two separate
C                paths---one on each side of the boundary.
C
C     VERTXS     is the vertex portion of the path list after
C                the input PATH has been cut into two separate
C                paths---one on each side of the boundary.
C
C     PTHIN      is the head node associated with the portion of the
C                path that lies inside the FOV specified by APEX, NFOV
C                and FOVEDG
C
C
C$ Parameters
C
C     None.
C
C$ Files
C
C     None.
C
C$ Exceptions
C
C     1) If less than three edges are input, 'SPICE(BADFOVSIZE)'
C        is signalled.
C
C$ Particulars
C
C     This routine determines the intersection of a polygonal path
C     with a "pyramid-shaped" field of view.
C
C$ Examples
C
C     Suppose you want to determine how much of a polygonal plate
C     (used to approximate a portion of an objects surface) lies
C     in the field of view of an instrument.
C
C$ Restrictions
C
C     It is assumed that the coordinates of the vertices of the
C     input path are relative to the same basis vectors as used
C     for APEX and FOVEDG.
C
C$ Author_and_Institution
C
C     J.A. Bytof      (JPL)
C     W.L. Taber      (JPL)
C
C$ Literature_References
C
C     None.
C
C$ Version
C
C-    SPICELIB Version 2.0.1, 08-JUL-1999 (JAB)
C
C        Correct FOVNRM vector is now passed to POLYCT.
C
C-    SPICELIB Version 2.0.0, 27-APR-1999 (JAB)
C
C        Updated from POLYFV, instead of vectors from
C        APEX to each field of view corner, input inward-pointing
C        normal vectors for each FOV side.
C
C-    SPICELIB Version 1.0.0, 28-MAY-1997 (WLT)
C
C
C-&
 
C$ Index_Entries
C
C     Intersection of a polygon and field of view
C
C-&
C
C     SPICELIB Functions
C
 
      LOGICAL               RETURN
 
C
C     Local Variables
C
 
      INTEGER               I
      INTEGER               MYPATH
      INTEGER               PTHOUT
 
C
C     Standard SPICE error handling.
C
      IF ( RETURN() ) THEN
         RETURN
      END IF
 
      CALL CHKIN ( 'POLYFV_2' )
 
      IF ( NFOV .LT. 3 ) THEN
 
         CALL SETMSG ( 'A polygonal field of view requires at '
     .   //            'least 3 edges. However, #, edges were '
     .   //            'supplied to POLYFV. ' )
 
         CALL ERRINT ( '#', NFOV )
         CALL SIGERR ( 'SPICE(BADFOVSIZE)'  )
         CALL CHKOUT ( 'POLYFV_2' )
         RETURN
 
      END IF
 
C
C     What part of the path is inside the field of view?
C
 
      MYPATH = PATH
 
      DO I = 1, NFOV
 
C
C     What part of the remaining path is "inside"
C
 
         CALL POLYCT ( MYPATH,  PATHS, VERTXS,  APEX,  FOVNRM(1,I),
     .                 PTHIN,   PTHOUT )
 
C
C        We don't need the portion of the path that is on the
C        outside of the last plane.  Get rid of it so we have
C        more room in the path list.
C
 
         CALL PTHKIL ( PTHOUT, PATHS )
 
         IF ( PTHIN .EQ. 0 ) THEN
            CALL CHKOUT ( 'POLYFV_2' )
            RETURN
         END IF
 
         MYPATH = PTHIN
 
      END DO
 
      CALL CHKOUT ( 'POLYFV_2' )
      RETURN
 
      END
