C$Procedure      PTHARA ( Path area )
 
      DOUBLE PRECISION FUNCTION PTHARA ( PATH, PATHS, VERTXS )
 
C$ Abstract
C
C    Determine the area of a convex polygon that is specified by
C    a node associated with some vertex of the polygon.
C
C$ Copyright
C
C     Copyright (1999), California Institute of Technology.
C     U.S. Government sponsorship acknowledged.
C
C$ Required_Reading
C
C     None.
C
C$ Keywords
C
C     GEOMETRY
C
C$ Declarations
 
      IMPLICIT NONE
      INTEGER               LBPOOL
      PARAMETER           ( LBPOOL = -5 )
 
      INTEGER               PATH
      INTEGER               PATHS  ( 2, LBPOOL:* )
      DOUBLE PRECISION      VERTXS ( * )
 
C$ Brief_I/O
C
C     VARIABLE  I/O  DESCRIPTION
C     --------  ---  --------------------------------------------------
C     PATH       I   is the node associated with a vertex of a polygon
C     PATHS      I   is the linked list component of an initialized list
C     VERTXS     I   is the coordinate component of an initialized list
C
C     The function returns the area of the polygon.
C
C$ Detailed_Input
C
C     NODE       is the node associated with some vertex of a polygonal
C                path contained in the path list.
C
C     PATHS      is the linked list portion an initialized path list
C                that holds the node values paths.
C
C     VERTXS     is the vertex portion of an initialized path list
C                that holds the coordinates path vertices.
C
C$ Detailed_Output
C
C     The function returns the area of the polygon spanned by the
C     points in the array PTS.
C
C$ Parameters
C
C     LBPOOL     is the lower bound used in the linked list software.
C                It must have the value -5.
C
C$ Files
C
C     None.
C
C$ Exceptions
C
C     1) If DIM is not 2 or 3, the error SPICE(BADDIMENSION) is
C        signalled and the function returns the value 0.
C
C$ Particulars
C
C     This routine computes the area of a polygon that has
C     been stored as a path in a path list.
C
C     If you have the vertices stored as an array, you should
C     not bother building a path.  Instead you should call the
C     routine POLYA.
C
C$ Examples
C
C     Suppose that you need the area of a polygon stored as a
C     path in a path list.  This routine allows you to get the
C     area in a single call
C
C        AREA = PTHARA ( PATH, PATHS, VERTXS )
C
C$ Restrictions
C
C     None.
C
C$ Author_and_Institution
C
C     J.A. Bytof      (JPL)
C     W.L. Taber      (JPL)
C
C$ Literature_References
C
C     None.
C
C$ Version
C
C-    SPICELIB Version 2.0.1, 27-OCT-1999 (JAB)
C
C-         Removed extraneous CHKIN call.
C
C-    SPICELIB Version 2.0.0, 02-FEB-1999 (JAB)
C
C          Removed unneeded declaration and parameter statements.
C
C-    SPICELIB Version 1.0.0, 17-APR-1997 (WLT)
C
C
C-&
 
C$ Index_Entries
C
C     Compute the area of a polygon in the plane
C     Compute the area of a polygon in space
C
C-&
C
C     SPICELIB Functions
C
      INTEGER               PTHDIM
      LOGICAL               RETURN
      DOUBLE PRECISION      VNORM
 
 
 
C
C     Parameters defining the row and column indices of these control
C     elements are given below.
C
      INTEGER               SIZROW
      PARAMETER           ( SIZROW =   1 )
 
      INTEGER               SIZCOL
      PARAMETER           ( SIZCOL =   0 )
 
C
C     Local Variables
C
 
      DOUBLE PRECISION      AREA
      DOUBLE PRECISION      POINT ( 3, 2 )
      DOUBLE PRECISION      TMP   ( 3 )
      DOUBLE PRECISION      VSUM  ( 3 )
      DOUBLE PRECISION      VECT  ( 3, 2 )
 
      INTEGER               COUNT
      INTEGER               DIM
      INTEGER               FRST
      INTEGER               I
      INTEGER               NEXT
      INTEGER               NODE
      INTEGER               PREV
      INTEGER               SCND
      INTEGER               SIZE
      INTEGER               START
 
      LOGICAL               FND
 
C
C     Give the function an initial value.
C
      PTHARA = 0.0D0
 
C
C     Standard SPICE error handling.
C
      IF ( RETURN() ) THEN
         RETURN
      END IF
 
      CALL CHKIN ( 'PTHARA')
 
C
C     If we don't have an "honest node" set the area to zero.
C
      IF ( PATH .LE. 0 ) THEN
         CALL CHKOUT ( 'PTHARA' )
         RETURN
      END IF
C
C     Determine the dimension of the space of the path as well
C     as the maximum number of vertices it can contain.
C
      DIM   = PTHDIM ( PATHS )
      SIZE  = PATHS(SIZROW,SIZCOL)
 
      IF (      DIM .LT. 2
     .     .OR. DIM .GT. 3 ) THEN
 
         CALL SETMSG ( 'The dimension of the space for computing '
     .   //            'the area must be 2 or 3.  The input path '
     .   //            'has dimension #. ' )
         CALL ERRINT ( '#', DIM )
         CALL SIGERR ( 'SPICE(BADDIMENSION)'  )
         CALL CHKOUT ( 'PTHARA' )
         RETURN
 
      END IF
 
 
C
C     Record the starting vertex and current number of pairs processed.
C
      START = PATH
      NODE  = PATH
      COUNT =  0
C
C     Perform the dimension independent initializations.
C
      FRST  =  1
      SCND  =  2
 
C
C     Get the first two vertices.
C
      DO I = 1, 2
 
         CALL PTHVRT( NODE, PATHS, VERTXS, FND, POINT(1,I), PREV, NEXT )
         COUNT = COUNT + 1
 
         IF ( .NOT. FND ) THEN
 
            CALL SETMSG ( 'The path specified has having a vertex '
     .      //            'with node #, is not a legitimate path of '
     .      //            'a path list.  Perhaps the path list was '
     .      //            'not properly initialized via PTHINI or '
     .      //            'perhaps, the linked list component has '
     .      //            'been corrupted. ' )
 
 
            CALL ERRINT ( '#', NODE )
            CALL SIGERR ( 'SPICE(BADPATHNODE)'  )
            CALL CHKOUT ( 'PTHARA' )
            RETURN
         END IF
 
         NODE  = ABS(NEXT)
 
         IF ( NODE .EQ. START ) THEN
            CALL CHKOUT ( 'PTHARA' )
            RETURN
         END IF
 
      END DO
 
 
 
 
 
      IF      ( DIM .EQ. 2 ) THEN
 
         AREA = 0.0D0
 
         CALL VSUBG ( POINT(1,2), POINT(1,1), 2, VECT(1,FRST) )
 
         DO WHILE ( NODE .NE. START )
C
C           Compute the vector from the first vertex to the next
C           vertex in the path.  Keep track of the number of calls
C           to PTHVRT (and hence the number of vertices that have
C           been retrieved).
C
            CALL PTHVRT ( NODE, PATHS, VERTXS, FND, POINT(1,2),
     .                    PREV, NEXT  )
            CALL VSUBG  ( POINT(1,2), POINT(1,1), 2, VECT(1,SCND) )
 
            COUNT = COUNT + 1
C
C           Perform the error check to make sure nothing is went wrong
C           in the above computation.
C
            IF ( .NOT. FND ) THEN
 
               CALL SETMSG ( 'The path specified has having a '
     .         //            'vertex with node #, is not a '
     .         //            'legitimate path of a path list. '
     .         //            'Perhaps the path list was not '
     .         //            'properly initialized via PTHINI or '
     .         //            'perhaps, the linked list component '
     .         //            'has been corrupted. ' )
               CALL ERRINT ( '#', NODE )
               CALL SIGERR ( 'SPICE(BADPATHNODE)'  )
               CALL CHKOUT ( 'PTHARA' )
               RETURN
            END IF
 
C
C           Make sure we haven't exceeded the maximum possible
C           number of vertices in the path list.
C
            IF ( COUNT .GT. SIZE ) THEN
 
               CALL SETMSG ( 'The path containing the specified '
     .         //            'node (#) has more vertices than '
     .         //            'the maximum allowed (#) for the '
     .         //            'path list. Perhaps the path list '
     .         //            'was not properly initialize or '
     .         //            'perhaps the link list component '
     .         //            'has been corrupted. ' )
               CALL ERRINT ( '#', START )
               CALL ERRINT ( '#', SIZE  )
               CALL SIGERR ( 'SPICE(PATHTOOLONG)'  )
               CALL CHKOUT ( 'PTHARA'              )
               RETURN
 
            END IF
 
C
C           Add the contribution of the FRST and SCND vectors to
C           the running sum of the total area.
C
            AREA = AREA + VECT(1,FRST)*VECT(2,SCND)
     .                  - VECT(1,SCND)*VECT(2,FRST)
C
C           Swap FRST and SCND so that we can avoid recomputing
C           the the vector in the last call to VSUBG.
C
            CALL SWAPI ( FRST, SCND )
C
C           Compute the next NODE.
C
            NODE = ABS(NEXT)
 
         END DO
 
 
      ELSE IF ( DIM .EQ. 3 ) THEN
 
C
C        Set the initial vector sum and compute the vector from the
C        first vertex to the second.
C
         VSUM(1) = 0.0D0
         VSUM(2) = 0.0D0
         VSUM(3) = 0.0D0
 
         CALL VSUB ( POINT(1,2), POINT(1,1), VECT(1,FRST) )
 
 
         DO WHILE ( NODE .NE. START )
C
C           Compute the vector from the first vertex to the next
C           vertex in the path.  Keep track of the number of calls
C           to PTHVRT (and hence the number of vertices that have
C           been retrieved).
C
            CALL PTHVRT ( NODE, PATHS, VERTXS, FND, POINT(1,2),
     .                    PREV, NEXT  )
            CALL VSUB   ( POINT(1,2),        POINT(1,1), VECT(1,SCND) )
 
            COUNT = COUNT + 1
C
C           Perform the error check to make sure nothing is went wrong
C           in the above computation.
C
            IF ( .NOT. FND ) THEN
 
               CALL SETMSG ( 'The path specified has having a '
     .         //            'vertex with node #, is not a '
     .         //            'legitimate path of a path list. '
     .         //            'Perhaps the path list was not '
     .         //            'properly initialized via PTHINI or '
     .         //            'perhaps, the linked list component '
     .         //            'has been corrupted. ' )
               CALL ERRINT ( '#', NODE )
               CALL SIGERR ( 'SPICE(BADPATHNODE)'  )
               CALL CHKOUT ( 'PTHARA' )
               RETURN
            END IF
 
C
C           Make sure we haven't exceeded the maximum possible
C           number of vertices in the path list.
C
            IF ( COUNT .GT. SIZE ) THEN
 
               CALL SETMSG ( 'The path containing the specified '
     .         //            'node (#) has more vertices than '
     .         //            'the maximum allowed (#) for the '
     .         //            'path list. Perhaps the path list '
     .         //            'was not properly initialize or '
     .         //            'perhaps the link list component '
     .         //            'has been corrupted. ' )
               CALL ERRINT ( '#', START )
               CALL ERRINT ( '#', SIZE  )
               CALL SIGERR ( 'SPICE(PATHTOOLONG)'  )
               CALL CHKOUT ( 'PTHARA'              )
               RETURN
 
            END IF
 
C
C           Add the contribution of the FRST and SCND vectors to
C           the running sum of the cross products.
C
            CALL VCRSS ( VECT(1,FRST), VECT(1,SCND) , TMP  )
            CALL VADD  ( VSUM,         TMP,           VSUM )
C
C           Swap FRST and SCND so that we can avoid recomputing
C           the the vector in the last call to VSUBG.
C
            CALL SWAPI ( FRST, SCND )
C
C           Compute the next NODE.
C
            NODE = ABS(NEXT)
 
         END DO
 
         AREA = VNORM ( VSUM )
 
      END IF
 
 
      PTHARA = 0.5D0 * ABS ( AREA )
 
      CALL CHKOUT ( 'PTHARA')
      RETURN
      END
