C$Procedure      PTHEXT ( Path Extent)
 
      SUBROUTINE PTHEXT ( PATH, PATHS, VERTXS, LOWER, UPPER )
 
C$ Abstract
C
C    Determine the points on the diagonal of the smallest ``box''
C    that contains all of the points of a path.
C
C$ Copyright
C
C     Copyright (1999), California Institute of Technology.
C     U.S. Government sponsorship acknowledged.
C
C$ Required_Reading
C
C      None.
C
C$ Keywords
C
C      PATHS
C
C$ Declarations
 
      IMPLICIT NONE
      INTEGER               LBPOOL
      PARAMETER           ( LBPOOL = -5 )
 
      INTEGER               PATH
      INTEGER               PATHS  ( 2, LBPOOL:* )
      DOUBLE PRECISION      VERTXS ( * )
      DOUBLE PRECISION      LOWER  ( * )
      DOUBLE PRECISION      UPPER  ( * )
 
C$ Brief_I/O
C
C     VARIABLE  I/O  DESCRIPTION
C     --------  ---  --------------------------------------------------
C     PATH       I   is the node of some vertex belonging to a path
C     PATHS      I   is the linked list component of an initialized list
C     VERTXS     I   is the coordinate component of an initialized list
C     LOWER      O   is the minimum corner of the extent of the path
C     UPPER      O   is the maximum corner of the extent of the path
C
C$ Detailed_Input
C
C     PATH       is the node associated with some vertex of a polygonal
C                path contained in the path list.
C
C     PATHS      is the linked list portion an initialized path list
C                that holds the node values paths.
C
C     VERTXS     is the vertex portion of an initialized path list
C                that holds the coordinates path vertices.
C
C
C$ Detailed_Output
C
C     LOWER      is the point on the extreme diagonal of the box that
C                contains the vertices associated with PATH that
C                satisfies the relationship
C
C                   LOWER(I)  <= VTX(I) for I = 1, 2, ... DIM.
C
C                for any vertex VTX in the path specified by PATH.
C                Moreover LOWER(I) is the maximum value that satisfies
C                the relationship for any vertex of the path..
C
C     UPPER      is the point on the extreme diagonal of the box that
C                contains the vertices associated with PATH that
C                satisfies the relationship
C
C                   LOWER(I)  => VTX(I) for I = 1, 2, ... DIM.
C
C                for any vertex VTX in the path specified by PATH.
C                Moreover LOWER(I) is the minimum value that satisfies
C                the relationship for any vertex of the path..
C
C$ Parameters
C
C     LBPOOL     is the lower bound used in the linked list software.
C                It must have the value -5.
C
C$ Files
C
C     None.
C
C$ Exceptions
C
C     1) If the PATH does not point to a legitimate path in the
C        path list, the error 'SPICE(BADPATH)' is signalled.
C
C$ Particulars
C
C     Given a set of points in a space of arbitrary dimension, there
C     is a smallest box with faces orthogonal to the coordinate
C     coordinate axes that contains the set of points.
C
C     This routine finds the two corners of this box that are on
C     the extreme diagonal of the smallest box that contains the
C     vertices of a path.  The points LOWER and UPPER are those
C     points on this diagonal.  Each coordinate of any vertex, VTX,
C     of the path  satisfies the following relationship
C
C        LOWER(I) <= VTX(I) <= UPPER(I)
C
C
C$ Examples
C
C     Suppose that you need to determine if a path passes through the
C     unit cube bounded by the corners ( 0, 0, ... , 0 ) and
C     ( 1, 1, ... , 1 ).  You can use the routine to quickly
C     eliminate many paths that never enter the unit cube as
C     follows:
C
C        First find the extent of the path.
C
C        CALL PTHEXT ( PATH, PATHS, VERTXS, LOWER, UPPER )
C
C        Now see if the extent overlaps the unit cube.  We do this
C        by finding the "extent" of the intersection.
C
C        OVLAP = .TRUE.
C        I     =  1
C        DO WHILE ( OVLAP .AND. I .LE. DIM )
C
C           OVLAP = MAX( LOWER(I), 0.0D0 ) .LT. MIN( UPPER(I), 1.0D0 )
C           I     = I + 1
C
C        END DO
C
C        IF ( .NOT. OVLAP ) THEN
C
C           the path doesn't intersect the unit cube.
C
C        ELSE
C
C           the extent of the path intersects the unit cube.
C           We need to apply a more sophisticated test to see
C           if the path intersects the unit cube.
C
C        END F
C
C
C$ Restrictions
C
C     None.
C
C$ Author_and_Institution
C
C     J.A. Bytof      (JPL)
C     W.L. Taber      (JPL)
C
C$ Literature_References
C
C     None.
C
C$ Version
C
C
C-    SPICELIB Version 2.0.0, 02-FEB-1999 (JAB)
C
C          Removed unneeded declaration, parameter and
C          assignment statements.
C
C-    SPICELIB Version 1.0.0, 18-APR-1997 (WLT)
C
C
C-&
 
C$ Index_Entries
C
C     Determine the extent of a path
C     Find the smallest rectangle containing a path
C
C-&
C
C     Parameters
C
 
C
C     The control area of PATHS contains 3 elements.  They are:
C
C        The "size" of the pool, that is, the number
C        of nodes in the pool.
C
C        The number of free nodes in the pool.
C
C        The "free pointer," which is the column index of the first free
C        node.
C
C     Parameters defining the row and column indices of these control
C     elements are given below.
C
      INTEGER               SIZROW
      PARAMETER           ( SIZROW =   1 )
 
      INTEGER               SIZCOL
      PARAMETER           ( SIZCOL =   0 )
 
      INTEGER               FORWRD
      PARAMETER           ( FORWRD  = 1 )
 
C
C     The rows of the path list dimension and path count are given
C     below.  The columns must be computed from values stored in
C     the array PATHS.
C
      INTEGER               DIMROW
      PARAMETER           ( DIMROW = 1 )
 
C
C     In-line Functions
C
      INTEGER               AT
C
C     Local Variables
C
      INTEGER               COUNT
      INTEGER               NODE
      INTEGER               I
      INTEGER               J
      INTEGER               PTHDIM
      INTEGER               SIZE
      INTEGER               START
      INTEGER               DIMCOL
 
C
C     In-line function definitions
C
      AT( I, NODE ) = (NODE-1)*PTHDIM + I
C
C     First determine the number of free nodes, and the size of
C     the path list.
C
 
      SIZE   = PATHS(SIZROW,SIZCOL)
 
C
C     Compute the column in PATHS where the dimension and number
C     of paths is stored.
C
      DIMCOL = SIZE + 1
 
      PTHDIM = PATHS(DIMROW,DIMCOL)
 
 
      IF ( PATH .LE. 0 .OR. PATH .GT. SIZE ) THEN
         CALL FILLD ( 0.0D0, PTHDIM, LOWER )
         CALL FILLD ( 0.0D0, PTHDIM, UPPER )
         RETURN
      END IF
 
      START = PATH
      NODE  = PATH
      COUNT = 0
C
C     Begin by storing the first point as the upper and lower
C     part of the diagonal
C
      J     = AT(1,NODE)
      COUNT = COUNT + 1
 
      DO I = 1, PTHDIM
         LOWER(I) = VERTXS(J)
         UPPER(I) = VERTXS(J)
         J        = J + 1
      END DO
C
C     Look up the next node of the path.
C
      NODE = ABS ( PATHS(FORWRD,NODE) )
 
      DO WHILE ( NODE .NE. START )
 
         COUNT = COUNT + 1
C
C        Perform some simple consistency checks.
C
         IF ( COUNT .GT. SIZE  ) THEN
 
            CALL CHKIN  ( 'PTHEXT' )
            CALL SETMSG ( 'The path list has been corrupted. '
     .      //            'While attempting to traverse a path '
     .      //            'in a path list of size #, more than # '
     .      //            'nodes were encountered. ' )
            CALL ERRINT ( '#', SIZE )
            CALL ERRINT ( '#', SIZE )
            CALL SIGERR ( 'SPICE(CORRUPTEDPATHLIST)'  )
            CALL CHKOUT ( 'PTHEXT' )
            RETURN
 
 
         ELSE IF ( NODE .GT. SIZE ) THEN
 
            CALL CHKIN  ( 'PTHEXT' )
            CALL SETMSG ( 'While attempting to traverse a path, '
     .      //            'a vertex node of # was encountered. '
     .      //            'The largest legitimate node value for '
     .      //            'the path list in question is #. ' )
            CALL ERRINT ( '#', NODE )
            CALL ERRINT ( '#', SIZE )
            CALL SIGERR ( 'SPICE(CORRUPTEDPATHLIST)'  )
            CALL CHKOUT ( 'PTHEXT' )
            RETURN
 
         END IF
 
C
C        Update the values of UPPER and LOWER.
C
         J     = AT(1,NODE)
 
         DO I = 1, PTHDIM
            LOWER(I) = MIN( LOWER(I), VERTXS(J) )
            UPPER(I) = MAX( UPPER(I), VERTXS(J) )
            J        = J + 1
         END DO
 
C
C        Fetch the next node.
C
         NODE = ABS ( PATHS(FORWRD,NODE) )
 
      END DO
 
 
      RETURN
      END
