C$Procedure      PTHINF ( Path information )
 
      SUBROUTINE PTHINF ( NODE, PATHS, HEAD, TAIL, SIZE )
 
C$ Abstract
C
C     Return the head, tail and number of vertices contained
C     in a specified path.
C
C$ Copyright
C
C     Copyright (1999), California Institute of Technology.
C     U.S. Government sponsorship acknowledged.
C
C$ Required_Reading
C
C      None.
C
C$ Keywords
C
C      PATHS
C
C$ Declarations
 
      IMPLICIT NONE
      INTEGER               LBPOOL
      PARAMETER           ( LBPOOL = -5 )
 
      INTEGER               NODE
      INTEGER               PATHS  ( 2, LBPOOL:* )
      INTEGER               HEAD
      INTEGER               TAIL
      INTEGER               SIZE
 
C$ Brief_I/O
C
C     VARIABLE  I/O  DESCRIPTION
C     --------  ---  --------------------------------------------------
C     NODE       I   is the node value of some vertex in a path.
C     PATHS      I   is the linked list component of an initialized list
C     HEAD       O   is the node of the first vertex in the path.
C     TAIL       O   is the node of the last vertex in the path
C     SIZE       O   is the total number of vertices in the path.
C
C$ Detailed_Input
C
C     NODE       is the node identifier of some vertex belonging to
C                a path.
C
C     PATHS      is the linked list portion an initialized path list
C                that holds the node pointers of paths.
C
C$ Detailed_Output
C
C     HEAD       is the node of the first vertex in the path specified
C                by NODE.  If the path is invalid, HEAD will be returned
C                with the value 0.
C
C     TAIL       is the node of the last vertex in the path specified
C                by NODE.  If the path is invalid, TAIL will be returned
C                with the value 0.
C
C     SIZE       is the total number of vertices in the path.  If the
C                path is invalid, SIZE will be returned with the
C                value 0.
C
C$ Parameters
C
C     LBPOOL     is the lower bound used in the linked list software.
C                It must have the value -5.
C
C$ Files
C
C     None.
C
C$ Exceptions
C
C     1) If the NODE specified does not belong to a valid path
C        the function returns the value 0 for HEAD, TAIL and SIZE.
C
C$ Particulars
C
C     This function returns the number of vertices belonging to
C     a path specified by a node corresponding to some vertex
C     of the path.
C
C     If the NODE specified is not part of a path, the value 0
C     is returned for all three outputs: HEAD, TAIL, and SIZE.
C
C$ Examples
C
C     Suppose you wish to eliminate all paths from a path list
C     that have length 2 or smaller.
C
C     We shall assume that the array HEADS contains integers that
C     have two meanings.  If HEADS(I) is positive it contains the
C     HEAD of some path in the path list.  If HEADS is non-positive
C     it is an available space and is currently unused.  The following
C     routine shows how you can use the routine PTHINF in conjunction
C     with the routine PTHKIL to remove any path that contains two
C     or fewer vertices.
C
C
C     DO I = 1, NHEADS
C
C        IF ( HEADS(I) .GT. 0 ) THEN
C
C           NODE = HEADS(I)
C
C           CALL PTHINF ( NODE, PATHS, HEAD, TAIL, SIZE )
C
C           IF ( SIZE .LE. 2 ) THEN
C
C              Remove this path from the path list.
C
C              IF ( SIZE .GT. 0 ) THEN
C                 CALL PTHKIL ( HEAD, PATHS, VERTXS )
C              END IF
C
C              Mark this slot in the array heads as being available
C
C              HEADS(I) = 0
C
C           END IF
C
C        END IF
C
C     END DO
C
C
C
C
C$ Restrictions
C
C     None.
C
C$ Author_and_Institution
C
C     J.A. Bytof      (JPL)
C     W.L. Taber      (JPL)
C
C$ Literature_References
C
C     None.
C
C$ Version
C
C-    SPICELIB Version 2.0.0, 02-FEB-1999 (JAB)
C
C          Removed unneeded declaration and parameters statements.
C
C-    SPICELIB Version 1.0.0, 02-APR-1997 (WLT)
C
C
C-&
 
C$ Index_Entries
C
C     Determine the size of a path
C     Determine the head and tail of a path
C
C-&
C
C     Parameters
C
 
C
C     The control area of PATHS contains 3 elements.  They are:
C
C        The "size" of the pool, that is, the number
C        of nodes in the pool.
C
C        The number of free nodes in the pool.
C
C        The "free pointer," which is the column index of the first free
C        node.
C
C     Parameters defining the row and column indices of these control
C     elements are given below.
C
      INTEGER               SIZROW
      PARAMETER           ( SIZROW =   1 )
 
      INTEGER               SIZCOL
      PARAMETER           ( SIZCOL =   0 )
 
      INTEGER               FORWRD
      PARAMETER           ( FORWRD  = 1 )
 
      INTEGER               BCKWRD
      PARAMETER           ( BCKWRD  = 2 )
 
C
C     Free nodes say that's what they are.  The way they say it
C     is by containing the value FREE in their backward pointers.
C     Needless to say, FREE is a value that cannot be a valid pointer.
C
      INTEGER               FREE
      PARAMETER           ( FREE   =  0  )
C
C     Local Variables
C
 
      INTEGER               COUNT
      INTEGER               NEXT
      INTEGER               PLSIZE
      INTEGER               THIS
 
 
 
C
C     First determine the size of the path list.
C
      PLSIZE   = PATHS(SIZROW,SIZCOL)
 
 
      IF ( NODE .LT. 1 .OR. NODE .GT. PLSIZE ) THEN
         SIZE = 0
         HEAD = 0
         TAIL = 0
         RETURN
      END IF
 
C
C     Traverse the path backwards until we reach the head of it or
C     detect an obvious error. Count each valid node as it is
C     encountered.
C
      THIS  = NODE
      COUNT = 1
      NEXT  = PATHS(BCKWRD,NODE)
 
      DO WHILE (     NEXT  .GT. 0
     .         .AND. NEXT  .LE. PLSIZE
     .         .AND. COUNT .LE. PLSIZE )
 
         THIS  = NEXT
         COUNT = COUNT + 1
         NEXT  = PATHS(BCKWRD,THIS)
 
      END DO
 
C
C     Make sure we did not terminate the loop due to an error.
C
      IF (     NEXT  .EQ. FREE
     .    .OR. COUNT .GT. PLSIZE
     .    .OR. NEXT  .GT. PLSIZE  ) THEN
         SIZE = 0
         HEAD = 0
         TAIL = 0
         RETURN
      END IF
 
C
C     THIS now points at the head of the path.
C
      HEAD = THIS
 
C
C     Now traverse the path forward until we reach the tail
C     or detect an obvious error.
C
      THIS = NODE
      NEXT = PATHS(FORWRD,NODE)
 
      DO WHILE (     NEXT  .GT. 0
     .         .AND. NEXT  .LE. PLSIZE
     .         .AND. COUNT .LE. PLSIZE )
 
         THIS  = NEXT
         COUNT = COUNT + 1
         NEXT  = PATHS(FORWRD,THIS)
 
      END DO
C
C     Check for the error conditions.
C
      IF (     NEXT  .EQ. FREE
     .    .OR. COUNT .GT. PLSIZE
     .    .OR. NEXT  .GT. PLSIZE  ) THEN
         SIZE = 0
         HEAD = 0
         TAIL = 0
         RETURN
      END IF
 
C
C     That's it.  THIS is the TAIL node, and COUNT contains the
C     total number of vertices in the path.
C
      TAIL = THIS
      SIZE = COUNT
 
 
      RETURN
      END
