C$Procedure      PTHINI ( Path list initialization )
 
      SUBROUTINE PTHINI ( DIM, MAXPTS, PTHSIZ, PATHS, VERTXS )
 
C$ Abstract
C
C    Initialize a path list.
C
C$ Copyright
C
C     Copyright (1997), California Institute of Technology.
C     U.S. Government sponsorship acknowledged.
C
C$ Required_Reading
C
C     PATHS.
C
C$ Keywords
C
C     PATHS
C
C$ Declarations
 
      IMPLICIT NONE
 
      INTEGER               LBPOOL
      PARAMETER           ( LBPOOL = -5 )
 
      INTEGER               DIM
      INTEGER               MAXPTS
      INTEGER               PTHSIZ
      INTEGER               PATHS  ( 2,   LBPOOL : * )
      DOUBLE PRECISION      VERTXS ( * )
 
C$ Brief_I/O
C
C     VARIABLE  I/O  DESCRIPTION
C     --------  ---  --------------------------------------------------
C     DIM        I   is the dimension of the space where paths come from
C     MAXPTS     I   is the maximum number of vertices allowed
C     PTHSIZ     I   is the size of the array PATHS
C     PATHS     I/O  is a linked list pool giving pointers to vertices
C     VERTXS    I/O  is a list of vertices for various paths.
C
C$ Detailed_Input
C
C     DIM        is the dimension of the space in which the paths
C                will be defined
C
C     MAXPTS     is the maximum number of vertices that can be held
C                in the array VERTXS.
C
C                MAXPTS should be sufficiently large to ensure that
C                all paths that might be constructed can have
C                their vertices stored in VERTXS. The minimum
C                acceptable size is 10.
C
C     PTHSIZ     is the upper bound of the second index of the
C                array PATHS.  PTHSIZ must be at least MAXPTS + 2
C
C     PATHS      is an integer array that will contain the linked
C                list pool for path vertices on output.
C
C                Paths should be declared by the calling routine
C                as shown here
C
C                    INTEGER               PATHS( 2, LBPOOL:PTHSIZ)
C
C                where PTHSIZ is at least MAXPTS + 2.
C
C     VERTXS     is an array of vertices of various paths.  VERTXS
C                should be declared as shown below
C
C                    DOUBLE PRECISION      VERTXS ( DIM * MAXPTS )
C
C
C$ Detailed_Output
C
C     POOL       is an initialized linked list pool from which a set
C                of reserved nodes have been allocated.  There will
C                be at least MAXPTS free nodes remaining in the free
C                list of the linked list pool.
C
C     VERTXS     is an initialized list of vertexes. (The first
C                vertex in the list will be set to zero).
C
C                Note that when in use the N'th point in the array
C                VERTXS will be stored in slot (N-1)*DIM +1 through
C                slot N*DIM of the array VERTXS.
C
C$ Parameters
C
C     LBPOOL     is the lower bound used in the linked list software.
C                It must have the value -5.
C
C$ Files
C
C     None.
C
C$ Exceptions
C
C     1)  If MAXPTS is less than 10, the error 'SPICE(NOTENOUGHROOM)'
C         will be signalled.
C
C     2)  If PTHSIZ is not at least MAXPTS + 2 the error
C        'SPICE(NOTENOUGHNODES)' will be signalled.
C
C$ Particulars
C
C     This routine initializes a ``path list''.  Manipulation of
C     the path list is carried out through use of the various
C     path routines of SPICELIB  (PTH*).
C
C     A path list consists of a customized linked list pool and
C     an array of vertices of various paths.
C
C     Note that the customized linked list contains "a private" list
C     that can be manipulated only by path access routines and
C     as well as public lists that can be manipulated using the
C     linked list software.
C
C     A ``path'' consists of a linked list (specified by PATHS) and
C     vertices of the path.
C
C     Suppose that HEAD is the head of a list in the array PATHS.
C     Then the full list of vertices in the Path can be
C     retrieved by exercising the loop below.
C
C
C        Inline Function Definition
C
C        AT(I,NODE) = (NODE-1)*DIM + I
C
C
C        Start out the loop
C
C        NODE = HEAD
C
C        DO WHILE ( NODE .GT. 0 )
C
C           Retrieve the coordinates of the vertex corresponding to
C           the current node.
C
C           CALL MOVED ( VERTXS(AT(1,NODE)), DIM, VERTEX )
C
C           Do something with the vertex just retrieved.
C
C              .
C              .
C              .
C
C           Get the next node of the path
C
C           NODE = LNKNXT ( NODE, PATHS )
C
C        END DO
C
C
C$ Examples
C
C     Suppose that you need to determine the portion of a path
C     inside a half-space and the portion in the other half-space.
C     The following block of code will perform this task.
C
C     Initialized a path list.
C
C     CALL PTHINI ( DIM, MAXPTS, PTHSIZ, PATHS, VERTXS )
C
C     Insert a new path into the path list.
C
C     CALL PTHMAK ( POINTS, N,   PATHS, VERTX,  PTHBEG )
C
C     Get the head of the portion inside the half-space and the
C     portion outside the half-space.  (Note that a null node
C     is returned if a path is empty.)
C
C     CALL POLYCT ( PTHBEG, PATHS, VERTXS, REFPT, NORML, PTHIN, PTHOUT )
C
C$ Restrictions
C
C     None.
C
C$ Author_and_Institution
C
C     W.L. Taber      (JPL)
C
C$ Literature_References
C
C     None.
C
C$ Version
C
C-    SPICELIB Version 1.0.0, 31-MAR-1997 (WLT)
C
C
C-&
 
C$ Index_Entries
C
C     Initialize a path list.
C
C-&
C     The rows of the path list dimension and path count are given
C     below.  The columns must be computed from values stored in
C     the array PATHS.
C
      INTEGER               DIMROW
      PARAMETER           ( DIMROW = 1 )
 
 
      INTEGER               NPROW
      PARAMETER           ( NPROW  = 2 )
 
C
C     Local Variables
C
 
      INTEGER               DIMCOL
      INTEGER               I
      INTEGER               NEEDND
      INTEGER               NEEDPT
      INTEGER               NPCOL
 
      NEEDPT = 10
      NEEDND = MAXPTS + 2
C
C     Check for obvious failures.
C
      IF ( MAXPTS .LT. 10 ) THEN
 
         CALL CHKIN  ( 'PTHINI' )
         CALL SETMSG ( 'A path list requires room for at least # '
     .   //            'vertices.  The value of MAXPTS supplied '
     .   //            'to PTHINI was #. ' )
 
         CALL ERRINT ( '#', NEEDPT )
         CALL ERRINT ( '#', MAXPTS )
         CALL SIGERR ( 'SPICE(NOTENOUGHROOM)'  )
         CALL CHKOUT ( 'PTHINI' )
         RETURN
 
      ELSE IF ( PTHSIZ .LT. NEEDND ) THEN
 
         CALL CHKIN  ( 'PTHINI' )
         CALL SETMSG ( 'A path list requires that the size of '
     .   //            'the PATHS linked list array, PTHSIZ, be '
     .   //            'at least two more than the number of '
     .   //            'vertices, MAXPTS, that can be stored in '
     .   //            'the array of vertices.  This is not the '
     .   //            'case.  The value of PTHSIZ is #; the '
     .   //            'value of MAXPTS is #. ' )
         CALL ERRINT ( '#', PTHSIZ )
         CALL ERRINT ( '#', MAXPTS )
         CALL SIGERR ( 'SPICE(NOTENOUGHNODES)'  )
         CALL CHKOUT ( 'PTHINI' )
         RETURN
 
      ELSE IF ( DIM .LT. 0 ) THEN
 
         CALL CHKIN  ( 'PTHINI' )
         CALL SETMSG ( 'The dimension, DIM, of the space from '
     .   //            'which path vertices will be taken must '
     .   //            'be at least 1.  The value specified for '
     .   //            'DIM was #. ' )
 
         CALL ERRINT ( '#', DIM )
         CALL SIGERR ( 'SPICE(BADDIMENSION)'  )
         CALL CHKOUT ( 'PTHINI' )
         RETURN
 
      END IF
C
C     Initialize the linked list pool.
C
      CALL LNKINI ( MAXPTS, PATHS )
 
C
C     Store the dimension of the space from which vertices will
C     be selected and the number of paths currently stored.
C
      DIMCOL                  = MAXPTS + 1
      NPCOL                   = MAXPTS + 1
 
      PATHS( DIMROW, DIMCOL ) = DIM
      PATHS( NPROW,  NPCOL  ) = 0
 
C
C     Zero out the first vertex.  This is done, simply to use
C     part of the VERTXS array.  This keeps pedantic compilers
C     from complaining about not using the array VERTXS.
C
      DO I = 1, DIM
         VERTXS(I) = 0.0D0
      END DO
 
 
      RETURN
      END
