C$Procedure      PTHINS ( Path --- Insert a vertex after)
 
      SUBROUTINE PTHINS ( NODE, AFTER, VERTEX, PATHS, VERTXS, NEW )
 
C$ Abstract
C
C     Insert a point into an existing path before/after a user specified
C     vertex of the path.
C
C$ Copyright
C
C     Copyright (1999), California Institute of Technology.
C     U.S. Government sponsorship acknowledged.
C
C$ Required_Reading
C
C      None.
C
C$ Keywords
C
C      PATHS
C
C$ Declarations
 
      IMPLICIT NONE
      INTEGER               LBPOOL
      PARAMETER           ( LBPOOL = -5 )
 
      INTEGER               NODE
      LOGICAL               AFTER
      DOUBLE PRECISION      VERTEX ( * )
      INTEGER               PATHS  ( 2, LBPOOL:* )
      DOUBLE PRECISION      VERTXS ( * )
      INTEGER               NEW
 
C$ Brief_I/O
C
C     VARIABLE  I/O  DESCRIPTION
C     --------  ---  --------------------------------------------------
C     NODE       I   is the node value associated with some path vertex
C     AFTER      I   a logical indicating insertion after or before
C     VERTEX     I   are the coordinates of the point to be inserted
C     PATHS     I/O  is the linked list component of an initialized list
C     VERTXS    I/O  is the coordinate component of an initialized list
C     NEW        O   is the node value associated with the new vertex
C
C$ Detailed_Input
C
C     NODE       is the node value associated with some vertex of
C                a path the is buffered in the path list specified
C                by the arrays PATHS and VERTXS.  The vertex to be
C                inserted into the path will be inserted immediately
C                before or after the vertex specified by NODE depending
C                upon the value of the logical AFTER.
C
C                If NODE is zero, a new path is begun.
C
C     AFTER      indicates whether the new vertex should be inserted
C                after or before the vertex specified by NODE.  If
C                AFTER is .TRUE. the new vertex will be inserted
C                following the specified vertex.  Otherwise it will
C                be inserted before the specified vertex.
C
C                See the particulars section for a tip on a coding
C                trick that will make calling this routine simpler.
C
C     VERTEX     is a double precision array that contains the
C                coordinates of the new point to be inserted into
C                an existing path.
C
C     PATHS      is the linked list portion an initialized path list
C                that holds the node values of paths.
C
C     VERTXS     is the vertex portion of an initialized path list
C                that holds the coordinates of path vertices.
C
C$ Detailed_Output
C
C     PATHS      is the linked list portion the path list after
C                insertion of the new vertex into a path.
C
C     VERTXS     is the vertex portion of the path list after
C                insertion of the new point into the specified path.
C
C     NEW        is the node value associated with the point that
C                was inserted into the path list.
C
C$ Parameters
C
C     LBPOOL     is the lower bound used in the linked list software.
C                It must have the value -5.
C
C$ Files
C
C     None.
C
C$ Exceptions
C
C     1) If the path list does not have room left for inserting
C        a new path vertex the error 'SPICE(OUTOFROOM)' is signalled.
C
C     2) If NODE is zero, a new path is started.
C
C$ Particulars
C
C     This routine allows you to insert a new vertex into an
C     existing path.  This vertex may be inserted before or after
C     any  existing vertex in the path.
C
C     Pictorially, you have one of the following three situations
C
C
C     Insertion between two vertices:
C
C
C          NODE1             NODE2
C
C             o-------*------o
C            /                \
C           /         ^        \
C                     |
C                     |
C           to insert a new vertex between the
C           vertices corresponding to NODE1 and NODE2
C
C
C        CALL PTHINS (  NODE1, .TRUE., VERTEX, PATHS, VERTXS, NEW )
C
C     or
C
C        CALL PTHINS (  NODE2, .FALSE.,VERTEX, PATHS, VERTXS, NEW )
C
C
C
C     Insertion before the first vertex of a path.
C
C
C              * <---- Insert a new vertex before the first
C               .      vertex of a path.
C                .
C                 .
C                 o-----------o
C                              \
C              NODE             \
C                                o---...
C
C
C        CALL ( NODE, .FALSE., VERTEX, PATHS, VERTXS, NEW )
C
C
C     Insertion following the last vertex of a path
C
C
C
C
C        ---o               *  <---- Insert a new vertex following the
C            \             .         last vertex of a path
C             \           .
C              \         .
C               o-------o
C
C                  NODE
C
C
C        CALL ( NODE,  .TRUE.,  VERTEX, PATHS, VERTXS, NEW )
C
C
C     In your code you may wish to create the following
C     variables
C
C        LOGICAL               BEFORE
C        LOGICAL               AFTER
C
C        AFTER  = .TRUE.
C        BEFORE = .NOT. AFTER
C
C
C     Having made these assignments, to insert before a vertex
C     make this subroutine call.
C
C        CALL PTHINS ( NODE, BEFORE, VERTEX, PATHS, VERTXS, NEW )
C
C     To insert after a vertex make the call below.
C
C        CALL PTHINS ( NODE, AFTER,  VERTEX, PATHS, VERTXS, NEW )
C
C$ Examples
C
C     Suppose that a path in 3-space crosses some boundary
C     such as a plane and that you wish to insert the point
C     of intersection with this boundary into the path.
C     The following illustrates how you could use this routine
C     to insert the intersection point.
C
C     We shall assume that we have a logical function, OPSIDE, that
C     can determine if two points are on opposite sides of
C     a boundary and that we have a second routine SOLVE that
C     finds the point of intersection of a segment and a boundary.
C
C     Let BEG point to the beginning of our path.  The routine
C     PTHNXT will find the next point in a path.
C
C     DOUBLE PRECISION      VERTEX( 3, 2 )
C
C     INTEGER               THIS
C     INTEGER               NEXT
C     INTEGER               LAST
C     INTEGER               CURR
C
C     LOGICAL               CROSS
C
C
C     So far we have not crossed the boundary.
C
C     CROSS = .FALSE.
C
C
C     Set up the indices we will use to keep track of the last
C     and current vertex coordinates.
C
C     LAST  = 1
C     CURR  = 2
C
C     Look up the coordinates of the first vertex of the path.
C
C     CALL PTHVRT ( BEG, PATHS, VERTXS, VERTX(1,LAST), FND, PREV, NEXT )
C
C     Find the NEXT node in the path.
C
C     THIS = BEG
C
C     As long as we have not crossed the boundary and there is
C     a NEXT vertex in the path, keep examining segments.
C
C     DO WHILE ( .NOT. CROSS .AND. NEXT .NE. 0)
C
C
C        Get the coordinates associated with the NEXT vertex
C        in the path.
C
C        CALL PTHVRT ( NEXT, PATHS, VERTXS, VERTX(1,CURR), FND, ...
C                      PREV, NEXT )
C
C        See if the path crossed the boundary while travelling
C        from the LAST vertex to the CURRent vertex.
C
C        CROSS =  OPSIDE ( VERTEX(1,LAST), VERTEX(1,CURR) )
C
C        IF ( CROSS ) THEN
C
C           CALL SOLVE ( VERTEX(1,LAST), VERTEX(1,CURR), NEWVER )
C
C           Insert the new vertex into the list BEFORE the current
C           vertex.
C
C           CALL PTHINS ( NEXT, BEFORE, NEWVER, PATHS, VERTXS, NEW )
C
C           That's all we have to do.  CROSS is now TRUE so we
C           will drop out of the DO-WHILE loop.
C
C        ELSE
C
C           Swap the values of LAST and CURR so that we don't
C           have to move the coordinates of the last point
C           we looked up.
C
C           CALL SWAP ( LAST, CURR )
C
C           Get the node value associated with the next vertex.
C
C           THIS = NEXT
C
C        END IF
C
C     END DO
C
C
C
C$ Restrictions
C
C     None.
C
C$ Author_and_Institution
C
C     J.A. Bytof      (JPL)
C     W.L. Taber      (JPL)
C
C$ Literature_References
C
C     None.
C
C$ Version
C
C-    SPICELIB Version 2.0.0, 02-FEB-1999 (JAB)
C
C          Removed unneeded declaration and parameter statements.
C
C-    SPICELIB Version 1.0.0, 01-APR-1997 (WLT)
C
C-&
 
C$ Index_Entries
C
C     Insert a vertex into a path
C
C-&
C
C     SPICELIB Functions
C
      LOGICAL               RETURN
 
C
C     Parameters
C
C
C     The control area of PATHS contains 3 elements.  They are:
C
C        The "size" of the pool, that is, the number
C        of nodes in the pool.
C
C        The number of free nodes in the pool.
C
C        The "free pointer," which is the column index of the first free
C        node.
C
C     Parameters defining the row and column indices of these control
C     elements are given below.
C
      INTEGER               SIZROW
      PARAMETER           ( SIZROW =   1 )
 
      INTEGER               SIZCOL
      PARAMETER           ( SIZCOL =   0 )
 
      INTEGER               NFRROW
      PARAMETER           ( NFRROW =   2 )
 
      INTEGER               NFRCOL
      PARAMETER           ( NFRCOL =   0 )
 
C
C     The rows of the path list dimension and path count are given
C     below.  The columns must be computed from values stored in
C     the array PATHS.
C
      INTEGER               DIMROW
      PARAMETER           ( DIMROW = 1 )
 
 
      INTEGER               NPROW
      PARAMETER           ( NPROW  = 2 )
 
C
C     In-line Functions
C
      INTEGER               AT
C
C     Local Variables
C
      INTEGER               I
      INTEGER               MYNODE
      INTEGER               PTHDIM
      INTEGER               ROOM
      INTEGER               SIZE
      INTEGER               DIMCOL
      INTEGER               NPCOL
      INTEGER               WHERE
 
 
C
C     In-line function definitions
C
      AT( I, WHERE ) = (WHERE-1)*PTHDIM + I
 
 
 
C
C     Standard SPICE error handling.
C
      IF ( RETURN() ) THEN
         RETURN
      END IF
 
      CALL CHKIN  ( 'PTHINS' )
 
 
C
C     Determine the number of free nodes, and the size of
C     the path list.
C
      ROOM   = PATHS(NFRROW,NFRCOL)
      SIZE   = PATHS(SIZROW,SIZCOL)
C
C     Compute the column in PATHS where the dimension and number
C     of paths is stored.
C
      DIMCOL = SIZE + 1
      NPCOL  = SIZE + 1
 
      PTHDIM = PATHS(DIMROW,DIMCOL)
 
C
C     See if there is room in the path list for another vertex.
C
      IF ( ROOM .LT. 1 ) THEN
 
         NEW = 0
 
         CALL SETMSG ( 'There is no space left in the path list '
     .   //            'for adding another vertex. ' )
         CALL SIGERR ( 'SPICE(OUTOFROOM)'  )
         CALL CHKOUT ( 'PTHINS' )
         RETURN
 
      END IF
C
C     Ok, we've got room, allocate a new node
C
      CALL LNKAN ( PATHS, MYNODE )
 
C
C     Depending upon whether AFTER is TRUE of false, insert the
C     new node into the existing path list either after or
C     before the specified node.
C
      IF ( NODE .NE. 0 ) THEN
 
         IF ( AFTER ) THEN
            CALL LNKILA ( NODE,   MYNODE, PATHS )
         ELSE
            CALL LNKILB ( MYNODE, NODE, PATHS )
         END IF
 
      ELSE
C
C        We've created a new path, we need to update the path
C        count.
C
         PATHS ( NPROW, NPCOL ) = PATHS ( NPROW, NPCOL ) + 1
 
      END IF
C
C     Now just copy the coordinates of the new vertex.
C
      DO I = 1, PTHDIM
         VERTXS( AT(I,MYNODE) ) = VERTEX(I)
      END DO
 
      NEW = MYNODE
 
      CALL CHKOUT ( 'PTHINS' )
      RETURN
      END
