C$Procedure      PTHKIL ( Path kill )
 
      SUBROUTINE PTHKIL ( NODE, PATHS )
 
C$ Abstract
C
C     Delete (kill) a path from a path list.
C
C$ Copyright
C
C     Copyright (1999), California Institute of Technology.
C     U.S. Government sponsorship acknowledged.
C
C$ Required_Reading
C
C      None.
C
C$ Keywords
C
C      PATHS
C
C$ Declarations
 
      IMPLICIT NONE
      INTEGER               LBPOOL
      PARAMETER           ( LBPOOL = -5 )
 
      INTEGER               NODE
      INTEGER               PATHS  ( 2, LBPOOL:* )
 
C$ Brief_I/O
C
C     VARIABLE  I/O  DESCRIPTION
C     --------  ---  --------------------------------------------------
C     NODE       I   is a node of some vertex in a path.
C     PATHS     I/O  is the linked list component of an initialized list
C
C$ Detailed_Input
C
C     NODE       is the node identifier of the first vertex
C                belonging to a path that is to be removed from a
C                path list.
C
C     PATHS      is the linked list portion an initialized path list
C                that holds the node values paths.
C
C$ Detailed_Output
C
C     PATHS      is the linked list portion of the path list after
C                the specified path has been removed.
C
C$ Parameters
C
C     LBPOOL     is the lower bound used in the linked list software.
C                It must have the value -5.
C
C$ Files
C
C     None.
C
C$ Exceptions
C
C     1) If the node specified is not part of a valid path, no
C        changes are made to the array PATHS.
C
C     2) If in the attempt to determine the full extent of the
C        path specified by NODE it is detected that the PATHS
C        linked list pool has been corrupted the error
C        'SPICE(CORRUPTEDLIST)' will be signalled.
C
C$ Particulars
C
C     This routine removes a path specified by the input NODE
C     from a path list.  Note that only the linked list portion
C     of the path list requires modification.  Hence the
C     list of vertex coordinates is not supplied to this routine.
C
C$ Examples
C
C
C     Suppose you wish to eliminate all paths from a path list
C     that have length 2 or smaller.  The code fragment below
C     shows how you would use this routine in conjunction with the
C     routine PTHINF to accomplish the task.
C
C     We shall assume that the array HEADS contains integers that
C     have two meanings.  If HEADS(I) is positive it contains the
C     HEAD of some path in the path list.  If HEADS is non-positive
C     it is an available space and is currently unused.  The following
C     routine shows how you can use the routine PTHINF in conjunction
C     with the routine PTHKIL to remove any path that contains two
C     or fewer vertices.
C
C
C     DO I = 1, NHEADS
C
C        IF ( HEADS(I) .GT. 0 ) THEN
C
C           NODE = HEADS(I)
C
C           CALL PTHINF ( NODE, PATHS, HEAD, TAIL, SIZE )
C
C           IF ( SIZE .LE. 2 ) THEN
C
C              Remove this path from the path list.
C
C              IF ( SIZE .GT. 0 ) THEN
C                 CALL PTHKIL ( HEAD, PATHS, VERTXS )
C              END IF
C
C              Mark this slot in the array heads as being available
C
C              HEADS(I) = 0
C
C           END IF
C
C        END IF
C
C     END DO
C
C
C$ Restrictions
C
C     None.
C
C$ Author_and_Institution
C
C     J.A. Bytof      (JPL)
C     W.L. Taber      (JPL)
C
C$ Literature_References
C
C     None.
C
C$ Version
C
C-    SPICELIB Version 2.0.0, 02-FEB-1999 (JAB)
C
C          Removed unneeded parameter and declaration statements.
C
C-    SPICELIB Version 1.0.0, 02-APR-1997 (WLT)
C
C-&
 
C$ Index_Entries
C
C     Remove a path from a path list
C
C-&
C
C     Parameters
C
C     The control area of PATHS contains 3 elements.  They are:
C
C        The "size" of the pool, that is, the number
C        of nodes in the pool.
C
C        The number of free nodes in the pool.
C
C        The "free pointer," which is the column index of the first free
C        node.
C
C     Parameters defining the row and column indices of these control
C     elements are given below.
C
      INTEGER               SIZROW
      PARAMETER           ( SIZROW =   1 )
 
      INTEGER               SIZCOL
      PARAMETER           ( SIZCOL =   0 )
 
      INTEGER               BCKWRD
      PARAMETER           ( BCKWRD  = 2 )
 
C
C     The rows of the path list dimension and path count are given
C     below.  The columns must be computed from values stored in
C     the array PATHS.
C
 
      INTEGER               NPROW
      PARAMETER           ( NPROW  = 2 )
 
C
C     Local Variables
C
      INTEGER               HEAD
      INTEGER               K
      INTEGER               NPATHS
      INTEGER               NPCOL
      INTEGER               PREV
      INTEGER               SIZE
      INTEGER               TAIL
      INTEGER               THIS
 
C
C     First determine the size of the path list.
C
      SIZE   = PATHS(SIZROW,SIZCOL)
 
      IF ( SIZE .LT. 1 ) THEN
         CALL CHKIN  ( 'PTHKIL' )
         CALL SETMSG ( 'The linked list portion of the path list '
     .   //            'has been corrupted (or else not '
     .   //            'initialized).  The SIZE of the linked '
     .   //            'list is set to #.  The of the linked '
     .   //            'list must be positive. ' )
         CALL ERRINT ( '#', SIZE )
         CALL SIGERR ( 'SPICE(CORRUPTEDLIST)'  )
         CALL CHKOUT ( 'PTHKIL' )
         RETURN
      END IF
C
C     Compute the column in PATHS where the dimension and number
C     of paths is stored.
C
      NPCOL  = SIZE + 1
      NPATHS = PATHS(NPROW, NPCOL)
 
      IF ( NODE .GT. SIZE .OR. NODE .LE. 0 ) THEN
         RETURN
      END IF
 
C
C     Traverse the path backwards until we find the head
C     of the list.
C
      THIS = NODE
      PREV = PATHS(BCKWRD,THIS)
      K    = 1
 
      DO WHILE ( PREV .GT. 0 .AND. K .LE. SIZE )
 
         THIS = PREV
         K    = K + 1
         PREV = PATHS(BCKWRD,THIS)
 
      END DO
 
C
C     If we counted more than SIZE nodes, we must have a corrupted
C     linked list component of the path list.
C
      IF ( K .GT. SIZE ) THEN
 
         CALL CHKIN  ( 'PTHKIL' )
         CALL SETMSG ( 'The linked list portion of the path list '
     .   //            'has been corrupted.  A list exists that '
     .   //            'does not have a "head". A likely cause '
     .   //            'of this problem is that some bit of code '
     .   //            'has overwritten the linked list '
     .   //            'component. ' )
         CALL SIGERR ( 'SPICE(CORRUPTEDLIST)'  )
         CALL CHKOUT ( 'PTHKIL' )
         RETURN
 
      END IF
 
C
C     If we reached the head of the list, PREV will be negative.
C     The opposite of PREV is the index of the TAIL node of
C     the linked list.
C
      IF ( PREV .LT. 0 ) THEN
 
         HEAD =   THIS
         TAIL = - PREV
 
         CALL LNKFSL ( HEAD, TAIL, PATHS )
 
         PATHS(NPROW,NPCOL) = NPATHS - 1
 
      END IF
 
      RETURN
      END
