C$Procedure      PTHMAK ( Make a path from a list of vertices )
 
      SUBROUTINE PTHMAK ( NPTS, DIM, PTS, PATHS, VERTXS, BEG )
 
C$ Abstract
C
C     Give an array containing the coordinates of a collection
C     of points and a path-list, add a new path to the path list.
C     The new path will have the vertices whose coordinates are
C     specified by the input array.
C
C$ Copyright
C
C     Copyright (1999), California Institute of Technology.
C     U.S. Government sponsorship acknowledged.
C
C$ Required_Reading
C
C      None.
C
C$ Keywords
C
C      PATHS
C
C$ Declarations
 
      IMPLICIT NONE
 
      INTEGER               LBPOOL
      PARAMETER           ( LBPOOL = -5 )
 
      INTEGER               NPTS
      INTEGER               DIM
      DOUBLE PRECISION      PTS    ( DIM, * )
      INTEGER               PATHS  ( 2, LBPOOL:* )
      DOUBLE PRECISION      VERTXS ( * )
      INTEGER               BEG
 
C$ Brief_I/O
C
C     VARIABLE  I/O  DESCRIPTION
C     --------  ---  --------------------------------------------------
C     NPTS       I   is the number of vertices to be added
C     PTS        I   is an array of coordinates of vertices
C     PATHS     I/O  is the linked list component of an initialized list
C     VERTXS    I/O  is the coordinate component of an initialized list
C     BEG        O   is the beginning node of the new path in the list.
C
C$ Detailed_Input
C
C     NPTS       is the number of points that will be inserted into
C                the path list.
C
C     DIM        is the number of coordinates for each new point
C                that is to be added to the path list.
C
C     PTS        is an array of coordinates of points that will
C                be the vertexes of the new path
C
C     PATHS      is the linked list portion an initialized path list
C                that will hold the node values of the new path.
C
C     VERTXS     is the vertex portion of an initialized path list
C                that will hold the coordinates of the new path.
C
C$ Detailed_Output
C
C     PATHS      is the linked list component of the input path
C                list with the nodes for the new path now in place.
C
C     VERTXS     is the vertex component of the input path list
C                with the coordinates of the input list of points
C                now stored.
C
C     BEG        is the head of the new path in the path list.
C
C$ Parameters
C
C     LBPOOL     is the lower bound used in the linked list software.
C                It must have the value -5.
C
C$ Files
C
C     None.
C
C$ Exceptions
C
C     1) If there is not enough room in the input path list to
C        hold the entire path list, no points will be added
C        to the path list, BEG will be set to zero and the
C        error 'SPICE(OUTOFROOM)' will be signalled.
C
C     2) If the dimension specified as input is not the same as
C        the dimension of the path list, BEG will be set to zero,
C        no points will be added to the path list and the error
C        'SPICE(BADDIMENSION)' will be signalled.
C
C$ Particulars
C
C     This routine allows you to create a new path from an ordered
C     list of points that are specified by their coordinates.
C     The pointer to the head of the list of points is returned
C     allowing you to manipulate the path through use of the
C     various path routines.
C
C     Note that the input array of coordinates must be in the
C     order that you want them to be traversed when following
C     the path they specify.
C
C        Vertex 1    =  PTS (1:DIM,1)
C        Vertex 2    =  PTS (1:DIM,2)
C          .
C          .
C          .
C        Vertex K    =  PTS (1:DIM,K)
C          .
C          .
C          .
C        Vertex NPTS = PTS(1:DIM, NPTS)
C
C$ Examples
C
C     Suppose that you have the coordinates of the vertices
C     of a triangle in 3 space whose vertices are at the end of
C     the standard basis vectors, K, I and J (in that order)
C     and that you would like to add these to
C     a path list.  The code block below will initialize the
C     path list and then insert the vertices of the triangle into
C     the path list.
C
C        INTEGER               MAXPTS
C        PARAMETER           ( MAXPTS = 20 )
C
C        INTEGER               PTHSIZ
C        PARAMETER           ( PTHSIZ = MXPTS + 2 )
C
C        INTEGER               DIM
C        PARAMETER           ( DIM = 3 )
C
C        INTEGER               PATHS ( 2, LBPOOL:PTHSIZ )
C        DOUBLE PRECISION      VERTXS( DIM * MAXPTS )
C
C        DOUBLE PRECISION      PTS ( 3, 3 )
C
C        INTEGER               NPTS
C        INTEGER               BEG
C
C        Here is our list of points.
C
C        NPTS        = 3
C
C        First K ...
C
C        PTS( 1, 1 ) = 0.0D0
C        PTS( 2, 1 ) = 0.0D0
C        PTS( 3, 1 ) = 1.0D0
C
C        ... then I ...
C
C        PTS( 1, 2 ) = 1.0D0
C        PTS( 2, 2 ) = 0.0D0
C        PTS( 3, 2 ) = 0.0D0
C
C        ... finally J.
C
C        PTS( 1, 3 ) = 0.0D0
C        PTS( 2, 3 ) = 1.0D0
C        PTS( 3, 3 ) = 0.0D0
C
C        Initialize the path list.
C
C        CALL  PTHINI ( DIM, MAXPTS, PTHSIZ, PATHS, VERTXS )
C
C
C
C        Insert the KIJ path into the path list.
C
C        CALL PTHMAK ( NPTS, DIM, PTS, PATHS, VERTXS, BEG )
C
C
C$ Restrictions
C
C     None.
C
C$ Author_and_Institution
C
C     J.A. Bytof      (JPL)
C     W.L. Taber      (JPL)
C
C$ Literature_References
C
C     None.
C
C$ Version
C
C-    SPICELIB Version 2.0.0, 02-FEB-1999 (JAB)
C
C          Removed unneeded parameter and declaration statements.
C
C-    SPICELIB Version 1.0.0, 01-APR-1997 (WLT)
C
C-&
 
C$ Index_Entries
C
C     Add a path to a path list from an array of points.
C
C-&
C
C     Parameters
C
C
C     The control area of PATHS contains 3 elements.  They are:
C
C        The "size" of the pool, that is, the number
C        of nodes in the pool.
C
C        The number of free nodes in the pool.
C
C        The "free pointer," which is the column index of the first free
C        node.
C
C     Parameters defining the row and column indices of these control
C     elements are given below.
C
      INTEGER               SIZROW
      PARAMETER           ( SIZROW =   1 )
 
      INTEGER               SIZCOL
      PARAMETER           ( SIZCOL =   0 )
 
      INTEGER               NFRROW
      PARAMETER           ( NFRROW =   2 )
 
      INTEGER               NFRCOL
      PARAMETER           ( NFRCOL =   0 )
 
C
C     The rows of the path list dimension and path count are given
C     below.  The columns must be computed from values stored in
C     the array PATHS.
C
      INTEGER               DIMROW
      PARAMETER           ( DIMROW =   1 )
 
 
      INTEGER               NPROW
      PARAMETER           ( NPROW  = 2 )
 
C
C     In-line Functions
C
      INTEGER               AT
 
C
C     Local Variables
C
      INTEGER               DIMCOL
      INTEGER               I
      INTEGER               NODE
      INTEGER               NPCOL
      INTEGER               PNT
      INTEGER               PREV
      INTEGER               PTHDIM
      INTEGER               ROOM
      INTEGER               SIZE
 
C
C     In-line function definitions.
C
      AT(I,NODE) = (NODE-1)*PTHDIM + I
C
C     First determine the number of free nodes, and the size of
C     the path list.
C
      ROOM   = PATHS(NFRROW,NFRCOL)
      SIZE   = PATHS(SIZROW,SIZCOL)
C
C     Compute the column in PATHS where the dimension and number
C     of paths is stored.
C
      DIMCOL = SIZE + 1
      NPCOL  = SIZE + 1
 
      PTHDIM = PATHS(DIMROW,DIMCOL)
 
      IF ( ROOM .LT. NPTS ) THEN
 
         BEG = 0
 
         CALL CHKIN  ( 'PTHMAK' )
         CALL SETMSG ( 'The path list does not have enough space '
     .   //            'available to store the specified path. '
     .   //            'The specified path requires room for '
     .   //            'storage of # vertices.  However, there '
     .   //            'is room for only # additional vertices. ' )
         CALL ERRINT ( '#', NPTS )
         CALL ERRINT ( '#', ROOM )
         CALL SIGERR ( 'SPICE(OUTOFROOM)'  )
         CALL CHKOUT ( 'PTHMAK' )
         RETURN
 
      ELSE IF ( PTHDIM .NE. DIM ) THEN
 
         BEG = 0
 
         CALL CHKIN  ( 'PTHMAK' )
         CALL SETMSG ( 'The dimension of the path list is #. '
     .   //            'The dimension of the points you are '
     .   //            'attempting to insert into path list is '
     .   //            '#. These are not compatible. ' )
         CALL ERRINT ( '#', PTHDIM )
         CALL ERRINT ( '#', DIM    )
         CALL SIGERR ( 'SPICE(BADDIMENSION)'  )
         CALL CHKOUT ( 'PTHMAK' )
         RETURN
 
      END IF
C
C     Allocate a node for the head of the new path, and insert
C     the new point into the list.
C
      CALL LNKAN ( PATHS, NODE )
 
      DO I = 1, DIM
         VERTXS( AT(I,NODE) ) = PTS( I,1 )
      END DO
C
C     Now insert the remaining points into the path list.
C
      PREV = NODE
      BEG  = NODE
 
      DO PNT = 2, NPTS
C
C        Allocate another node and insert it after the last node
C        for the path.
C
         CALL LNKAN ( PATHS, NODE )
         CALL LNKILA( PREV,  NODE, PATHS )
C
C        Store the coordinates of this point.
C
         DO I = 1, DIM
            VERTXS ( AT(I,NODE) ) = PTS( I,PNT )
         END DO
 
         PREV = NODE
 
      END DO
 
C
C     We've added the new path, increment the number of paths
C     that are stored in the path list.
C
      PATHS(NPROW,NPCOL) = PATHS(NPROW,NPCOL) + 1
      RETURN
 
      END
