C$Procedure      PTHNEW ( Path New )
 
      SUBROUTINE PTHNEW ( VERTEX, PATHS, VERTXS, BEG )
 
C$ Abstract
C
C     Begin a new path by placing the first vertex of the path into
C     a path list.
C
C$ Copyright
C
C     Copyright (1999), California Institute of Technology.
C     U.S. Government sponsorship acknowledged.
C
C$ Required_Reading
C
C      None.
C
C$ Keywords
C
C      PATHS
C
C$ Declarations
 
      IMPLICIT NONE
      INTEGER               LBPOOL
      PARAMETER           ( LBPOOL = -5 )
 
      DOUBLE PRECISION      VERTEX ( * )
      INTEGER               PATHS  ( 2, LBPOOL:* )
      DOUBLE PRECISION      VERTXS ( * )
      INTEGER               BEG
 
C$ Brief_I/O
C
C     VARIABLE  I/O  DESCRIPTION
C     --------  ---  --------------------------------------------------
C     VERTEX     I   is the first vertex of some path to be created
C     PATHS     I/O  is the linked list component of an initialized list
C     VERTXS    I/O  is the coordinate component of an initialized list
C     BEG        O   node associated with the start of the new path
C
C$ Detailed_Input
C
C     VERTEX     is an array of double precision number that contains
C                the coordinates of the first vertex of some path.
C                The dimension of the space from which this point
C                was taken is assumed to be the same as the dimension
C                associated with the path list specified by PATHS and
C                VERTXS.  No check is performed to verify this (the
C                information needed to perform the check is not
C                contained in the calling sequence).
C
C     PATHS      is the linked list portion an initialized path list
C                that holds the node pointers of paths.
C
C     VERTXS     is the vertex portion of an initialized path list
C                that holds the coordinates of path vertices.
C
C$ Detailed_Output
C
C     PATHS      is the linked list portion an of the path list
C                updated to contain information regarding the new path.
C
C     VERTXS     is the vertex portion of the path list
C                updated to contain the information regarding the new
C                path.
C
C     BEG        is the NODE number of the vertex of the newly created
C                path.
C
C$ Parameters
C
C     LBPOOL     is the lower bound used in the linked list software.
C                It must have the value -5.
C
C
C$ Files
C
C     None.
C
C$ Exceptions
C
C     1) If the path list does not have room left for creating
C        a new path with a single vertex the error 'SPICE(OUTOFROOM)'
C        is signalled.
C
C$ Particulars
C
C     This routine creates a new path that contains a single vertex.
C     It is intended to be used in conjunction with the routine
C     PTHINS which allows you to add points to a path.
C
C     The envisioned usage is to initialize a path by calling this
C     routine and then use the routine PTHINS to add more points
C     to the path as they are discovered.  See the Examples section
C     below.
C
C$ Examples
C
C     This example illustrated the expected usage of this routine
C     together with the routine PTHINS to construct a path one
C     vertex at a time.  We assume that a path list has already
C     been initialized via a call to PTHINI.  See that routine
C     for more details.
C
C     We assume that the routine GETVRT returns the next vertex
C     from some vertex generation software.  The logical FOUND
C     indicates success or failure in the vertex generation routine.
C
C        INTEGER               BEG
C        INTEGER               NEXT
C        INTEGER               THIS
C        DOUBLE PRECISION      VERTEX ( 3 )
C        LOGICAL               FOUND
C
C        LOGICAL               AFTER
C
C        AFTER = .TRUE.
C
C
C        CALL GETVRT ( VERTEX, FOUND )
C
C        IF ( FOUND ) THEN
C
C           Construct the first vertex of the path.
C
C           CALL PTHNEW ( VERTEX, PATHS, VERTXS, BEG )
C
C        END IF
C
C        We copy the beginning of the path to the variable THIS
C        so that we can use BEG is subsequent manipulation of
C        the path once construction of it has been completed.
C
C        THIS = BEG
C
C        Now add vertices onto the path until none are returned
C        by the vertex generation routine.
C
C        CALL GETVRT ( VERTEX, FOUND )
C
C        DO WHILE ( FOUND )
C
C           CALL PTHINS ( AFTER, THIS, VERTEX, PATHS, VERTXS, NEXT )
C
C           Set the current vertex to NEXT and get the next
C           vertex of the path.
C
C           THIS = NEXT
C           CALL GETVRTX( VERTEX, FOUND )
C
C        END DO
C
C
C        The path has now been completed.  Perform any other
C        manipulation on it that is needed.
C
C           .
C           .
C           .
C$ Restrictions
C
C     None.
C
C$ Author_and_Institution
C
C     J.A. Bytof      (JPL)
C     W.L. Taber      (JPL)
C
C$ Literature_References
C
C     None.
C
C$ Version
C
C-    SPICELIB Version 2.0.0, 02-FEB-1999 (JAB)
C
C          Removed some unneeded parameter and declaration
C          statements.
C
C-    SPICELIB Version 1.0.0, 19-JUN-1997 (WLT)
C
C
C-&
 
C$ Index_Entries
C
C     Create a new path from a single point
C
C-&
C
C     Parameters
C
 
C
C     The control area of PATHS contains 3 elements.  They are:
C
C        The "size" of the pool, that is, the number
C        of nodes in the pool.
C
C        The number of free nodes in the pool.
C
C        The "free pointer," which is the column index of the first free
C        node.
C
C     Parameters defining the row and column indices of these control
C     elements are given below.
C
      INTEGER               SIZROW
      PARAMETER           ( SIZROW =   1 )
 
      INTEGER               SIZCOL
      PARAMETER           ( SIZCOL =   0 )
 
      INTEGER               NFRROW
      PARAMETER           ( NFRROW =   2 )
 
      INTEGER               NFRCOL
      PARAMETER           ( NFRCOL =   0 )
 
C
C     The rows of the path list dimension and path count are given
C     below.  The columns must be computed from values stored in
C     the array PATHS.
C
 
      INTEGER               DIMROW
      PARAMETER           ( DIMROW = 1 )
 
 
      INTEGER               NPROW
      PARAMETER           ( NPROW  = 2 )
 
C
C     In-line Functions
C
      INTEGER               AT
C
C     Local Variables
C
      INTEGER               NODE
      INTEGER               I
      INTEGER               PTHDIM
      INTEGER               ROOM
      INTEGER               SIZE
      INTEGER               DIMCOL
      INTEGER               NPCOL
 
 
C
C     In-line function definitions
C
      AT( I, NODE ) = (NODE-1)*PTHDIM + I
C
C     First determine the number of free nodes, and the size of
C     the path list.
C
      ROOM   = PATHS(NFRROW,NFRCOL)
      SIZE   = PATHS(SIZROW,SIZCOL)
C
C     Compute the column in PATHS where the dimension and number
C     of paths is stored.
C
      DIMCOL = SIZE + 1
      NPCOL  = SIZE + 1
 
      PTHDIM = PATHS(DIMROW,DIMCOL)
 
 
      IF ( ROOM .LT. 1 ) THEN
 
         BEG = 0
 
         CALL CHKIN  ( 'PTHNEW' )
         CALL SETMSG ( 'There is no space left in the path list '
     .   //            'for beginning a new path. ' )
         CALL SIGERR ( 'SPICE(OUTOFROOM)'  )
         CALL CHKOUT ( 'PTHNEW' )
         RETURN
 
      END IF
C
C     Allocate a node from the linked list pool.
C
      CALL LNKAN ( PATHS, BEG )
C
C     Copy the node and then copy the coordinates of the
C     VERTEX into the storage array.
C
      NODE = BEG
 
      DO I = 1, PTHDIM
         VERTXS( AT(I,NODE) ) = VERTEX(I)
      END DO
C
C     Increment the number of paths.
C
      PATHS(NPROW, NPCOL) = PATHS(NPROW, NPCOL) + 1
 
C
C     That's all.
C
      RETURN
      END
