C$Procedure      PTHRPT ( Path remove point)
 
      SUBROUTINE PTHRPT ( NODE, PATHS, PREV, NEXT )
 
C$ Abstract
C
C     Remove a vertex from a path in a path list
C
C$ Copyright
C
C     Copyright (1999), California Institute of Technology.
C     U.S. Government sponsorship acknowledged.
C
C$ Required_Reading
C
C      None.
C
C$ Keywords
C
C      PATHS
C
C$ Declarations
 
      IMPLICIT NONE
      INTEGER               LBPOOL
      PARAMETER           ( LBPOOL = -5 )
 
      INTEGER               NODE
      INTEGER               PATHS  ( 2, LBPOOL:* )
      INTEGER               PREV
      INTEGER               NEXT
 
C$ Brief_I/O
C
C     VARIABLE  I/O  DESCRIPTION
C     --------  ---  --------------------------------------------------
C     NODE       I   is the node value associated with some path
C     PATHS     I/O  is the linked list component of an initialized list
C     PREV       O   is the node value of the vertex that preceded NODE
C     NEXT       O   is the node value of the vertex that followed NODE
C
C$ Detailed_Input
C
C     NODE       is the node value associated with some vertex in a
C                path contained in the path list.  This vertex will
C                be deleted from the path.
C
C     PATHS      is the linked list portion an initialized path list
C                that holds the node values paths.
C
C
C$ Detailed_Output
C
C     PATHS      is the linked list portion of the path list after
C                the vertex corresponding to NODE has been removed
C                from the path list.
C
C     PREV       is the node value associated with the vertex that
C                preceded the vertex associated with NODE (prior to
C                the deletion of the vertex associated with NODE).
C                If the deleted vertex was the first vertex of the path,
C                PREV will be the opposite of the node value associated
C                with the last vertex of the path.  If there was
C                only one vertex, PREV will be returned with the
C                value 0.
C
C
C     NEXT       is the node value associated with the vertex that
C                followed the vertex associated with NODE (prior to
C                the deletion of the vertex associated with NODE).
C                If the deleted vertex was the last vertex of the path,
C                NEXT will be the opposite of the node value associated
C                with the first vertex of the path.  If there was
C                only one vertex, NEXT will be returned with the
C                value 0.
C
C$ Parameters
C
C     LBPOOL     is the lower bound used in the linked list software.
C                It must have the value -5.
C
C$ Files
C
C     None.
C
C$ Exceptions
C
C     Error free.
C
C     1) If node does not correspond to some vertex of a path
C        in the path list, PATHS is returned unchanged and both
C        PREV and NEXT will be returned as zero.
C
C$ Particulars
C
C     This routine allows you to remove a single vertex from
C     a path.  For example suppose the path contains vertexes
C     as shown below.
C
C                  NODE to remove
C                 o
C        o       / \
C         \     /   \     o-----o
C          \   /     \   /
C           \ /       \ /
C            o         o
C
C
C     After remove of the specified vertex the path becomes
C
C
C        o
C         \               o-----o
C          \             /
C           \           /
C            o---------o
C
C
C$ Examples
C
C     Suppose you wish to remove vertices from a path that are
C     are interior points of a line segment.  The following
C     block of code illustrates how you might use this routine
C     to perform this task.
C
C     Start with the head node of a path.
C
C     DOUBLE PRECISION      VERTEX(3,3)
C     INTEGER               LEFT
C     INTEGER               MIDDLE
C     INTEGER               RIGHT
C
C
C     LEFT   = 1
C     MIDDLE = 2
C     RIGHT  = 3
C
C
C     Get the first three points.  Note that if there are not
C     3 points, FND will be returned as FALSE by the last
C     call to PTHVRT.
C
C     CALL PTHVRT ( HEAD, PATHS, VERTXS, FND, VERTEX(1,LEFT), ...
C                   TAIL, MIDPT )
C
C     CALL PTHVRT ( MIDPT, PATHS, VERTXS, FND, VERTEX(1,MIDDLE), ...
C                   FIRST, LAST )
C
C     CALL PTHVRT ( LAST,  PATHS, VERTXS,  FND, VERTEX(1,RIGHT), ...
C                   PREV,  NEXT )
C
C
C     We will take advantage of the fact that FND is true
C     only if the vertex requested has a positive node value.
C
C     DO WHILE ( FND )
C
C        Use the user supplied function INLINE that determines
C        whether or not the segment from the first to the third
C        point passes through the second point.
C
C        IF ( INLINE( VERTEX(1,LEFT),   ...
C                     VERTEX(1,MIDDLE), ...
C                     VERTEX(1,RIGHT) ) ) THEN
C
C           The segment from LEFT to RIGHT passed through the
C           MIDDLE point.  We drop it from the list.
C
C           CALL PTHRPT ( MIDPT,  PATHS, PREV, NEXT )
C
C           To avoid moving coordinates we simply swap the
C           values of MIDDLE and RIGHT
C
C           CALL SWAPI  ( MIDDLE, RIGHT )
C
C        ELSE
C
C           The points in hand are not aligned.  Cycle the
C           values of our "pointers" so that we don't have
C           to move the coordinates of the points.
C
C           TEMP   = LEFT
C           LEFT   = MIDDLE
C           MIDDLE = RIGHT
C           RIGHT  = TEMP
C
C        END IF
C
C        LAST = NEXT
C        CALL PTHVRT ( LAST,  PATHS, VERTXS,  FND, VERTEX(1,RIGHT), ...
C                      PREV,  NEXT )
C
C     END DO
C
C$ Restrictions
C
C     None.
C
C$ Author_and_Institution
C
C     J.A. Bytof      (JPL)
C     W.L. Taber      (JPL)
C
C$ Literature_References
C
C     None.
C
C$ Version
C
C-    SPICELIB Version 2.0.0, 02-FEB-1999 (JAB)
C
C          Removed some unneeded declaration and parameter statements.
C
C-    SPICELIB Version 1.0.0, 03-APR-1997 (WLT)
C
C-&
 
C$ Index_Entries
C
C     Remove a vertex from a path
C
C-&
C
C     Parameters
C
 
C
C     The control area of PATHS contains 3 elements.  They are:
C
C        The "size" of the pool, that is, the number
C        of nodes in the pool.
C
C        The number of free nodes in the pool.
C
C        The "free pointer," which is the column index of the first free
C        node.
C
C     Parameters defining the row and column indices of these control
C     elements are given below.
C
      INTEGER               SIZROW
      PARAMETER           ( SIZROW =   1 )
 
      INTEGER               SIZCOL
      PARAMETER           ( SIZCOL =   0 )
 
      INTEGER               FORWRD
      PARAMETER           ( FORWRD  = 1 )
 
      INTEGER               BCKWRD
      PARAMETER           ( BCKWRD  = 2 )
 
      INTEGER               FREE
      PARAMETER           ( FREE   = 0 )
C
C     The rows of the path list dimension and path count are given
C     below.  The columns must be computed from values stored in
C     the array PATHS.
C
 
      INTEGER               NPROW
      PARAMETER           ( NPROW  = 2 )
 
C
C     In-line Functions
C
      LOGICAL               INRNGE
C
C     Local Variables
C
      INTEGER               APREV
      INTEGER               ANEXT
      INTEGER               NPATH
      INTEGER               SIZE
      INTEGER               NPCOL
 
 
 
      INRNGE( NODE ) = NODE .LE. SIZE .AND. NODE .GT. 0
C
C     First determine the number of free nodes, and the size of
C     the path list.
C
      SIZE   = PATHS(SIZROW,SIZCOL)
C
C     Compute the location of, and retrieve, the current number of
C     paths in the path list.
C
      NPCOL  = SIZE + 1
      NPATH  = PATHS( NPROW, NPCOL )
C
C     Make sure the node is in range.
C
      IF ( .NOT. INRNGE(NODE) ) THEN
 
C
C        There's nothing to return, assign zero to PREV and NEXT
C
         PREV = 0
         NEXT = 0
         RETURN
 
      END IF
 
C
C     The node is in range, get the previous and next nodes
C     of the path.
C
      PREV = PATHS(BCKWRD,NODE)
      NEXT = PATHS(FORWRD,NODE)
C
C     If NODE happens to be a free node then there's nothing to
C     do.
C
      IF ( PREV .EQ. FREE .OR. NEXT .EQ. FREE ) THEN
         NEXT = 0
         PREV = 0
         RETURN
      END IF
C
C     Get the absolute values of PREV and NEXT.  Make sure
C     both absolute values are in range.
C
      APREV = ABS(PREV)
      ANEXT = ABS(NEXT)
 
      IF ( INRNGE(APREV) .AND. INRNGE(ANEXT) ) THEN
C
C        If both PREV and NEXT are negative, this
C        is a singleton path that will be removed
C        We need to update the path count.
C
         IF ( PREV .LT. 0 .AND. NEXT .LT. 0 ) THEN
            NPATH                 = NPATH - 1
            PATHS( NPROW, NPCOL ) = NPATH
            PREV                  = 0
            NEXT                  = 0
         END IF
 
         CALL LNKFSL ( NODE, NODE, PATHS )
 
      ELSE
C
C        There is some kind of problem with this node.
C        It can't be a node for a correct path.
C        Return 0 for both PREV and NEXT
C
         PREV = 0
         NEXT = 0
 
      END IF
 
      RETURN
      END
