C$Procedure      PTHSTS ( Path Status )
 
      SUBROUTINE PTHSTS ( PATHS, DIM, SIZE, NFREE, NPATHS )
 
C$ Abstract
C
C     Return the current status of the linked list component of
C     a path list.
C
C$ Copyright
C
C     Copyright (1999), California Institute of Technology.
C     U.S. Government sponsorship acknowledged.
C
C$ Required_Reading
C
C      None.
C
C$ Keywords
C
C      PATHS
C
C$ Declarations
 
      IMPLICIT NONE
 
      INTEGER               LBPOOL
      PARAMETER           ( LBPOOL = -5 )
 
      INTEGER               PATHS  ( 2, LBPOOL:* )
      INTEGER               DIM
      INTEGER               SIZE
      INTEGER               NFREE
      INTEGER               NPATHS
 
C$ Brief_I/O
C
C     VARIABLE  I/O  DESCRIPTION
C     --------  ---  --------------------------------------------------
C     PATHS      I   linked list component of an initialized list
C     DIM        O   dimension of points in the path list
C     SIZE       O   maximum number of vertices the path list can hold
C     NFREE      O   room still available in the path list
C     NPATHS     O   current number of paths stored in the path list
C
C$ Detailed_Input
C
C     PATHS      is the linked list portion an initialized path list
C                that holds the node values paths.
C
C
C$ Detailed_Output
C
C     DIM        is the dimension of the vertices in the path list.
C
C     SIZE       is the maximum number of vertices that can be stored
C                in the path list.
C
C     NFREE      is the number of vertices that may be inserted into the
C                path list at this time.
C
C                The number of vertices currently stored in the path
C                list is SIZE - NFREE.
C
C     NPATHS     is the current number of paths that is stored in the
C                path list.
C
C$ Parameters
C
C     None.
C
C$ Files
C
C     None.
C
C$ Exceptions
C
C     Error free.
C
C$ Particulars
C
C     This is a utility routine that allows you to retrieve the
C     current status of a path list from the linked list
C     component of the path list.   This routine allows you to
C     develop higher level routines without having to be concerned
C     with locations of these status items in the path list.
C
C$ Examples
C
C     Suppose that you are going to write a routine that inserts
C     points into a path list and that the number of points that
C     will need to be inserted is known but the points will be
C     created sequentially.  This routine allows you to check
C     the path list to make sure that room is available for the
C     points prior the calls to PTHINS which will insert them.
C     First call PTHSTS to determine the number of free storage
C     locations available in the path list.
C
C        CALL PTHSTS ( PATHS, DIM, SIZE, NFREE, NPATHS )
C
C        If the room needed for the new points is greater than
C        the amount of free space, diagnose the problem and quit.
C
C        IF ( NEEDED .GT. NFREE ) THEN
C
C           WRITE (*,*) 'There isn''t room to store the next path.'
C           WRITE (*,*) 'Spaces is needed for: ', NEEDED, ' points.'
C           WRITE (*,*) 'However, there is only room for ', NFREE
C           WRITE (*,*) 'points.'
C
C        END IF
C
C        Insert the first point.
C
C        CALL PTHNEW ( POINT, PATHS, VERTXS, BEG )
C
C        NODE  = BEG
C        AFTER = .TRUE.
C
C        Now insert the remaining points one at a time.
C
C        DO I = 2, NEEDED
C
C           Construct the next point (the user routine GETPNT magically
C           performs this task).
C
C           CALL GETPNT ( POINT )
C
C           Now insert this point AFTER the last point we inserted.
C
C           CALL PTHINS ( NODE, AFTER, POINT, PATHS, VERTXS, NEW )
C
C           NODE = NEW
C
C        END DO
C
C$ Restrictions
C
C     None.
C
C$ Author_and_Institution
C
C     J.A. Bytof      (JPL)
C     W.L. Taber      (JPL)
C
C$ Literature_References
C
C     None.
C
C$ Version
C
C
C-    SPICELIB Version 2.0.0, 02-FEB-1999 (JAB)
C
C          Removed some unneeded declaration statements.
C
C-    SPICELIB Version 1.0.0, 11-JUN-1997 (WLT)
C
C
C-&
 
C$ Index_Entries
C
C     Determine the status of a path list
C
C-&
 
 
C
C     The control area of PATHS contains 3 elements.  They are:
C
C        The "size" of the pool, that is, the number
C        of nodes in the pool.
C
C        The number of free nodes in the pool.
C
C        The "free pointer," which is the column index of the first free
C        node.
C
C     Parameters defining the row and column indices of these control
C     elements are given below.
C
      INTEGER               SIZROW
      PARAMETER           ( SIZROW =   1 )
 
      INTEGER               SIZCOL
      PARAMETER           ( SIZCOL =   0 )
 
      INTEGER               NFRROW
      PARAMETER           ( NFRROW =   2 )
 
      INTEGER               NFRCOL
      PARAMETER           ( NFRCOL =   0 )
 
C
C     The rows of the path list dimension and path count are given
C     below.  The columns must be computed from values stored in
C     the array PATHS.
C
      INTEGER               DIMROW
      PARAMETER           ( DIMROW = 1 )
 
 
      INTEGER               NPROW
      PARAMETER           ( NPROW  = 2 )
 
C
C     Local Variables
C
      INTEGER               DIMCOL
      INTEGER               NPCOL
 
 
C
C     First determine the number of free nodes, and the size of
C     the path list.
C
      NFREE  = PATHS(NFRROW,NFRCOL)
      SIZE   = PATHS(SIZROW,SIZCOL)
C
C     Compute the column in PATHS where the dimension and number
C     of paths is stored.
C
      DIMCOL = SIZE + 1
      NPCOL  = SIZE + 1
 
      DIM    = PATHS(DIMROW,DIMCOL)
      NPATHS = PATHS(NPROW, NPCOL )
 
      RETURN
      END
