C$Procedure      PTHVRT ( Path vertex )
 
      SUBROUTINE PTHVRT ( NODE, PATHS, VERTXS, FND, VERTEX, PREV, NEXT )
 
C$ Abstract
C
C     Return the coordinates of a vertex associated with a
C     particular node of a path and return the pointers to the
C     previous and next vertices of the path.
C
C$ Copyright
C
C     Copyright (1999), California Institute of Technology.
C     U.S. Government sponsorship acknowledged.
C
C$ Required_Reading
C
C      None.
C
C$ Keywords
C
C      PATHS
C
C$ Declarations
 
      IMPLICIT NONE
 
      INTEGER               LBPOOL
      PARAMETER           ( LBPOOL = -5 )
 
      INTEGER               NODE
      INTEGER               PATHS  ( 2, LBPOOL:* )
      DOUBLE PRECISION      VERTXS ( * )
      LOGICAL               FND
      DOUBLE PRECISION      VERTEX ( * )
      INTEGER               PREV
      INTEGER               NEXT
 
C$ Brief_I/O
C
C     VARIABLE  I/O  DESCRIPTION
C     --------  ---  --------------------------------------------------
C     NODE       I   is the node associated with some vertex of a path
C     PATHS      I   is the linked list component of an initialized list
C     VERTXS     I   is the coordinate component of an initialized list
C     FND        O   logical indicating success of request
C     VERTEX     O   the coordinates of the specified point
C     PREV       O   the node of the previous vertex on the path.
C     NEXT       O   the node of the next vertex on the path
C
C$ Detailed_Input
C
C     NODE       is the node associated with some vertex of a path
C                whose coordinates are to be returned.
C
C     PATHS      is the linked list portion an initialized path list
C                that holds the node values paths.
C
C     VERTXS     is the vertex portion of an initialized path list
C                that holds the coordinates path vertices.
C
C$ Detailed_Output
C
C     FND        is a logical flag indicating whether or not the
C                requested vertex is available.  If NODE is
C                a valid node of a vertex of some path in the path
C                list FND will be returned with the value TRUE.
C                FND will only be returned as FALSE if NODE is not
C                part of some path in the path list.
C
C     VERTEX     is a double precision array containing the coordinates
C                of the requested vertex.  If FND is FALSE VERTEX
C                will be set to the zero vector.
C
C     PREV       is the node associated with the vertex that comes
C                before NODE in the path.  If NODE is not part of
C                a legitimate path, PREV will be returned as zero.
C                If NODE is the head of a path, PREV will be set
C                to -TAIL where TAIL is the node value of the last
C                vertex in the path.
C
C     TAIL       is the node associated with the vertex that comes
C                after NODE in the path.  If NODE is not part of
C                a legitimate path, NEXT will be returned as zero.
C                If NODE is the last vertex of a path, PREV will
C                be set to -HEAD where HEAD is the node value of the
C                first vertex in the path.
C
C$ Parameters
C
C     LBPOOL     is the lower bound used in the linked list software.
C                It must have the value -5.
C
C$ Files
C
C     None.
C
C$ Exceptions
C
C     Error free.
C
C$ Particulars
C
C     This routine allows you to easily collect the coordinates
C     of all of the vertices of a path.  It is intended for situations
C     in which you wish to start at some vertex of a path and
C     sequentially return successive (or preceding nodes).
C
C     This is simply a macro (with modest error checking) for the
C     following block of code
C
C
C        FND  = .TRUE.
C
C        DIM  = PTHDIM(PATHS)
C
C        PREV = PATHS(BCKWRD,NODE)
C        NEXT = PATHS(FORWRD,NODE)
C
C        AT   = (NODE-1)*DIM + 1
C
C        CALL MOVED ( VERTXS(AT), DIM, VERTEX )
C
C
C$ Examples
C
C     Suppose you wish to remove vertices from a path that are
C     are interior points of a line segment.  The following
C     block of code illustrates how you might use this routine
C     to perform this task.
C
C     Start with the head node of a path.
C
C     DOUBLE PRECISION      VERTEX(3,3)
C     INTEGER               LEFT
C     INTEGER               MIDDLE
C     INTEGER               RIGHT
C
C
C     LEFT   = 1
C     MIDDLE = 2
C     RIGHT  = 3
C
C
C     Get the first three points.  Note that if there are not
C     3 points, FND will be returned as FALSE by the last
C     call to PTHVRT.
C
C     CALL PTHVRT ( HEAD, PATHS, VERTXS, FND, VERTEX(1,LEFT), ...
C                   TAIL, MIDPT )
C
C     CALL PTHVRT ( MIDPT, PATHS, VERTXS, FND, VERTEX(1,MIDDLE), ...
C                   FIRST, LAST )
C
C     CALL PTHVRT ( LAST,  PATHS, VERTXS,  FND, VERTEX(1,RIGHT), ...
C                   PREV,  NEXT )
C
C
C     We will take advantage of the fact that FND is true
C     only if the vertex requested has a positive node value.
C
C     DO WHILE ( FND )
C
C        Use the user supplied function INLINE that determines
C        whether or not the segment from the first to the third
C        point passes through the second point.
C
C        IF ( INLINE( VERTEX(1,LEFT),   ...
C                     VERTEX(1,MIDDLE), ...
C                     VERTEX(1,RIGHT) ) ) THEN
C
C           The segment from LEFT to RIGHT passed through the
C           MIDDLE point.  We drop it from the list.
C
C           CALL PTHRPT ( MIDPT,  PATHS, PREV, NEXT )
C
C           To avoid moving coordinates we simply swap the
C           values of MIDDLE and RIGHT
C
C           CALL SWAPI  ( MIDDLE, RIGHT )
C
C        ELSE
C
C           The points in hand are not aligned.  Cycle the
C           values of our "pointers" so that we don't have
C           to move the coordinates of the points.
C
C           TEMP   = LEFT
C           LEFT   = MIDDLE
C           MIDDLE = RIGHT
C           RIGHT  = TEMP
C
C        END IF
C
C        LAST = NEXT
C        CALL PTHVRT ( LAST,  PATHS, VERTXS,  FND, VERTEX(1,RIGHT), ...
C                      PREV,  NEXT )
C
C     END DO
C
C$ Restrictions
C
C     None.
C
C$ Author_and_Institution
C
C     J.A. Bytof      (JPL)
C     B.V. Semenov    (JPL)
C     W.L. Taber      (JPL)
C
C$ Literature_References
C
C     None.
C
C$ Version
C
C-    SPICELIB Version 2.1.0, 03-SEP-2002 (WLT)(BVS)
C
C        Replaced in-line functions with the actual required
C        computations.  The in-line functions were not being
C        interpreted properly under Linux g77
C
C-    SPICELIB Version 2.0.0, 02-FEB-1999 (JAB)
C
C        Removed some unneeded declaration and
C        assignment statements.
C
C-    SPICELIB Version 1.0.0, 03-APR-1997 (WLT)
C
C
C-&
 
C$ Index_Entries
C
C     Return the coordinates of a vertex in a path
C
C-&
C
C     Parameters
C
 
C
C     The control area of PATHS contains 3 elements.  They are:
C
C        The "size" of the pool, that is, the number
C        of nodes in the pool.
C
C        The number of free nodes in the pool.
C
C        The "free pointer," which is the column index of the first free
C        node.
C
C     Parameters defining the row and column indices of these control
C     elements are given below.
C
      INTEGER               SIZROW
      PARAMETER           ( SIZROW =   1 )
 
      INTEGER               SIZCOL
      PARAMETER           ( SIZCOL =   0 )
 
      INTEGER               FORWRD
      PARAMETER           ( FORWRD  = 1 )
 
      INTEGER               BCKWRD
      PARAMETER           ( BCKWRD  = 2 )

C
C     The rows of the path list dimension and path count are given
C     below.  The columns must be computed from values stored in
C     the array PATHS.
C

      INTEGER               DIMROW
      PARAMETER           ( DIMROW = 1 )
 
C
C     Local Variables
C
      INTEGER               AT
      LOGICAL               INRNGE
      INTEGER               ANEXT
      INTEGER               APREV
      INTEGER               DIMCOL
      INTEGER               I
      INTEGER               PTHDIM
      INTEGER               SIZE
      INTEGER               MYNODE
 
 
C
C     First determine the number of free nodes, and the size of
C     the path list.
C
      SIZE   = PATHS(SIZROW,SIZCOL)
      INRNGE = NODE .LE. SIZE .AND. NODE .GT. 0
 
C
C     Compute the column in PATHS where the dimension and number
C     of paths is stored.
C
      DIMCOL = SIZE + 1
 
      PTHDIM = PATHS(DIMROW,DIMCOL)
 
C
C     Make sure the node is in range.
C
      IF ( .NOT. INRNGE ) THEN
 
C
C        There's nothing to return, assign zero to VERTEX, PREV and NEXT
C
         DO I = 1, PTHDIM
            VERTEX(I) = 0.0D0
         END DO
 
         PREV = 0
         NEXT = 0
         FND  = .FALSE.
         RETURN
 
      END IF

C
C     The node is in range, get the previous and next nodes
C     of the path.
C
      MYNODE = NODE
 
      PREV   = PATHS(BCKWRD,MYNODE)
      NEXT   = PATHS(FORWRD,MYNODE)
C
C     We check to make sure that PREV and NEXT are legitimate
C     values.
C
      APREV = ABS(PREV)
      ANEXT = ABS(NEXT)
  
      IF (       APREV .LE. SIZE
     .     .AND. APREV .GT. 0
     .     .AND. ANEXT .LE. SIZE
     .     .AND. ANEXT .GT. 0 ) THEN
 
         AT = (MYNODE-1)*PTHDIM + 1
         CALL MOVED ( VERTXS( AT ), PTHDIM, VERTEX )
         FND = .TRUE.
 
      ELSE
C
C        One of the values PREV or NEXT was either FREE or
C        out of range.  Set them and VERTEX to zero; set FND
C        to FALSE and return.
C
         PREV = 0
         NEXT = 0
 
         DO I = 1, PTHDIM
            VERTEX(I) = 0.0D0
         END DO
 
         FND = .FALSE.
 
      END IF
 
      RETURN
      END
