C$Procedure      PTSHLF ( Points to half-space )
 
      SUBROUTINE PTSHLF ( P1, P2, P3, NORMAL )
 
C$ Abstract
C
C    Given the vertices of a non-degenerate triangle, construct
C    a vector that is orthogonal to the edge containing the
C    first two points and that points into the half-plane
C    that contains the third point and has an edge containing
C    the first two points.
C
C$ Copyright
C
C     Copyright (1997), California Institute of Technology.
C     U.S. Government sponsorship acknowledged.
C
C$ Required_Reading
C
C     None.
C
C$ Keywords
C
C     GEOMETRY
C
C$ Declarations
 
      IMPLICIT NONE
 
      DOUBLE PRECISION      P1     ( 2 )
      DOUBLE PRECISION      P2     ( 2 )
      DOUBLE PRECISION      P3     ( 2 )
      DOUBLE PRECISION      NORMAL ( 2 )
 
C$ Brief_I/O
C
C     VARIABLE  I/O  DESCRIPTION
C     --------  ---  --------------------------------------------------
C     P1         I   the first vertex of a non-degenerate triangle
C     P2         I   the second vertex of a non-degenerate triangle
C     P3         I   the third vertex of a non-degenerate triangle
C     NORMAL     O   a vector normal to the edge P1 to P2 towards P3
C
C$ Detailed_Input
C
C     P1         are the vertices of a non-degenerate triangle.
C     P2
C     P3
C
C$ Detailed_Output
C
C     NORMAL     is a vector orthogonal to the vector from P1 to P2
C                that points into the half-plane bounded by the
C                line through P1 and P2 and that contains P3.
C
C$ Parameters
C
C     None.
C
C$ Files
C
C     None.
C
C$ Exceptions
C
C     Error free.
C
C     1) If P1, P2 and P3 do not form a non-degenerate triangle
C        NORMAL is returned as the zero vector.
C
C$ Particulars
C
C     Given three points in the plane that are the vertices
C     of a non-degenerate triangle as shown below
C
C                P3
C                o
C               /.\
C              /...\
C             /.....\
C            /.......\
C           /.........\
C          o-----------o
C          P1          P2
C
C
C     this routine returns a vector NORMAL that points into
C     the half plane containing P3 and whose edge is given by
C     the line through P1 and P2
C
C
C                P3
C                o
C               /.\      ^
C              /...\     |
C             /.....\    | NORMAL
C            /.......\   |
C           /.........\  |
C          o-----------o
C          P1          P2
C
C
C
C$ Examples
C
C     Suppose you would like to determine if a point Q, in the
C     plane lies in the interior of the triangle whose
C     vertices are given by P1, P2, and P3.
C
C     Using this routine together with PINHLF you can use
C     the code loop below you can find out.
C
C
C     LOGICAL               H1
C     LOGICAL               H2
C     LOGICAL               H3
C
C     CLOSED  = .TRUE.
C     OPENED  = .NOT. CLOSED
C
C     CALL PTSHLF ( P1, P2, P3, NORML )
C     H1 = PINHLF ( Q,  P2,     NORMAL, OPENED )
C
C     CALL PTSHLF ( P2, P3, P1, NORML )
C     H2 = PINHLF ( Q,  P3,     NORMAL, OPENED )
C
C     CALL PTSHLF ( P3, P1, P2, NORML )
C     H3 = PINHLF ( Q,  P1,     NORMAL, OPENED )
C
C     INSIDE = H1 .AND. H2. .AND. H3
C
C
C$ Restrictions
C
C     None.
C
C$ Author_and_Institution
C
C     W.L. Taber      (JPL)
C
C$ Literature_References
C
C     None.
C
C$ Version
C
C-    SPICELIB Version 1.0.0, 28-FEB-1997 (WLT)
C
C
C-&
 
C$ Index_Entries
C
C     Normal to the edge of a half-plane given by three points
C
C-&
C
C     Spicelib Functions
C
      DOUBLE PRECISION      VDOTG
 
C
C     Local Variables
C
      DOUBLE PRECISION      A
      DOUBLE PRECISION      B
      DOUBLE PRECISION      V1  ( 2 )
      DOUBLE PRECISION      V2  ( 2 )
      DOUBLE PRECISION      TMP ( 2 )
 
C
C     First compute the vectors from P1 to P2 and from P1 to P3.
C     We'll call these V1 and V2.
C
      CALL VSUBG ( P2, P1, 2, V1 )
      CALL VSUBG ( P3, P1, 2, V2 )
 
C
C     V2 can be decomposed as the sum of two vectors, one
C     parallel to V1 and one orthogonal to V1.  The component
C     orthogonal to V1 points into the half plane of interest.
C
C     Call the component of V2 orthogonal V1, N.  N is given by
C
C                  <V2, V1>
C        N = V2 -  --------   V1
C                  <V1, V1>
C
C     So that
C
C       <V1,V1> N  = <V1,V1> V2 - <V2,V1> V1
C
C     This positive multiple of N can be constructed easily
C     and requires no division.  Moreover, if the triangle
C     is degenerate,  this form automatically produces the
C     zero vector.
C
C     (Also note that this method generalizes nicely in arbitrary
C     dimensions.)
C
 
      A =   VDOTG ( V1, V1, 2 )
      B = - VDOTG ( V1, V2, 2 )
 
      CALL V2LCOM ( A, V2, B, V1, TMP    )
      CALL NRMIZE ( TMP,   2,     NORMAL )
 
      RETURN
      END
