C$Procedure  RAYOCC   ( test if a ray intersects a given plate )
 
      LOGICAL FUNCTION RAYOCC ( START, RAYDIR, PLDEX, DIST )
 
C$ Abstract
C
C     Determine if a ray intersects a given plate.
C
C$ Copyright
C
C     Copyright (2000), California Institute of Technology.
C     U.S. Government sponsorship acknowledged.
C
C$ Required_Reading
C
C     None.
C
C$ Keywords
C
C     PLATE OCCLUSION
C
C$ Declarations
 
      IMPLICIT              NONE
 
 
      DOUBLE PRECISION      START   ( 3 )
      DOUBLE PRECISION      RAYDIR  ( 3 )
 
      INTEGER               PLDEX
 
      DOUBLE PRECISION      DIST
 
 
C$ Brief_I/O
C
C     VARIABLE  I/O  DESCRIPTION
C     --------  ---  --------------------------------------------------
C     START      I   Coordinates of ray's start point.
C     RAYDIR     I   Ray direction vector.
C     PLDEX      I   Plate ID.
C     RAYOCC     O   .TRUE. if ray intersects plate PLDEX.
C     DIST       O   Scale factor of RAYDIR to intersection point.
C
C$ Detailed_Input
C
C     START      Coordinates of the ray's starting point.
C
C     RAYDIR     Ray's vector from the starting point.
C
C     PLDEX      Plate ID.
C
C$ Detailed_Output
C
C     RAYOCC     A value of .TRUE. is returned if ray hits plate PLDEX.
C
C     DIST       Scale factor of RAYDIR to intersection point, if
C                the ray intersects the plate.
C
C$ Parameters
C
C     None.
C
C$ Files
C
C     None.
C
C$ Exceptions
C
C     1) If information required by this routine has not been
C        loaded prior to calling this routine, the omission will
C        be diagnosed by a routine in the call tree of this routine.
C
C$ Particulars
C
C     Given a plate index, a ray and its starting point, determines
C     if the ray intersects the plate, and if so, the scaling
C     factor applied to RAYDIR to obtain the intersection point.
C
C$ Examples
C
C     Later.
C
C$ Restrictions
C
C     None.
C
C$ Author_and_Institution
C
C     J.A. Bytof      (JPL)
C
C$ Literature_References
C
C     None.
C
C$ Version
C
C-    SPICELIB Version 2.0.0, 17-JAN-2000 (JAB)
C
C        Simplified code, improved comments.
C
C-    SPICELIB Version 1.0.0, 04-FEB-1999 (JAB)
C
C
C-&
 
C$ Index_Entries
C
C     test for ray intersection with a plate
C
C-&
 
 
C
C     Local variables.
C
 
      DOUBLE PRECISION      E1      ( 3 )
      DOUBLE PRECISION      E2      ( 3 )
      DOUBLE PRECISION      E3      ( 3 )
      DOUBLE PRECISION      POINTS  ( 3, 3 )
 
 
      LOGICAL               FOUND
      LOGICAL               RETURN
 
C
C     Initialize function return value. If an intersection
C     is found, RAYOCC becomes .TRUE.
C
 
      RAYOCC = .FALSE.
 
C
C     Standard SPICELIB error handling.
C
 
      IF ( RETURN () ) THEN
         RETURN
      END IF
 
      CALL CHKIN  ( 'RAYOCC' )
 
C
C     Obtain vertex coordinates of the target plate and generate
C     vectors from the ray's starting point to each plate vertex.
C
 
      CALL NTHPLV ( PLDEX, POINTS )
 
      CALL VSUB   ( POINTS(1,1), START, E1 )
      CALL VSUB   ( POINTS(1,2), START, E2 )
      CALL VSUB   ( POINTS(1,3), START, E3 )
 
C
C     Determine if RAYDIR lies inside the solid tetrahedral
C     angle formed by the vectors E1, E2, and E3.  DIST is the scalar
C     multiple of RAYDIR to intersection with the plane of the
C     plate.
C
 
      CALL INSANG ( RAYDIR, E1, E2, E3, FOUND, DIST )
 
      IF ( FOUND ) THEN
 
         RAYOCC = .TRUE.
 
      END IF
 
C
C     Standard SPICE error handling.
C
 
      CALL CHKOUT ( 'RAYOCC' )
 
      RETURN
      END
