C$Procedure VOXDIM ( return voxel grid information )

      SUBROUTINE VOXDIM ( NVOX, VOXSIZ, VOXORI )

C$ Abstract
C
C     Returns voxel grid information defining the model to 
C     voxel space mapping.
C
C$ Copyright
C
C     Copyright (1999), California Institute of Technology.
C     U.S. Government sponsorship acknowledged.
C
C$ Required_Reading
C
C     None.
C
C$ Keywords
C
C     VOXEL
C
C$ Declarations
 
      IMPLICIT              NONE
 
      INTEGER               NVOX    ( 3 )
 
      DOUBLE PRECISION      VOXSIZ
      DOUBLE PRECISION      VOXORI  ( 3 )
 
C$ Brief_I/O
C
C     VARIABLE  I/O  DESCRIPTION
C     --------  ---  --------------------------------------------------
C     NVOX       O   Dimension of voxel grid in voxel units.
C     VOXSIZ     O   Size of voxel in model units.
C     VOXORI     O   Coordinates of voxel grid origin in model units.
C
C$ Detailed_Input
C
C     None.
C
C$ Detailed_Output
C
C     NVOX       An integer 3-vector defining the span of
C                the cubic voxels for each coordinate axis. 
C
C                   NVOX(1) = NINT( X         - X         )
C                                    Vox_max     Vmin_min
C
C                   NVOX(2) = NINT( Y         - Y         )
C                                    Vox_max     Vmin_min
C
C                   NVOX(3) = NINT( Z         - Z         )
C                                    Vox_max     Vmin_min
C
C                Where:
C
C                      X          Y          Z
C                       Vox_max    Vox_max    Vox_max
C
C                represent the maximum values of all voxel
C                space coordinates along each axis, and:
C
C                      X          Y          Z
C                       Vox_min    Vox_min    Vox_min
C
C                represent the minimum values of all voxel
C                space coordinates along each axis. 
C
C                NINT being the FORTRAN "nearest integer" function.
C
C     VOXSIZ     The the scalar length in model units of a voxel's
C                side.
C
C     VOXORI     An double precision 3-vector defining the 
C                coordinates in model units of voxel grid origin.
C                The coordinates relate to the voxel grid by the 
C                mapping:
C
C                   VOXORI(1) =  X         / VOXSIZ
C                                 Vmin_min
C
C                   VOXORI(2) = Y         / VOXSIZ
C                                Vmin_min
C
C                   VOXORI(3) = Z         / VOXSIZ
C                                Vmin_min
C
C$ Parameters
C
C     None.
C
C$ Files
C
C     None.
C
C$ Exceptions
C
C     Error free.
C
C$ Particulars
C
C     This routine serves as a wrapper to a VOXSUM call,
C     returning to the caller only the variables defining
C     the mapping between model space and voxel space, 
C     NVOX, VOXSIZ, and VOXORI. The VOXSUM call 
C     executues with a value of 1 for the BODEX values.
C
C$ Examples
C
C     Later.
C
C$ Restrictions
C
C     None.
C
C$ Author_and_Institution
C
C     J.A. Bytof      (JPL)
C
C$ Literature_References
C
C     PLATELIB.REQ
C
C$ Version
C
C-    SPICELIB Version 1.0.1, 05-SEP-2004 (EDW)
C
C         Extended header documentation.
C
C-    SPICELIB Version 1.0.0, 04-FEB-1999 (JAB)
C
C-&
 
C$ Index_Entries
C
C     return voxel grid information
C
C-&
 
      LOGICAL               RETURN
 
      INTEGER               NVXTOT
      INTEGER               BODEX
      INTEGER               NVX     ( 1 )
 
 
      SAVE  BODEX
      DATA  BODEX  / 1 /
 
C
C     Standard SPICELIB error handling.
C
      IF ( RETURN () ) THEN
         RETURN
      END IF
 
      CALL CHKIN  ( 'VOXDIM' )
 
 
C
C     Retrieve the voxel grid geometry for body BODEX. For
C     current use, the body index, BODEX, has value 1 
C     (the plate system can process only a single plate at 
C     a time.
C
      CALL VOXSUM ( BODEX, NVOX, VOXSIZ, VOXORI, NVXTOT, NVX )

C
C     Standard SPICE error handling.
C
      CALL CHKOUT ( 'VOXDIM' )
 
 
      RETURN
      END
