C$Procedure      XCOLC ( get a character array from an E-kernel )
 
      SUBROUTINE XCOLC ( EKFILE, HANDLE, SEGNO, CNAME, NR,
     .                   VALS,   NLFLGS, NROWS             )
 
C$ Abstract
C
C     Read a given segment and column in an E-kernel and
C     return an array of character string column entry values.
C
C$ Copyright
C
C     Copyright (1997), California Institute of Technology.
C     U.S. Government sponsorship acknowledged.
C
C$ Required_Reading
C
C     None.
C
C$ Keywords
C
C     E-KERNEL
C
C$ Declarations
 
      IMPLICIT NONE
 
      CHARACTER*(*)         EKFILE
      INTEGER               HANDLE
      INTEGER               SEGNO
      CHARACTER*(*)         CNAME
      INTEGER               NR
      CHARACTER*(*)         VALS     ( * )
      LOGICAL               NLFLGS   ( * )
      INTEGER               NROWS
 
 
C$ Brief_I/O
C
C     VARIABLE  I/O  DESCRIPTION
C     --------  ---  --------------------------------------------------
C     EKFILE     I   Name of E-kernel.
C     HANDLE     I   E-kernel handle.
C     SEGNO      I   Requested segment number.
C     CNAME      I   Name of column.
C     NR         I   Number of values expected in output array.
C     VALS       O   Array of character string in column.
C     NLFLGS     O   Null array.
C     NROWS      O   Number of elements in output array.
C
C
C$ Detailed_Input
C
C     EKFILE     Name of E-kernel.
C     HANDLE     E-kernel handle.
C     SEGNO      Requested segment number.
C     CNAME      Name of column.
C     NR         Number of values expected in output array.
C
C$ Detailed_Output
C
C     VALS       Array of character string values in column.
C     NLFLGS     Null array.
C     NROWS      Number of elements in output array.
C
C
C$ Parameters
C
C     CDSCSZ     Size of column descriptor.
C     NRIDX      Index of segment's record count.
C     RTIDX      Index of the root page number of the record tree.
C     SDSCSZ     Size of segment descriptor.
C
C$ Files
C
C     See: $Restrictions.
C
C$ Exceptions
C
C     1) If the number of values in the array does not match
C        the number found, the error 'SPICE(ROWCOUNTMISMATCH)'
C        is signaled.
C
C$ Particulars
C
C     Later.
C
C$ Examples
C
C     Later.
C
C$ Restrictions
C
C     1) An E-kernel should have been opened with EKOPR, or else a
C        routine in the calling tree for this routine will signal an
C        error.
C
C$ Author_and_Institution
C
C     J.A. Bytof      (JPL)
C     N.J. Bachman    (JPL)
C
C$ Literature_References
C
C     None.
C
C$ Version
C
C-    SPICELIB Version 2.0.0, 12-JUL-2004 (NJB)
C
C        Updated to support class 9 columns.
C
C-    SPICELIB Version 1.0.0, 20-AUG-1997 (JAB)
C
C
C-&
 
C$ Index_Entries
C
C     return a character string array from an E-kernel
C
C-&
 
 
C
C     SPICELIB functions.
C
 
      LOGICAL               RETURN
 
C
C     SPICELIB include files.
C
 
      INCLUDE 'eksegdsc.inc'
      INCLUDE 'ekcoldsc.inc'
 
C
C     Local Declarations.
C
      INTEGER               CCLASS
      INTEGER               COLDSC ( CDSCSZ )
      INTEGER               I
      INTEGER               PTR
      INTEGER               CVLEN
      INTEGER               SEGDSC ( SDSCSZ )
 
C
C     Standard SPICE error handling.
C
 
      IF ( RETURN() ) THEN
         RETURN
      END IF
 
      CALL CHKIN ( 'XCOLC' )
 
C
C     Obtain the number of entries in the column from the
C     segment descriptor.
C
 
      CALL ZZEKSDSC ( HANDLE, SEGNO,  SEGDSC         )
      CALL ZZEKCDSC ( HANDLE, SEGDSC, CNAME,  COLDSC )
 
      NROWS  =  SEGDSC (NRIDX)
 
C
C     Check and see of the number of entries expected
C     is the number obtained; if not, signal an error.
C
 
      IF( NR .NE. NROWS ) THEN
 
         CALL SETMSG ( 'E-kernel file #, column name # '    )
         CALL SETMSG ( 'has # rows, but # were requested.'  )
         CALL ERRCH  ( '#', EKFILE                          )
         CALL ERRCH  ( '#', CNAME                           )
         CALL ERRINT ( '#', NROWS                           )
         CALL ERRINT ( '#', NR                              )
         CALL SIGERR ( 'SPICE(ROWCOUNTMISMATCH)'            )
 
         CALL CHKOUT ( 'XCOLC' )
         RETURN
 
      END IF
 
C
C     Obtain the entry values and store them in the output array. 
C
C     The method of reading the data depends on the column class.
C
      CCLASS = COLDSC(CLSIDX)
 
 
      IF ( CCLASS .EQ. 3 ) THEN
C
C        This is the scalar character column class for type 1 EK
C        segments.
C
         DO I = 1, NROWS

            CALL ZZEKTRDP ( HANDLE,  SEGDSC(RTIDX), I,         PTR  )
            CALL ZZEKRD03 ( HANDLE,  SEGDSC,        COLDSC,    PTR,
     .                      CVLEN,   VALS(I),       NLFLGS(1)       )
         END DO


      ELSE IF ( CCLASS .EQ. 9 ) THEN 
C
C        This is the scalar character column class for type 2 EK
C        segments.
C
         DO I = 1, NROWS

            CALL ZZEKRD09 ( HANDLE,  SEGDSC,   COLDSC, 
     .                      I,       CVLEN,    VALS(I),  NLFLGS(1) )
         END DO

      ELSE

         CALL SETMSG ( 'Unexpected column class # seen.' )
         CALL ERRINT ( '#',  CCLASS                      )
         CALL SIGERR ( 'SPICE(NOTSUPPORTED)'             )
         CALL CHKOUT ( 'XCOLC'                           )
         RETURN

      END IF
 
      CALL CHKOUT ( 'XCOLC' )
      RETURN
      END
