C$Procedure  XDDA  ( list voxels intersected by a ray )
 
      SUBROUTINE XDDA ( OBS, RAY, NVOX, NVXLST, NVX, VXLST2 )
 
C$ Abstract
C
C     Given a ray and a voxel grid, return a list of unique 
C     voxels the ray intersects.
C
C$ Copyright
C
C     Copyright (1999), California Institute of Technology.
C     U.S. Government sponsorship acknowledged.
C
C$ Required_Reading
C
C     None.
C
C$ Keywords
C
C     PLATE VOXEL
C
C$ Declarations
 
      IMPLICIT              NONE

      DOUBLE PRECISION      OBS      ( 3 )
      DOUBLE PRECISION      RAY      ( 3 )
      INTEGER               NVOX     ( 3 )
      INTEGER               NVXLST
      INTEGER               NVX
      INTEGER               VXLST2   ( * )

C$ Brief_I/O
C
C     VARIABLE  I/O  DESCRIPTION
C     --------  ---  --------------------------------------------------
C     OBS        I   Voxel grid coordinates of ray's starting point.
C     RAY        I   Direction vector of ray.
C     NVOX       I   Dimensions of grid in voxel units.
C     NVXLST     I   Order if VXLST2.
C     NVX        O   Number of voxels in the VXLST2 list.
C     VXLST2     O   List of voxels intersected by ray.
C
C$ Detailed_Input
C
C     OBS        Voxel grid coordinates of ray's starting point.
C
C     RAY        Direction vector of ray from OBS, note ray direction
C                is invarient with regards to the model to voxel
C                space mapping.
C
C     NVOX       The integer 3-vector containing the dimensions of
C                the voxel grid in voxel units, [Nx, Ny, Nz].
C
C     NVXLST     The order of VXLST2 as defined in the calling routine.
C                The counter NVX will not exceed this value.
C
C$ Detailed_Output
C
C     NVX        Number of voxel IDs contained by the VXLST2 list.
C
C     VXLST2     List of voxel IDs intersected by ray. The list returns
C                sorted in ascending order and without duplicates.
C
C$ Parameters
C
C     None.
C
C$ Files
C
C     None.
C
C$ Exceptions
C
C     1) The SPICE error SPICE(RAYISZEROVECTOR) signals when
C        the input RAY has all zero components.
C
C     2) The SPICE error SPICE(NOTINGRID) signals when
C        the input OBS describes a position not in or on the
C        voxel grid.
C
C     3) The SPICE error SPICE(ARRAYTOOSMALL) signals when
C        the value of the NVX counter (number of intersected
C        voxels) exceeds the order of the VXLST2 input vector.
C
C$ Particulars
C
C     Later.
C
C$ Examples
C
C     Later.
C
C$ Restrictions
C
C     None.
C
C$ Author_and_Institution
C
C     J.A. Bytof      (JPL)
C
C$ Literature_References
C
C     None.
C
C$ Version
C
C-    SPICELIB Version 1.1.0, 19-OCT-2004 (EDW)
C
C        Added logic to remove duplicate voxel IDs from
C        the return list. Extended programing comments.
C
C-    SPICELIB Version 1.0.1, 26-AUG-2002 (BVS)
C
C        Replaced WRITE with normal error reporting calls.
C
C-    SPICELIB Version 1.0.0, 03-FEB-1999 (JAB)
C
C-&
 
C$ Index_Entries
C
C     list voxels intersected by a ray
C
C-&
 
      INTEGER               I
      INTEGER               IX
      INTEGER               IY
      INTEGER               IZ
      INTEGER               VOX2ID
      INTEGER               STEP     ( 3 )
      INTEGER               VXLST    ( 3 )
 
      LOGICAL               INGRD
      LOGICAL               VZERO
      LOGICAL               RETURN
 
      DOUBLE PRECISION      X0
      DOUBLE PRECISION      Y0
      DOUBLE PRECISION      Z0
      DOUBLE PRECISION      SXY
      DOUBLE PRECISION      SXZ
      DOUBLE PRECISION      YERR
      DOUBLE PRECISION      ZERR
 
C
C     Standard SPICELIB error handling.
C
 
      IF ( RETURN () ) THEN
         RETURN
      END IF
 
      CALL CHKIN  ( 'XDDA' )
 
C
C     Check if ray direction vector is a zero vector.
C
      IF ( VZERO( RAY ) ) THEN
         CALL SETMSG ( 'Ray is the zero vector.'      )
         CALL SIGERR ( 'SPICE(RAYISZEROVECTOR)'       )
         CALL CHKOUT ( 'XDDA'                         )
         RETURN
      END IF
 
C
C     Which voxel contains the observer? Truncate the observer
C     coordinates to integers. Add 1 to each coord to compensate
C     for 1 based counting.
C
      IX = 1 + INT( OBS(1) )
      IY = 1 + INT( OBS(2) )
      IZ = 1 + INT( OBS(3) )
 
      CALL IPACK3 ( IX, IY, IZ, VXLST )

C
C     Check the observer coordinates exist as part
C     of the voxel grid.
C
      IF ( .NOT. INGRD(NVOX, VXLST ) ) THEN
         CALL SETMSG ( 'Ray origin not in voxel grid.' )
         CALL SIGERR ( 'SPICE(NOTINGRID)'              )
         CALL CHKOUT ( 'XDDA'                          )
         RETURN
      END IF

      VXLST2(1) = VOX2ID ( VXLST, NVOX )


C
C     Initialize the counter for number of voxels the ray 
C     intercepts. The test above convermed the observer 
C     position lies in a voxel belonging to the grid.
C
      NVX = 1

C
C     Calculate the relative location of starting point
C     within the voxel. The coordinates of a voxel's corners 
C     are integer values with each voxel side length 1 
C     (in voxel coords).
C
C     The DMOD calls equate to the function:
C
C          OBS(I) - DBLE( INT( OBS(I) ) )
C
C     So the X0, Y0, Z0 variables have the fractional
C     component values of the corresponding OBS(I).
C
      X0 = DMOD( OBS(1), 1.D0 )
      Y0 = DMOD( OBS(2), 1.D0 )
      Z0 = DMOD( OBS(3), 1.D0 )
 
C
C     Determine the error term initial values and increments.
C
C     Different quadrant directions result in slightly
C     different initial conditions; the only effect of
C     quadrant in the main loop is in the step direction.
C
      YERR = 9.D9
      ZERR = 9.D9
 
C
C     What does this do?
C
      IF ( RAY(2) .NE. 0.D0 ) THEN
      
         SXY = DABS(RAY(1)/RAY(2))
      
         
         IF ( RAY(1) .GT. 0.D0 ) THEN

            IF( RAY(2) .GT. 0.D0 ) THEN
               YERR = SXY*(1.D0 - Y0) + X0 - 1.D0
            ELSE
               YERR = SXY*Y0 + X0 - 1.D0
            END IF

         ELSE

            IF( RAY(2) .GT. 0.D0 ) THEN
               YERR = SXY*(1.D0 - Y0) - X0
            ELSE
               YERR = SXY*Y0 - X0
            END IF

         END IF

      ELSE
      
         SXY = 0.D0
      
      END IF
 

C
C     What does this do?
C 
      IF ( RAY(3) .NE. 0.D0 ) THEN
      
         SXZ = DABS(RAY(1)/RAY(3))
         
         IF ( RAY(1) .GT. 0.D0 ) THEN

            IF( RAY(3) .GT. 0.D0 ) THEN
               ZERR = SXZ*(1.D0 - Z0) + X0 - 1.D0
            ELSE
               ZERR = SXZ*Z0 + X0 - 1.D0
            END IF

         ELSE

            IF( RAY(3) .GT. 0.D0 ) THEN
               ZERR = SXZ*(1.D0 - Z0) - X0
            ELSE
               ZERR = SXZ*Z0 - X0
            END IF

         END IF

      ELSE
      
         SXZ = 0.D0
      
      END IF


C
C     A voxel, in voxel space, has length one for each size,
C     a step has magnitude one or zero, zero only when the 
C     magnitude of a component of RAY has value zero.
C
      DO I = 1, 3
         
         IF ( RAY(I) .GT. 0.D0 ) THEN
         
C
C           Positive component direction, positive step.
C
            STEP(I) =  1

         ELSE IF ( RAY(I) .LT. 0.D0 ) THEN

C
C           Negative component direction, negative step.
C
            STEP(I) = -1
            
         ELSE IF ( RAY(I) .EQ. 0.D0 ) THEN

C
C           No component in this direction, no step.
C
            STEP(I) =  0

         END IF
      
      END DO

 
C
C     Follow the ray until it exits the voxel grid.
C
      DO WHILE ( INGRD( NVOX, VXLST ) )
 
         IF ( (YERR .LT. 0.D0) .OR. (ZERR .LT. 0.D0) ) THEN
 
C
C           Ray has crossed over into the next voxel in Y or Z.
C
            IF ( YERR .LT. ZERR ) THEN
 
               IY   = IY + STEP(2)
               YERR = YERR + SXY
               NVX  = NVX + 1
 
            ELSE
 
               IZ   = IZ + STEP(3)
               ZERR = ZERR + SXZ
               NVX  = NVX + 1
  
            END IF
 
         ELSE
 
C
C           No change in Y or Z, step in X.
C
            IX   = IX + STEP(1)
            NVX  = NVX + 1
            YERR = YERR - 1.D0
            ZERR = ZERR - 1.D0
 
         END IF

C
C        Check we have room in VXLST2.
C
         IF( NVX .GT. NVXLST ) THEN
 
            CALL SETMSG ( 'Index larger than array. '//
     .                    'Index = #1. Array size = #2.' )
            CALL ERRINT ( '#1', NVX                       )
            CALL ERRINT ( '#2', NVXLST                    )
            CALL SIGERR ( 'SPICE(ARRAYTOOSMALL)'         )
 
            CALL CHKOUT ( 'XDDA' )
            RETURN
            
         END IF

C
C        Pack the voxel indices into VXLST using
C        the values calculated in this loop pass.
C
         CALL IPACK3 ( IX, IY, IZ, VXLST  )

C
C        Store the voxel ID corresponding to the 
C        VXLST voxel indices.
C
         VXLST2(NVX) = VOX2ID ( VXLST, NVOX )

      END DO
 
C
C     Subtract one off the voxel count since the final voxel 
C     exists outside the grid.
C
      NVX = NVX - 1

C
C     Remove duplicates from the list. This call also sorts 
C     the list VXLST2 in ascending order, but that has no effect
C     on other platelib functions.
C
      CALL RMDUPI ( NVX, VXLST2 )

C
C     Standard SPICE error handling.
C
      CALL CHKOUT ( 'XDDA' )
 
 
      RETURN
      END
