C$Procedure      PV2VP ( plate->vertex list into vertex->plate list )
 
      SUBROUTINE PV2VP ( NPLT,    VLIST,
     .                   VPDEX,   PLIST   )
 
C$ Abstract
C
C     From a list of vertices belonging to a plate,
C     produce the reverse: a list of plates belonging to
C     a vertex.
C
C$ Copyright
C
C     Copyright (1997), California Institute of Technology.
C     U.S. Government sponsorship acknowledged.
C
C$ Required_Reading
C
C     None.
C
C$ Keywords
C
C     PLATE
C
C$ Declarations
 
      IMPLICIT  NONE
 
      INTEGER               NPLT
      INTEGER               VLIST      ( 3, * )
      INTEGER               VPDEX      ( 2, * )
      INTEGER               PLIST      ( * )
 
 
C$ Brief_I/O
C
C     VARIABLE  I/O  DESCRIPTION
C     --------  ---  --------------------------------------------------
C     NPLT       I   Number of plates.
C     VLIST      I   List of vertices belonging to each plate.
C     VPDEX      O   Vertex begin and end locations.
C     PLIST      O   Plate list.
C
C$ Detailed_Input
C
C     NPLT       Number of plates.
C     VLIST      List of vertices belonging to each plate.
C
C$ Detailed_Output
C
C     VPDEX      Array of locations in PLIST where a
C                new vertex begins and ends.
C     PLIST      Array of plates, in vertex order.
C
C$ Parameters
C
C     None.
C
C$ Files
C
C     See: $Restrictions.
C
C$ Exceptions
C
C     Later.
C
C$ Particulars
C
C     Later.
C
C$ Examples
C
C     Later.
C
C$ Restrictions
C
C     1) A plate model must be loaded by INPLAT and currently
C        resident in memory before this routine is called.
C
C$ Author_and_Institution
C
C     J.A. Bytof      (JPL)
C
C$ Literature_References
C
C     None.
C
C$ Version
C
C-    SPICELIB Version 1.0.0, 20-AUG-1997 (JAB)
C
C
C-&
 
C$ Index_Entries
C
C     return which plates belongs to a vertex
C
C-&
 
      INCLUDE   'pltmax.inc'
 
C
C     SPICELIB functions.
C
 
      LOGICAL               RETURN
 
C
C     Local Declarations.
C
 
      INTEGER               I
      INTEGER               J
      INTEGER               NV
      INTEGER               NPV
      INTEGER               CURV
      INTEGER               VFIRST
      INTEGER               WORK       ( MAXNPV )
      INTEGER               VWORK      ( MAXNPV )
 
 
      IF ( RETURN() ) THEN
         RETURN
      END IF
 
      CALL CHKIN ( 'PV2VP' )
 
C
C
C
 
      NPV = 0
 
      DO I = 1, NPLT
         DO J = 1, 3
            NPV            = NPV + 1
            VWORK ( NPV )  = VLIST(J,I)
            PLIST ( NPV )  = I
         END DO
      END DO
 
 
C
C     Reorder VWORK in ascending order of vertex number;
C     save the order vector in WORK.
C
 
      CALL ORDERI ( VWORK, NPV, WORK  )
 
C
C     Perform the reordering of the vertex array (VWORK) and the
C     associated plate list (VP) using the order vector (WORK)
C     which sorts the vertex array into ascending numerical order.
C
 
      CALL REORDI ( WORK,  NPV, VWORK )
      CALL REORDI ( WORK,  NPV, PLIST )
 
 
C
C     Create the index vector (VPDEX) that
C     indicates when the vertex changes.  That
C     way we can later specify a particular vertex
C     and lookup in the array of plates which ones
C     contain that particular vertex.
C
C     CURV holds the current vertex index.
C
 
      CURV        = VWORK(1)
      VFIRST      = 1
      NV          = 0
 
 
      DO I = 1, NPV
 
         IF ( VWORK(I) .GT. CURV ) THEN
 
C
C     If there is a vertex missing from the list, abort.
C
 
            IF ( VWORK(I) .NE. (CURV+1) ) THEN
 
               CALL SETMSG ( 'Vertex # expected, # found.  ' )
               CALL ERRINT ( '#', CURV+1                     )
               CALL ERRINT ( '#', VWORK(I)                   )
               CALL SIGERR ( 'SPICE(MISSINGVERTEX)'          )
 
               CALL CHKOUT ( 'PV2VP' )
               RETURN
 
            END IF
 
C
C     If the vertex read in VWORK(I) has changed, add
C     entries to the vertex start and end index.
C
 
            NV           = NV + 1
 
            VPDEX(1,NV)  = VFIRST
            VPDEX(2,NV)  = I - 1
            CURV         = VWORK(I)
 
            VFIRST = I
 
         END IF
 
      END DO
 
      NV = NV + 1
      VPDEX(1,NV) = VFIRST
      VPDEX(2,NV) = NPV
 
      CALL CHKOUT ( 'PV2VP' )
 
 
      RETURN
 
      END
